#ifndef MATHTOOLKIT_GENERIC_ARITHMETICOPERATIONS_H
#define MATHTOOLKIT_GENERIC_ARITHMETICOPERATIONS_H

#include <numeric>

#include "NumericalVector.h"
#include "NumericalMatrix.h"


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Unary operators

namespace MathToolkit
{
	namespace Generic
	{
		template <typename T>
		inline NumericalVector<T> operator+(NumericalVector<T> nva, const NumericalVector<T>& nvb)
		{
			nva += nvb;
			return nva;
		}

		template <typename T>
		inline NumericalMatrix<T> operator+(NumericalMatrix<T> nma, const NumericalMatrix<T>& nmb)
		{
			nma += nmb;
			return nma;
		}

		template <typename T>
		inline NumericalVector<T> operator-(NumericalVector<T> nva, const NumericalVector<T>& nvb)
		{
			nva -= nvb;
			return nva;
		}

		template <typename T>
		inline NumericalMatrix<T> operator-(NumericalMatrix<T> nma, const NumericalMatrix<T>& nmb)
		{
			nma -= nmb;
			return nma;
		}

		template <typename T>
		inline NumericalVector<T> operator*(const double scalingMultiplier, NumericalVector<T> nv)
		{
			nv *= scalingMultiplier;
			return nv;
		}

		template <typename T>
		inline NumericalMatrix<T> operator*(const double scalingMultiplier, NumericalMatrix<T> nm)
		{
			nm *= scalingMultiplier;
			return nm;
		}

		template <typename T>
		inline NumericalVector<T> operator*(const NumericalMatrix<T>& nm, NumericalVector<T> nv)
		{
			nv.transform(nm);
			return nv;
		}

		template <typename T>
		inline NumericalMatrix<T> operator*(NumericalMatrix<T> nma, const NumericalMatrix<T>& nmb)
		{
			nma *= nmb;
			return nma;
		}
	}
}

// Unary operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Arithmetic methods

namespace MathToolkit
{
	namespace Generic
	{
		template <typename T>
		inline NumericalMatrix<T> multiply(const double scalingMultiplier, NumericalMatrix<T> nma, const NumericalMatrix<T>& nmb)
		{
			nma.multiply(scalingMultiplier, nmb);
			return nma;
		}

		template <typename T>
		inline double getInnerProduct(const NumericalVector<T>& nva, const NumericalVector<T>& nvb)
		{
			return std::inner_product(nva.begin(), nva.end(), nvb.begin(), 0.0);
		}
	}
}

// Arithmetic methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !MATHTOOLKIT_GENERIC_ARITHMETICOPERATIONS_H
