#ifndef CHEMTOOLKIT_GENERIC_ATOMBASE_H
#define CHEMTOOLKIT_GENERIC_ATOMBASE_H

#include <string>
#include <cmath>

#include "LinearAlgebra.h"


namespace ChemToolkit
{
	namespace Generic
	{
		class AtomBase
		{
		public:
			using size_type = std::size_t;
			using NumericalVector = MathToolkit::Generic::NumericalVector<double>;
			using NumericalMatrix = MathToolkit::Generic::NumericalMatrix<double>;
			using AffineMapping = MathToolkit::Generic::AffineMapping<double>;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructors, and operators

		public:
			explicit AtomBase(const std::string& elementSymbol);
			AtomBase(const std::string& elementSymbol, const NumericalVector& coordinate);
			virtual ~AtomBase() = default;

			AtomBase(const AtomBase&) = default;
			AtomBase(AtomBase&&) noexcept = default;
			AtomBase& operator=(const AtomBase&) = default;
			AtomBase& operator=(AtomBase&&) noexcept = default;

		// Constructors, destructors, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Access methods

			std::string elementSymbol() const;
			
			const NumericalVector& coordinate() const;
			NumericalVector& coordinate();
			const NumericalVector& appliedForce() const;
			NumericalVector& appliedForce();

		// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			virtual void move();
			virtual void move(const double maxDisplacement);

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			std::string m_elementSymbol;
			NumericalVector m_coordinate;
			NumericalVector m_appliedForce;


		private:
			AtomBase() = delete;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline std::string ChemToolkit::Generic::AtomBase::elementSymbol() const
{
	return m_elementSymbol;
}

inline const MathToolkit::Generic::NumericalVector<double>& ChemToolkit::Generic::AtomBase::coordinate() const
{
	return m_coordinate;
}

inline MathToolkit::Generic::NumericalVector<double>& ChemToolkit::Generic::AtomBase::coordinate()
{
	return m_coordinate;
}

inline const MathToolkit::Generic::NumericalVector<double>& ChemToolkit::Generic::AtomBase::appliedForce() const
{
	return m_appliedForce;
}

inline MathToolkit::Generic::NumericalVector<double>& ChemToolkit::Generic::AtomBase::appliedForce()
{
	return m_appliedForce;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline void ChemToolkit::Generic::AtomBase::move()
{
	m_coordinate += m_appliedForce;
	m_appliedForce = 0.0;
}

inline void ChemToolkit::Generic::AtomBase::move(const double maxDisplacement)
{
	double displacementSquare = m_appliedForce.normSquare();
	{
		if ((maxDisplacement * maxDisplacement) < displacementSquare)
		{
			double displacementSize = std::sqrt(displacementSquare);
			m_coordinate.add((maxDisplacement / displacementSize), m_appliedForce);
		}

		else
			m_coordinate += m_appliedForce;
	}

	m_appliedForce = 0.0;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CHEMTOOLKIT_GENERIC_ATOMBASE_H
