#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_ATOMCATEGORY_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_ATOMCATEGORY_H

#include <stdexcept>
#include <string>

#include "ReportException.h"


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace Features
		{
			class AtomCategory
			{
				using size_type = std::size_t;

				enum class Item
				{
					metal,
					oxide_ion,
					anion,
					chemical_element
				};

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				AtomCategory();
				explicit AtomCategory(const std::string& categorySymbol);
				virtual ~AtomCategory() = default;

				AtomCategory(const AtomCategory&) = default;
				AtomCategory(AtomCategory&&) noexcept = default;
				AtomCategory& operator=(const AtomCategory&) = default;
				AtomCategory& operator=(AtomCategory&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				std::string symbol() const;

				bool isMetal() const;
				bool isCation() const;

				bool isOxideIon() const;
				bool isAnion() const;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Utility

				static AtomCategory toAtomCategory(const std::string& elementSymbol);

			// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				Item m_item;


				friend bool operator==(const AtomCategory&, const AtomCategory&);
				friend bool operator!=(const AtomCategory&, const AtomCategory&);
			};


			inline bool operator<(const AtomCategory& aca, const AtomCategory& acb)
			{
				if (aca.isCation())
				{
					if (acb.isAnion())
						return true;
					else
						return false;
				}

				else
					return false;
			}

			inline bool operator>(const AtomCategory& aca, const AtomCategory& acb)
			{
				return (acb < aca);
			}

			inline bool operator==(const AtomCategory& aca, const AtomCategory& acb)
			{
				return (aca.m_item == acb.m_item);
			}

			inline bool operator!=(const AtomCategory& aca, const AtomCategory& acb)
			{
				return (aca.m_item != acb.m_item);
			}
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::string MathematicalCrystalChemistry::StructureModel::Features::AtomCategory::symbol() const
{
	if (m_item == Item::metal)
		return std::string{ "M" };
	else if (m_item == Item::oxide_ion)
		return std::string{ "O" };
	else if (m_item == Item::anion)
		return std::string{ "A" };

	else
		return std::string{};
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::AtomCategory::isMetal() const
{
	if (m_item == Item::metal)
		return true;
	else
		return false;
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::AtomCategory::isCation() const
{
	if (m_item == Item::metal)
		return true;
	else
		return false;
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::AtomCategory::isOxideIon() const
{
	if (m_item == Item::oxide_ion)
		return true;
	else
		return false;
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::AtomCategory::isAnion() const
{
	if (m_item == Item::oxide_ion || m_item == Item::anion)
		return true;
	else
		return false;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_ATOMCATEGORY_H
