#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_ATOMCATEGORYCOMPOSITION_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_ATOMCATEGORYCOMPOSITION_H

#include <string>
#include <map>


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace Features
		{
			class AtomCategoryComposition
			{
				using size_type = std::size_t;
				using iterator = std::map<std::string, size_type>::iterator;
				using const_iterator = std::map<std::string, size_type>::const_iterator;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				AtomCategoryComposition() = default;
				virtual ~AtomCategoryComposition() = default;

				AtomCategoryComposition(const AtomCategoryComposition&) = default;
				AtomCategoryComposition(AtomCategoryComposition&&) noexcept = default;
				AtomCategoryComposition& operator=(const AtomCategoryComposition&) = default;
				AtomCategoryComposition& operator=(AtomCategoryComposition&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
	// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				iterator begin();
				const_iterator begin() const;
				iterator end();
				const_iterator end() const;

				size_type countAtomCategories() const;
				bool contains(const std::string& atomCategorySymbol) const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				std::string name() const;
				std::string getSystemName() const;

				AtomCategoryComposition getReducedAtomCategoryComposition() const;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Customize methods

				void add(const std::string& elementSymbol, const size_type numAtoms = 1);
				void erase(const std::string& elementSymbol);
				void join(const AtomCategoryComposition&);

			// Customize methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Utility

				bool isEmpty() const;
				void clear();
				size_type getHashValue() const;

			// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::map<std::string, size_type> m_dictionary;


				friend bool operator<(const AtomCategoryComposition&, const AtomCategoryComposition&);
				friend bool operator>(const AtomCategoryComposition&, const AtomCategoryComposition&);
				friend bool operator==(const AtomCategoryComposition&, const AtomCategoryComposition&);
				friend bool operator!=(const AtomCategoryComposition&, const AtomCategoryComposition&);
			};


			inline bool operator<(const AtomCategoryComposition& acca, const AtomCategoryComposition& accb)
			{
				return (acca.m_dictionary < accb.m_dictionary);
			}

			inline bool operator>(const AtomCategoryComposition& acca, const AtomCategoryComposition& accb)
			{
				return (acca.m_dictionary > accb.m_dictionary);
			}

			inline bool operator==(const AtomCategoryComposition& acca, const AtomCategoryComposition& accb)
			{
				return (acca.m_dictionary == accb.m_dictionary);
			}

			inline bool operator!=(const AtomCategoryComposition& acca, const AtomCategoryComposition& accb)
			{
				return (acca.m_dictionary != accb.m_dictionary);
			}
		}
	}
}


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::iterator MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::begin()
{
	return m_dictionary.begin();
}

inline MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::const_iterator MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::begin() const
{
	return m_dictionary.begin();
}

inline MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::iterator MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::end()
{
	return m_dictionary.end();
}

inline MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::const_iterator MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::end() const
{
	return m_dictionary.end();
}

inline MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::size_type MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::countAtomCategories() const
{
	return m_dictionary.size();
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::contains(const std::string& symbol) const
{
	return !(m_dictionary.find(symbol) == m_dictionary.end());
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Customize methods

inline void MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::add(const std::string& symbol, const size_type numAtoms)
{
	iterator iter = m_dictionary.find(symbol);

	if (iter == m_dictionary.end())
		m_dictionary.emplace(symbol, numAtoms);
	else
		iter->second += numAtoms;
}

inline void MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::erase(const std::string& symbol)
{
	m_dictionary.erase(symbol);
}

inline void MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::join(const AtomCategoryComposition& atomCategoryComposition)
{
	for (const auto& symbolAndNum : atomCategoryComposition)
		add(symbolAndNum.first, symbolAndNum.second);
}

// Customize methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Utility

inline bool MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::isEmpty() const
{
	return m_dictionary.empty();
}

inline void MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::clear()
{
	m_dictionary.clear();
}

inline std::size_t MathematicalCrystalChemistry::StructureModel::Features::AtomCategoryComposition::getHashValue() const
{
	size_type hashValue = 0;
	{
		for (const auto& symbolAndNum : m_dictionary)
		{
			hashValue ^= std::hash<std::string>{}(symbolAndNum.first);
			hashValue ^= std::hash<size_type>{}(symbolAndNum.second);
		}
	}

	return hashValue;
}

// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_ATOMCATEGORYCOMPOSITION_H
