#ifndef MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_CHEMICALCOMPOSITIONCONSTRAINTS_H
#define MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_CHEMICALCOMPOSITIONCONSTRAINTS_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <vector>
#include <map>

#include "AtomCategory.h"

#include "CrystalCategory.h"


namespace MathematicalCrystalChemistry
{
	namespace Analysis
	{
		class ChemicalCompositionConstraints
		{
			using size_type = std::size_t;
			using AtomCategory = MathematicalCrystalChemistry::StructureModel::Features::AtomCategory;
			using CrystalCategory = MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		public:
			ChemicalCompositionConstraints();
			virtual ~ChemicalCompositionConstraints() = default;

			ChemicalCompositionConstraints(const ChemicalCompositionConstraints&) = default;
			ChemicalCompositionConstraints(ChemicalCompositionConstraints&&) noexcept = default;
			ChemicalCompositionConstraints& operator=(const ChemicalCompositionConstraints&) = default;
			ChemicalCompositionConstraints& operator=(ChemicalCompositionConstraints&&) noexcept = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			void initialize();
			void initializeFromInputFileTexts(const std::string& inputTexts);

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Access methods

			size_type minNumAtoms() const noexcept;
			size_type maxNumAtoms() const noexcept;

			const std::vector<CrystalCategory>& feasibleCrystalCategories() const noexcept;
			const std::vector<AtomCategory>& feasibleAtomCategories() const noexcept;
			const std::vector<std::string>& necessaryElementSymbols() const noexcept;
			const std::vector<std::string>& infeasibleElementSymbols() const noexcept;

		// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

		private:
			void readFeasibleCrystalCategories(const std::string& inputTexts);
			void readFeasibleAtomCategories(const std::string& inputTexts);

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			size_type m_minNumAtoms;
			size_type m_maxNumAtoms;
			std::vector<CrystalCategory> m_feasibleCrystalCategories;
			std::vector<AtomCategory> m_feasibleAtomCategories;
			std::vector<std::string> m_necessaryElementSymbols;
			std::vector<std::string> m_infeasibleElementSymbols;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline std::size_t MathematicalCrystalChemistry::Analysis::ChemicalCompositionConstraints::minNumAtoms() const noexcept
{
	return m_minNumAtoms;
}

inline std::size_t MathematicalCrystalChemistry::Analysis::ChemicalCompositionConstraints::maxNumAtoms() const noexcept
{
	return m_maxNumAtoms;
}

inline const std::vector<MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory>& MathematicalCrystalChemistry::Analysis::ChemicalCompositionConstraints::feasibleCrystalCategories() const noexcept
{
	return m_feasibleCrystalCategories;
}

inline const std::vector<MathematicalCrystalChemistry::StructureModel::Features::AtomCategory>& MathematicalCrystalChemistry::Analysis::ChemicalCompositionConstraints::feasibleAtomCategories() const noexcept
{
	return m_feasibleAtomCategories;
}

inline const std::vector<std::string>& MathematicalCrystalChemistry::Analysis::ChemicalCompositionConstraints::necessaryElementSymbols() const noexcept
{
	return m_necessaryElementSymbols;
}

inline const std::vector<std::string>& MathematicalCrystalChemistry::Analysis::ChemicalCompositionConstraints::infeasibleElementSymbols() const noexcept
{
	return m_infeasibleElementSymbols;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_CHEMICALCOMPOSITIONCONSTRAINTS_H
