#ifndef CHEMTOOLKIT_CRYSTALLOGRAPHY_FILEIO_CIFFILEEXCAVATOR_H
#define CHEMTOOLKIT_CRYSTALLOGRAPHY_FILEIO_CIFFILEEXCAVATOR_H

#include <string>
#include <filesystem>

#include "CifFileReader.h"


namespace ChemToolkit
{
	namespace Crystallography
	{
		namespace FileIO
		{
			class CifFileExcavator
			{
				using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				explicit CifFileExcavator(const size_type threadRank);
				virtual ~CifFileExcavator() = default;

				CifFileExcavator(const CifFileExcavator&) = default;
				CifFileExcavator(CifFileExcavator&&) noexcept = default;
				CifFileExcavator& operator=(const CifFileExcavator&) = default;
				CifFileExcavator& operator=(CifFileExcavator&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				const CifFileReader& cifFileReader() const;
				void excavateNextCifFile();

				virtual void initialize(const std::filesystem::path& cifDirectoryPath);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				void initializeCifFileReader();

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				size_type m_threadRank;
				CifFileReader m_cifFileReader;

				mutable std::filesystem::recursive_directory_iterator m_recursiveCifDataDirectoryIter{};
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline const ChemToolkit::Crystallography::FileIO::CifFileReader& ChemToolkit::Crystallography::FileIO::CifFileExcavator::cifFileReader() const
{
	return m_cifFileReader;
}

inline void ChemToolkit::Crystallography::FileIO::CifFileExcavator::initialize(const std::filesystem::path& cifDirectoryPath)
{
	if (std::filesystem::is_directory(cifDirectoryPath))
	{
		m_recursiveCifDataDirectoryIter = std::filesystem::recursive_directory_iterator{ cifDirectoryPath };
		initializeCifFileReader();
	}

	else
	{
		std::string errorMessage;
		{
			errorMessage += "Directory path of \"";
			errorMessage += cifDirectoryPath.generic_string();
			errorMessage += "\" is not a path to a directory.";
		}

		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "initialize", errorMessage));
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !CHEMTOOLKIT_CRYSTALLOGRAPHY_FILEIO_CIFFILEEXCAVATOR_H
