#ifndef CHEMTOOLKIT_CRYSTALLOGRAPHY_FILEIO_CIFFILEREADER_H
#define CHEMTOOLKIT_CRYSTALLOGRAPHY_FILEIO_CIFFILEREADER_H

#include "FileReader.h"

#include "CrystallographicInformation.h"


namespace ChemToolkit
{
	namespace Crystallography
	{
		namespace FileIO
		{
			class CifFileReader :public OperatingSystem::IO::FileReader
			{
				using size_type = std::size_t;
				using AffineMapping = MathToolkit::Generic::AffineMapping<double>;

				using AtomSite = ChemToolkit::Crystallography::SpaceGroups::AtomSite;
				using CrystallographicInformation = ChemToolkit::Crystallography::SpaceGroups::CrystallographicInformation;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				CifFileReader() = default;
				explicit CifFileReader(const std::filesystem::path& filePath);
				explicit CifFileReader(std::filesystem::path&& filePath);

				CifFileReader(const CifFileReader&) = default;
				CifFileReader(CifFileReader&&) noexcept = default;
				CifFileReader& operator=(const CifFileReader&) = default;
				CifFileReader& operator=(CifFileReader&&) noexcept = default;

				virtual ~CifFileReader() = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				CrystallographicInformation readStructure() const;

				static std::string correctExtension();
				static bool isCorrectExtension(const std::filesystem::path filePath);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				ChemToolkit::Crystallography::UnitCell readUnitCell(const std::string& inputText) const;
				std::vector<AtomSite> readAtomSites(const std::string& inputText) const;
				void readSpaceGroupInformation(const std::string& inputText, CrystallographicInformation&) const;

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private utility

				std::string getLoopText(const std::string& inputText, const std::string& dataSign) const;
				std::vector<std::string> readLoopDataNames(const std::string& inputText, const std::string& dataSign) const;

				size_type readSpaceGroupNumber(const std::string& inputText) const;
				AffineMapping readSymmetryOperation(const std::string& xOperation, const std::string& yOperation, const std::string& zOperation) const;

				double translateIntoTermValue(std::string termText) const;

			// Private utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline ChemToolkit::Crystallography::SpaceGroups::CrystallographicInformation ChemToolkit::Crystallography::FileIO::CifFileReader::readStructure() const
{
	std::string inputText = OperatingSystem::IO::FileReader::readFile();

	CrystallographicInformation crystallographicInformation;
	{
		crystallographicInformation.setUnitCell(readUnitCell(inputText));
		crystallographicInformation.setAtomSites(readAtomSites(inputText));

		readSpaceGroupInformation(inputText, crystallographicInformation);
	}

	return crystallographicInformation;
}

inline std::string ChemToolkit::Crystallography::FileIO::CifFileReader::correctExtension()
{
	return std::string{ ".cif" };
}

inline bool ChemToolkit::Crystallography::FileIO::CifFileReader::isCorrectExtension(const std::filesystem::path filePath)
{
	if (filePath.extension().string() == correctExtension())
		return true;
	else
		return false;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline double ChemToolkit::Crystallography::FileIO::CifFileReader::translateIntoTermValue(std::string termText) const
{
	if (termText.empty())
		return 1.0;
	else
		return readFraction(termText);
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CHEMTOOLKIT_CRYSTALLOGRAPHY_FILEIO_CIFFILEREADER_H
