#ifndef OPERATINGSYSTEM_CONFIGURATION_CLOCK_H
#define OPERATINGSYSTEM_CONFIGURATION_CLOCK_H

#include <stdexcept>
#include <string>
#include <chrono>


namespace OperatingSystem
{
	namespace Configuration
	{
		class Clock
		{
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operator

		public:
			Clock();
			virtual ~Clock() = default;

			Clock(const Clock&) = default;
			Clock(Clock&&) noexcept = default;
			Clock& operator=(const Clock&) = default;
			Clock& operator=(Clock&&) noexcept = default;

		// Constructors, destructor, and operator
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			void setStartTimePoint();
			std::string getElapsedTime() const;

			static std::string writeZonedTime();

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			std::string writeElapsedTime(const std::chrono::seconds&) const;

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			std::chrono::system_clock::time_point m_startTimePoint;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline void OperatingSystem::Configuration::Clock::setStartTimePoint()
{
	m_startTimePoint = std::chrono::system_clock::now();
}

inline std::string OperatingSystem::Configuration::Clock::getElapsedTime() const
{
	std::chrono::seconds elapsedSeconds = std::chrono::duration_cast<std::chrono::seconds>(std::chrono::system_clock::now().time_since_epoch() - m_startTimePoint.time_since_epoch());
	return writeElapsedTime(elapsedSeconds);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !OPERATINGSYSTEM_CONFIGURATION_CLOCK_H
