#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_CONSTRAINTOWNER_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_CONSTRAINTOWNER_H

#include <memory>
#include <stdexcept>

#include "ReportException.h"


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace GeometricalConstraints
		{
			template <typename C>
			class ConstraintOwner
			{
			public:
				using Constraint = C;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				ConstraintOwner(const std::shared_ptr<Constraint>& constraintPtr, const std::shared_ptr<Constraint>& reverseConstraintPtr);
				ConstraintOwner(std::shared_ptr<Constraint>&& constraintPtr, std::shared_ptr<Constraint>&& reverseConstraintPtr);
				virtual ~ConstraintOwner() = default;

				ConstraintOwner(const ConstraintOwner&) = default;
				ConstraintOwner(ConstraintOwner&&) noexcept = default;
				ConstraintOwner& operator=(const ConstraintOwner&) = default;
				ConstraintOwner& operator=(ConstraintOwner&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access method

				const std::shared_ptr<Constraint>& constraintPtr() const;
				const std::shared_ptr<Constraint>& reverseConstraintPtr() const;
				Constraint& constraintRef() const;
				Constraint& reverseConstraintRef() const;

			// Access method
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::shared_ptr<Constraint> m_constraintPtr;
				std::shared_ptr<Constraint> m_reverseConstraintPtr;


			private:
				ConstraintOwner() = delete;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

template <typename C>
inline MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::ConstraintOwner<C>::ConstraintOwner(const std::shared_ptr<Constraint>& constraintPtr, const std::shared_ptr<Constraint>& reverseConstraintPtr)
	: m_constraintPtr{ constraintPtr }
	, m_reverseConstraintPtr{ reverseConstraintPtr }
{
	if (m_constraintPtr == nullptr)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "constructor", "\"constraintPtr\" does not own any object."));
	}

	if (m_reverseConstraintPtr == nullptr)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "constructor", "\"reverseConstraintPtr\" does not own any object."));
	}
}

template <typename C>
inline MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::ConstraintOwner<C>::ConstraintOwner(std::shared_ptr<Constraint>&& constraintPtr, std::shared_ptr<Constraint>&& reverseConstraintPtr)
	: m_constraintPtr{ std::move(constraintPtr) }
	, m_reverseConstraintPtr{ std::move(reverseConstraintPtr) }
{
	if (m_constraintPtr == nullptr)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "constructor", "\"constraintPtr\" does not own any object."));
	}

	if (m_reverseConstraintPtr == nullptr)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "constructor", "\"reverseConstraintPtr\" does not own any object."));
	}
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

template <typename C>
inline const std::shared_ptr<C>& MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::ConstraintOwner<C>::constraintPtr() const
{
	return m_constraintPtr;
}

template <typename C>
inline const std::shared_ptr<C>& MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::ConstraintOwner<C>::reverseConstraintPtr() const
{
	return m_reverseConstraintPtr;
}

template <typename C>
inline C& MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::ConstraintOwner<C>::constraintRef() const
{
	return (*m_constraintPtr);
}

template <typename C>
inline C& MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::ConstraintOwner<C>::reverseConstraintRef() const
{
	return (*m_reverseConstraintPtr);
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_CONSTRAINTOWNER_H
