#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_COMPONENTS_INTERNAL_COORDINATIONCENTERBASE_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_COMPONENTS_INTERNAL_COORDINATIONCENTERBASE_H

#include "LinearAlgebra.h"

#include "CoordinationFeature.h"
#include "CoordinationFeatureSymbol.h"


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace Components
		{
			namespace Internal
			{
				class CoordinationCenterBase
				{
				protected:
					using CoordinationFeature = MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeature;
					using CoordinationFeatureSymbol = MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol;

// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Constructors, destructor, and operators

				protected:
					explicit CoordinationCenterBase(const std::string& elementSymbol);
					CoordinationCenterBase(const std::string& elementSymbol, const CoordinationFeatureSymbol&);

				public:
					virtual ~CoordinationCenterBase() = default;

					CoordinationCenterBase(const CoordinationCenterBase&);
					CoordinationCenterBase(CoordinationCenterBase&&);
					CoordinationCenterBase& operator=(const CoordinationCenterBase&);
					CoordinationCenterBase& operator=(CoordinationCenterBase&&);

				// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Access methods

					const CoordinationFeature& coordinationFeature() const noexcept;
					CoordinationFeature& coordinationFeature() noexcept;

					static double defaultPointGroupPrecision();
					static void setPointGroupPrecision();
					static void setPointGroupPrecision(const double);

				// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Protected methods

				protected:
					double attractiveForceConstant() const;
					double repulsiveForceConstant() const;
					double pointGroupPrecision() const;

					void assignCoordinationFeatureFromElementSymbol(const std::string& elementSymbol, const int oxidationState);

				// Protected methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

				private:
					CoordinationFeature m_coordinationFeature;

					double m_attractiveForceConstant;
					double m_repulsiveForceConstant;
					static double m_pointGroupPrecision;
					static double m_defaultPointGroupPrecision;
				};
			}
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeature& MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase::coordinationFeature() const noexcept
{
	return m_coordinationFeature;
}

inline MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeature& MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase::coordinationFeature() noexcept
{
	return m_coordinationFeature;
}

inline double MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase::defaultPointGroupPrecision()
{
	return m_defaultPointGroupPrecision;
}

inline void MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase::setPointGroupPrecision()
{
	m_pointGroupPrecision = m_defaultPointGroupPrecision;
}

inline void MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase::setPointGroupPrecision(const double pointGroupPrecision)
{
	if (0.0 < pointGroupPrecision)
		m_pointGroupPrecision = pointGroupPrecision;

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException("setPointGroupPrecision of MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase", "\"pointGroupPrecision\" is not more than zero."));
	}
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Protected methods

inline double MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase::attractiveForceConstant() const
{
	return m_attractiveForceConstant;
}

inline double MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase::repulsiveForceConstant() const
{
	return m_repulsiveForceConstant;
}

inline double MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase::pointGroupPrecision() const
{
	return m_pointGroupPrecision;
}

inline void MathematicalCrystalChemistry::StructureModel::Components::Internal::CoordinationCenterBase::assignCoordinationFeatureFromElementSymbol(const std::string& elementSymbol, const int oxidationState)
{
	m_coordinationFeature.initializeFromElementSymbol(elementSymbol, oxidationState);
}

// Protected methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_COMPONENTS_INTERNAL_COORDINATIONCENTERBASE_H
