#ifndef MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_ANALYSIS_COORDINATIONFEATURESPECIFIER_H
#define MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_ANALYSIS_COORDINATIONFEATURESPECIFIER_H

#include <stdexcept>
#include <string>

#include "ModelCrystalStructure.h"

#include "CoordinationFeatureSpecifyOptions.h"


namespace MathematicalCrystalChemistry
{
	namespace Runtime
	{
		namespace Analysis
		{
			class CoordinationFeatureSpecifier
			{
				using size_type = std::size_t;
				using ModelCrystalStructure = MathematicalCrystalChemistry::StructureModel::Components::ModelCrystalStructure;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				CoordinationFeatureSpecifier() = default;
				CoordinationFeatureSpecifier(const CoordinationFeatureSpecifyOptions&);
				CoordinationFeatureSpecifier(CoordinationFeatureSpecifyOptions&&);
				virtual ~CoordinationFeatureSpecifier() = default;

				CoordinationFeatureSpecifier(const CoordinationFeatureSpecifier&) = default;
				CoordinationFeatureSpecifier(CoordinationFeatureSpecifier&&) noexcept = default;
				CoordinationFeatureSpecifier& operator=(const CoordinationFeatureSpecifier&) = default;
				CoordinationFeatureSpecifier& operator=(CoordinationFeatureSpecifier&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				const CoordinationFeatureSpecifyOptions& coordinationFeatureSpecifyOptions() const noexcept;

				void setCoordinationFeatureSpecifyOptions(const CoordinationFeatureSpecifyOptions&);
				void setCoordinationFeatureSpecifyOptions(CoordinationFeatureSpecifyOptions&&);

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void specify() const;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				void initializeDirectories() const;
				void initializeChemicalParameters() const;
				void initializeGeometricalConstraints() const;

				void createDirectoryIfNecessary(const std::filesystem::path& directoryPath) const;
				void createDirectoriesIfNecessary(const std::filesystem::path& directoryPath) const;

				void createResultFile() const;
				size_type countInputCrystalStructureFingerprintFiles(const std::filesystem::path&) const;

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				CoordinationFeatureSpecifyOptions m_coordinationFeatureSpecifyOptions;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const MathematicalCrystalChemistry::Runtime::Analysis::CoordinationFeatureSpecifyOptions& MathematicalCrystalChemistry::Runtime::Analysis::CoordinationFeatureSpecifier::coordinationFeatureSpecifyOptions() const noexcept
{
	return m_coordinationFeatureSpecifyOptions;
}

inline void MathematicalCrystalChemistry::Runtime::Analysis::CoordinationFeatureSpecifier::setCoordinationFeatureSpecifyOptions(const CoordinationFeatureSpecifyOptions& options)
{
	m_coordinationFeatureSpecifyOptions = options;
}

inline void MathematicalCrystalChemistry::Runtime::Analysis::CoordinationFeatureSpecifier::setCoordinationFeatureSpecifyOptions(CoordinationFeatureSpecifyOptions&& options)
{
	m_coordinationFeatureSpecifyOptions = std::move(options);
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline void MathematicalCrystalChemistry::Runtime::Analysis::CoordinationFeatureSpecifier::createDirectoryIfNecessary(const std::filesystem::path& directoryPath) const
{
	if (!(std::filesystem::is_directory(directoryPath)))
	{
		if (!(std::filesystem::create_directory(directoryPath)))
		{
			std::string errorMessage;
			{
				errorMessage += "Could not create the directory of \"";
				errorMessage += directoryPath.generic_string();
				errorMessage += "\"";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "createDirectoryIfNecessary", errorMessage));
		}
	}
}

inline void MathematicalCrystalChemistry::Runtime::Analysis::CoordinationFeatureSpecifier::createDirectoriesIfNecessary(const std::filesystem::path& directoryPath) const
{
	if (!(std::filesystem::is_directory(directoryPath)))
	{
		if (!(std::filesystem::create_directories(directoryPath)))
		{
			std::string errorMessage;
			{
				errorMessage += "Could not create the directory of \"";
				errorMessage += directoryPath.generic_string();
				errorMessage += "\"";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "createDirectoriesIfNecessary", errorMessage));
		}
	}
}

inline std::size_t MathematicalCrystalChemistry::Runtime::Analysis::CoordinationFeatureSpecifier::countInputCrystalStructureFingerprintFiles(const std::filesystem::path& reoptimizationResultDirectoryPath) const
{
	size_type numFiles = 0;
	{
		if (std::filesystem::is_directory(reoptimizationResultDirectoryPath))
		{
			for (const auto& directoryEntry : std::filesystem::recursive_directory_iterator(reoptimizationResultDirectoryPath))
			{
				if (directoryEntry.is_regular_file())
				{
					std::filesystem::path filePath = directoryEntry.path();

					if ("input_crystal_structure_fingerprint.txt" == filePath.filename().string())
						++numFiles;
				}
			}
		}
	}

	return numFiles;
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_ANALYSIS_COORDINATIONFEATURESPECIFIER_H
