#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_COORDINATIONFEATURESYMBOL_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_COORDINATIONFEATURESYMBOL_H

#include <stdexcept>
#include <string>


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace Features
		{
			class CoordinationFeatureSymbol
			{
				using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				CoordinationFeatureSymbol();
				explicit CoordinationFeatureSymbol(const std::string& name);
				virtual ~CoordinationFeatureSymbol() = default;

				CoordinationFeatureSymbol(const CoordinationFeatureSymbol&) = default;
				CoordinationFeatureSymbol(CoordinationFeatureSymbol&&) noexcept = default;
				CoordinationFeatureSymbol& operator=(const CoordinationFeatureSymbol&) = default;
				CoordinationFeatureSymbol& operator=(CoordinationFeatureSymbol&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				const std::string& atomCategorySymbol() const;
				const std::string& upperIonicBondRadiusSymbol() const;
				const std::string& lowerIonicBondRadiusSymbol() const;
				const std::string& repulsiveIonicRadiusSymbol() const;
				const std::string& upperMetallicBondRadiusSymbol() const;
				const std::string& lowerMetallicBondRadiusSymbol() const;
				const std::string& ionicBondCompositionSymbol() const;
				const std::string& metallicBondCompositionSymbol() const;

				void setAtomCategorySymbol(const std::string&);
				void setUpperIonicBondRadiusSymbol(const std::string&);
				void setLowerIonicBondRadiusSymbol(const std::string&);
				void setRepulsiveIonicRadiusSymbol(const std::string&);
				void setUpperMetallicBondRadiusSymbol(const std::string&);
				void setLowerMetallicBondRadiusSymbol(const std::string&);
				void setIonicBondCompositionSymbol(const std::string&);
				void setMetallicBondCompositionSymbol(const std::string&);

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				std::string name() const;

				void clear();
				bool isValid() const;

				void initializeFromElementSymbol(const std::string& elementSymbol);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				static std::string readAtomCategorySymbol(const std::string&);

				static std::string readUpperIonicBondRadiusSymbol(const std::string&);
				static std::string readLowerIonicBondRadiusSymbol(const std::string&);
				static std::string readRepulsiveIonicRadiusSymbol(const std::string&);
				static std::string readUpperMetallicBondRadiusSymbol(const std::string&);
				static std::string readLowerMetallicBondRadiusSymbol(const std::string&);

				static std::string readIonicBondCompositionSymbol(const std::string&);
				static std::string readMetallicBondCompositionSymbol(const std::string&);

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::string m_atomCategorySymbol;

				std::string m_upperIonicBondRadiusSymbol;
				std::string m_lowerIonicBondRadiusSymbol;
				std::string m_repulsiveIonicRadiusSymbol;
				std::string m_upperMetallicBondRadiusSymbol;
				std::string m_lowerMetallicBondRadiusSymbol;
				std::string m_ionicBondCompositionSymbol;
				std::string m_metallicBondCompositionSymbol;

				static std::string m_symbolPat;
			};


			inline bool operator==(const CoordinationFeatureSymbol& cfa, const CoordinationFeatureSymbol& cfb)
			{
				return (cfa.name() == cfb.name());
			}

			inline bool operator!=(const CoordinationFeatureSymbol& cfa, const CoordinationFeatureSymbol& cfb)
			{
				return (cfa.name() != cfb.name());
			}
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const std::string& MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::atomCategorySymbol() const
{
	return m_atomCategorySymbol;
}

inline const std::string& MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::upperIonicBondRadiusSymbol() const
{
	return m_upperIonicBondRadiusSymbol;
}

inline const std::string& MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::lowerIonicBondRadiusSymbol() const
{
	return m_lowerIonicBondRadiusSymbol;
}

inline const std::string& MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::repulsiveIonicRadiusSymbol() const
{
	return m_repulsiveIonicRadiusSymbol;
}

inline const std::string& MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::upperMetallicBondRadiusSymbol() const
{
	return m_upperMetallicBondRadiusSymbol;
}

inline const std::string& MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::lowerMetallicBondRadiusSymbol() const
{
	return m_lowerMetallicBondRadiusSymbol;
}

inline const std::string& MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::ionicBondCompositionSymbol() const
{
	return m_ionicBondCompositionSymbol;
}

inline const std::string& MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::metallicBondCompositionSymbol() const
{
	return m_metallicBondCompositionSymbol;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::string MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::name() const
{
	std::string totalSymbol;
	{
		totalSymbol += m_atomCategorySymbol;
		totalSymbol += "-";
		totalSymbol += m_upperIonicBondRadiusSymbol;
		totalSymbol += m_lowerIonicBondRadiusSymbol;
		totalSymbol += "^{";
		totalSymbol += m_ionicBondCompositionSymbol;
		totalSymbol += "}-";
		totalSymbol += m_repulsiveIonicRadiusSymbol;
		totalSymbol += "-";
		totalSymbol += m_upperMetallicBondRadiusSymbol;
		totalSymbol += m_lowerMetallicBondRadiusSymbol;
		totalSymbol += "^{";
		totalSymbol += m_metallicBondCompositionSymbol;
		totalSymbol += "}";
	}

	return totalSymbol;
}

inline void MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::clear()
{
	m_atomCategorySymbol.clear();
	m_upperIonicBondRadiusSymbol = std::string{ "Z" };
	m_lowerIonicBondRadiusSymbol = std::string{ "z" };
	m_repulsiveIonicRadiusSymbol = std::string{ "Z" };
	m_upperMetallicBondRadiusSymbol = std::string{ "Z" };
	m_lowerMetallicBondRadiusSymbol = std::string{ "z" };
	m_ionicBondCompositionSymbol.clear();
	m_metallicBondCompositionSymbol.clear();
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol::isValid() const
{
	if (m_atomCategorySymbol.empty())
		return false;
	if (m_upperIonicBondRadiusSymbol.empty())
		return false;
	if (m_lowerIonicBondRadiusSymbol.empty())
		return false;
	if (m_repulsiveIonicRadiusSymbol.empty())
		return false;
	if (m_upperMetallicBondRadiusSymbol.empty())
		return false;
	if (m_lowerMetallicBondRadiusSymbol.empty())
		return false;

	return true;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_COORDINATIONFEATURESYMBOL_H
