#ifndef MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_CRYSTALANALYSISOPTIONS_H
#define MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_CRYSTALANALYSISOPTIONS_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <vector>

#include "ChemicalCompositionConstraints.h"
#include "CoordinationPolyhedraConstraints.h"
#include "WholeStructuralConstraints.h"


namespace MathematicalCrystalChemistry
{
	namespace Analysis
	{
		class CrystalAnalysisOptions
		{
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operators

		public:
			CrystalAnalysisOptions() = default;
			virtual ~CrystalAnalysisOptions() = default;

			CrystalAnalysisOptions(const CrystalAnalysisOptions&) = default;
			CrystalAnalysisOptions(CrystalAnalysisOptions&&) noexcept = default;
			CrystalAnalysisOptions& operator=(const CrystalAnalysisOptions&) = default;
			CrystalAnalysisOptions& operator=(CrystalAnalysisOptions&&) noexcept = default;

		// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			void initialize();
			void initializeFromFiles(const std::vector<std::filesystem::path>& filePaths);

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Access methods

			const ChemicalCompositionConstraints& chemicalCompositionConstraints() const noexcept;
			const CoordinationPolyhedraConstraints& coordinationPolyhedraConstraints() const noexcept;
			const WholeStructuralConstraints& wholeStructuralConstraints() const noexcept;

			void setChemicalCompositionConstraints(const ChemicalCompositionConstraints&);
			void setCoordinationPolyhedraConstraints(const CoordinationPolyhedraConstraints&);
			void setWholeStructuralConstraints(const WholeStructuralConstraints&);

		// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************


		private:
			ChemicalCompositionConstraints m_chemicalCompositionConstraints;
			CoordinationPolyhedraConstraints m_coordinationPolyhedraConstraints;
			WholeStructuralConstraints m_wholeStructuralConstraints;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const MathematicalCrystalChemistry::Analysis::ChemicalCompositionConstraints& MathematicalCrystalChemistry::Analysis::CrystalAnalysisOptions::chemicalCompositionConstraints() const noexcept
{
	return m_chemicalCompositionConstraints;
}

inline const MathematicalCrystalChemistry::Analysis::CoordinationPolyhedraConstraints& MathematicalCrystalChemistry::Analysis::CrystalAnalysisOptions::coordinationPolyhedraConstraints() const noexcept
{
	return m_coordinationPolyhedraConstraints;
}

inline const MathematicalCrystalChemistry::Analysis::WholeStructuralConstraints& MathematicalCrystalChemistry::Analysis::CrystalAnalysisOptions::wholeStructuralConstraints() const noexcept
{
	return m_wholeStructuralConstraints;
}

inline void MathematicalCrystalChemistry::Analysis::CrystalAnalysisOptions::setChemicalCompositionConstraints(const ChemicalCompositionConstraints& chemicalCompositionConstraints)
{
	m_chemicalCompositionConstraints = chemicalCompositionConstraints;
}

inline void MathematicalCrystalChemistry::Analysis::CrystalAnalysisOptions::setCoordinationPolyhedraConstraints(const CoordinationPolyhedraConstraints& coordinationPolyhedraConstraints)
{
	m_coordinationPolyhedraConstraints = coordinationPolyhedraConstraints;
}

inline void MathematicalCrystalChemistry::Analysis::CrystalAnalysisOptions::setWholeStructuralConstraints(const WholeStructuralConstraints& wholeStructuralConstraints)
{
	m_wholeStructuralConstraints = wholeStructuralConstraints;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_CRYSTALANALYSISOPTIONS_H
