#ifndef MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_CRYSTALANALYZER_H
#define MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_CRYSTALANALYZER_H

#include <stdexcept>
#include <string>

#include "ModelCrystalStructure.h"

#include "IsFeasibleChemicalComposition.h"
#include "IsFeasibleCoordinationPolyhedra.h"
#include "IsFeasibleWholeStructure.h"
#include "SpecifyCoordinationFeatures.h"


namespace MathematicalCrystalChemistry
{
	namespace Analysis
	{
		class CrystalAnalyzer
		{
			using size_type = std::size_t;
			using ModelCrystalStructure = MathematicalCrystalChemistry::StructureModel::Components::ModelCrystalStructure;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operators

		public:
			CrystalAnalyzer() = default;
			virtual ~CrystalAnalyzer() = default;

			CrystalAnalyzer(const CrystalAnalyzer&) = default;
			CrystalAnalyzer(CrystalAnalyzer&&) noexcept = default;
			CrystalAnalyzer& operator=(const CrystalAnalyzer&) = default;
			CrystalAnalyzer& operator=(CrystalAnalyzer&&) noexcept = default;

		// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			bool isOptimalSolution(const ModelCrystalStructure&) const;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Access methods

			void setChemicalCompositionConstraints(const ChemicalCompositionConstraints&);
			void setCoordinationPolyhedraConstraints(const CoordinationPolyhedraConstraints&);
			void setWholeStructuralConstraints(const WholeStructuralConstraints&);

		// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			Internal::IsFeasibleChemicalComposition m_isFeasibleChemicalComposition;
			Internal::IsFeasibleCoordinationPolyhedra m_isFeasibleCoordinationPolyhedra;
			Internal::IsFeasibleWholeStructure m_isFeasibleWholeStructure;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline bool MathematicalCrystalChemistry::Analysis::CrystalAnalyzer::isOptimalSolution(const ModelCrystalStructure& modelCrystalStructure) const
{
	if (m_isFeasibleChemicalComposition(modelCrystalStructure))
	{
		if (m_isFeasibleCoordinationPolyhedra(modelCrystalStructure))
			return m_isFeasibleWholeStructure(modelCrystalStructure);
		else
			return false;
	}

	else
		return false;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline void MathematicalCrystalChemistry::Analysis::CrystalAnalyzer::setChemicalCompositionConstraints(const ChemicalCompositionConstraints& chemicalCompositionConstraints)
{
	m_isFeasibleChemicalComposition.setChemicalCompositionConstraints(chemicalCompositionConstraints);
}

inline void MathematicalCrystalChemistry::Analysis::CrystalAnalyzer::setCoordinationPolyhedraConstraints(const CoordinationPolyhedraConstraints& coordinationPolyhedraConstraints)
{
	m_isFeasibleCoordinationPolyhedra.setCoordinationPolyhedraConstraints(coordinationPolyhedraConstraints);
}

inline void MathematicalCrystalChemistry::Analysis::CrystalAnalyzer::setWholeStructuralConstraints(const WholeStructuralConstraints& wholeStructuralConstraints)
{
	m_isFeasibleWholeStructure.setWholeStructuralConstraints(wholeStructuralConstraints);
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_CRYSTALANALYZER_H
