#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_CRYSTALCATEGORY_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_CRYSTALCATEGORY_H

#include <stdexcept>
#include <string>

#include "ReportException.h"


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace Features
		{
			class CrystalCategory
			{
				using size_type = std::size_t;

				enum class Item
				{
					ionic,
					metallic,
					metallic_ionic,
					covalent,
					crystal
				};

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				CrystalCategory();
				explicit CrystalCategory(const std::string& categorySymbol);
				virtual ~CrystalCategory() = default;

				CrystalCategory(const CrystalCategory&) = default;
				CrystalCategory(CrystalCategory&&) noexcept = default;
				CrystalCategory& operator=(const CrystalCategory&) = default;
				CrystalCategory& operator=(CrystalCategory&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				std::string symbol() const;

				bool isIonic() const;
				bool isMetallic() const;
				bool isMetallicIonic() const;
				bool isCovalent() const;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				Item m_item;


				friend bool operator==(const CrystalCategory&, const CrystalCategory&);
				friend bool operator!=(const CrystalCategory&, const CrystalCategory&);
			};


			inline bool operator==(const CrystalCategory& cca, const CrystalCategory& ccb)
			{
				return (cca.m_item == ccb.m_item);
			}

			inline bool operator!=(const CrystalCategory& cca, const CrystalCategory& ccb)
			{
				return (cca.m_item != ccb.m_item);
			}
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

inline MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory::CrystalCategory()
	: m_item{ Item::crystal }
{
}

inline MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory::CrystalCategory(const std::string& symbol)
	: m_item{ Item::crystal }
{
	if (symbol == "IONIC" || symbol == "Ionic" || symbol == "ionic")
		m_item = Item::ionic;
	else if (symbol == "METALLIC" || symbol == "Metallic" || symbol == "metallic")
		m_item = Item::metallic;
	else if (symbol == "METALLIC_IONIC" || symbol == "Metallic_ionic" || symbol == "metallic_ionic")
		m_item = Item::metallic_ionic;
	else if (symbol == "COVALENT" || symbol == "Covalent" || symbol == "covalent")
		m_item = Item::covalent;

	else
	{
		std::string errorMessage;
		{
			errorMessage += "\"";
			errorMessage += symbol;
			errorMessage += "\" is not a symbol of crystal category.";
		}

		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException("getItem of MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory", errorMessage));
	}
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::string MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory::symbol() const
{
	if (m_item == Item::ionic)
		return std::string{ "ionic" };

	else if (m_item == Item::metallic)
		return std::string{ "metallic" };

	else if (m_item == Item::metallic_ionic)
		return std::string{ "metallic_ionic" };

	else if (m_item == Item::covalent)
		return std::string{ "covalent" };

	else
		return std::string{};
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory::isIonic() const
{
	if (m_item == Item::ionic)
		return true;
	else
		return false;
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory::isMetallic() const
{
	if (m_item == Item::metallic)
		return true;
	else
		return false;
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory::isMetallicIonic() const
{
	if (m_item == Item::metallic_ionic)
		return true;
	else
		return false;
}

inline bool MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory::isCovalent() const
{
	if (m_item == Item::covalent)
		return true;
	else
		return false;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_CRYSTALCATEGORY_H

