#ifndef OPERATINGSYSTEM_SYSTEMMANAGEMENT_CRYSTALPROTOTYPEDESIGNER_H
#define OPERATINGSYSTEM_SYSTEMMANAGEMENT_CRYSTALPROTOTYPEDESIGNER_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <vector>
#include <iterator>


namespace OperatingSystem
{
	namespace SystemManagement
	{
		class CrystalPrototypeDesigner final
		{
			using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		public:
			CrystalPrototypeDesigner(int argc, char* argv[]);
			~CrystalPrototypeDesigner() = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			int execute();

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			void initializeExecutionEnvironment();

			void readAnalysisOptionFilePaths();
			void readCreationOptionFilePaths();
			void readSearchOptionFilePaths();

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private utility

			std::vector<std::string> readCommandLineArgument(const std::string& flag) const;

		// Private utility
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			std::vector<std::string> m_commandLineArgument;

			std::vector<std::filesystem::path> m_analysisOptionFilePaths;
			std::vector<std::filesystem::path> m_designOptionFilePaths;
			std::vector<std::filesystem::path> m_searchOptionFilePaths;


		private:
			CrystalPrototypeDesigner() = delete;
			CrystalPrototypeDesigner(const CrystalPrototypeDesigner&) = delete;
			CrystalPrototypeDesigner(CrystalPrototypeDesigner&&) = delete;
			CrystalPrototypeDesigner& operator=(const CrystalPrototypeDesigner&) = delete;
			CrystalPrototypeDesigner& operator=(CrystalPrototypeDesigner&&) = delete;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline void OperatingSystem::SystemManagement::CrystalPrototypeDesigner::readAnalysisOptionFilePaths()
{
	std::vector<std::string> designStrings = readCommandLineArgument(std::string{ "-A" });

	for (const auto& designString : designStrings)
		m_analysisOptionFilePaths.push_back(std::filesystem::path{ designString });
}

inline void OperatingSystem::SystemManagement::CrystalPrototypeDesigner::readCreationOptionFilePaths()
{
	std::vector<std::string> designStrings = readCommandLineArgument(std::string{ "-SC" });

	for (const auto& designString : designStrings)
		m_designOptionFilePaths.push_back(std::filesystem::path{ designString });
}

inline void OperatingSystem::SystemManagement::CrystalPrototypeDesigner::readSearchOptionFilePaths()
{
	std::vector<std::string> searchStrings = readCommandLineArgument(std::string{ "-SS" });

	for (const auto& searchString : searchStrings)
		m_searchOptionFilePaths.push_back(std::filesystem::path{ searchString });
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !OPERATINGSYSTEM_SYSTEMMANAGEMENT_CRYSTALPROTOTYPEDESIGNER_H
