#ifndef MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_SELECTION_CRYSTALSELECTPLAN_H
#define MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_SELECTION_CRYSTALSELECTPLAN_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <vector>

#include "ChemicalParameters.h"
#include "CrystalAnalysisOptions.h"
#include "GeometricalConstraintParameters.h"
#include "StructuralOptimizationParameters.h"


namespace MathematicalCrystalChemistry
{
	namespace Runtime
	{
		namespace Selection
		{
			class CrystalSelectPlan
			{
				using ChemicalParameters = ChemToolkit::Generic::ChemicalParameters;
				using CrystalAnalysisOptions = MathematicalCrystalChemistry::Analysis::CrystalAnalysisOptions;
				using GeometricalConstraintParameters = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters;
				using StructuralOptimizationParameters = MathematicalCrystalChemistry::Design::Optimization::StructuralOptimizationParameters;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructor and destructor

			public:
				CrystalSelectPlan() = default;
				virtual ~CrystalSelectPlan() = default;

				CrystalSelectPlan(const CrystalSelectPlan&) = default;
				CrystalSelectPlan(CrystalSelectPlan&&) noexcept = default;
				CrystalSelectPlan& operator=(const CrystalSelectPlan&) = default;
				CrystalSelectPlan& operator=(CrystalSelectPlan&&) noexcept = default;

			// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void initialize();
				void initializeFromFiles(const std::vector<std::filesystem::path>& filePaths);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				const std::filesystem::path& cifDataDirectoryPath() const;
				const std::filesystem::path& outputDirectoryPath() const;

				const ChemicalParameters& chemicalParameters() const;
				const CrystalAnalysisOptions& crystalAnalysisOptions() const;
				const GeometricalConstraintParameters& geometricalConstraintParameters() const;
				const StructuralOptimizationParameters& structuralOptimizationParameters() const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				void readIOPaths(const std::string& fileTexts);

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::filesystem::path m_cifDataDirectoryPath;
				std::filesystem::path m_outputDirectoryPath;

				ChemicalParameters m_chemicalParameters;
				CrystalAnalysisOptions m_crystalAnalysisOptions;
				GeometricalConstraintParameters m_geometricalConstraintParameters;
				StructuralOptimizationParameters m_structuralOptimizationParameters;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const std::filesystem::path& MathematicalCrystalChemistry::Runtime::Selection::CrystalSelectPlan::cifDataDirectoryPath() const
{
	return m_cifDataDirectoryPath;
}

inline const std::filesystem::path& MathematicalCrystalChemistry::Runtime::Selection::CrystalSelectPlan::outputDirectoryPath() const
{
	return m_outputDirectoryPath;
}

inline const ChemToolkit::Generic::ChemicalParameters& MathematicalCrystalChemistry::Runtime::Selection::CrystalSelectPlan::chemicalParameters() const
{
	return m_chemicalParameters;
}

inline const MathematicalCrystalChemistry::Analysis::CrystalAnalysisOptions& MathematicalCrystalChemistry::Runtime::Selection::CrystalSelectPlan::crystalAnalysisOptions() const
{
	return m_crystalAnalysisOptions;
}

inline const MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters& MathematicalCrystalChemistry::Runtime::Selection::CrystalSelectPlan::geometricalConstraintParameters() const
{
	return m_geometricalConstraintParameters;
}

inline const MathematicalCrystalChemistry::Design::Optimization::StructuralOptimizationParameters& MathematicalCrystalChemistry::Runtime::Selection::CrystalSelectPlan::structuralOptimizationParameters() const
{
	return m_structuralOptimizationParameters;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_SELECTION_CRYSTALSELECTPLAN_H
