#ifndef MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_SELECTION_CRYSTALSELECTOR_H
#define MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_SELECTION_CRYSTALSELECTOR_H

#include "CrystalSelectPlan.h"


namespace MathematicalCrystalChemistry
{
	namespace Runtime
	{
		namespace Selection
		{
			class CrystalSelector
			{
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				CrystalSelector() = default;
				CrystalSelector(const CrystalSelectPlan&);
				CrystalSelector(CrystalSelectPlan&&);
				virtual ~CrystalSelector() = default;

				CrystalSelector(const CrystalSelector&) = default;
				CrystalSelector(CrystalSelector&&) noexcept = default;
				CrystalSelector& operator=(const CrystalSelector&) = default;
				CrystalSelector& operator=(CrystalSelector&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void execute() const;

				void setCrystalSelectPlan(const CrystalSelectPlan&);
				void setCrystalSelectPlan(CrystalSelectPlan&&);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				void initializeDirectories() const;
				void initializeChemicalParameters() const;
				void initializeGeometricalConstraints() const;

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				CrystalSelectPlan m_crystalSelectPlan;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline void MathematicalCrystalChemistry::Runtime::Selection::CrystalSelector::setCrystalSelectPlan(const CrystalSelectPlan& crystalSelectPlan)
{
	m_crystalSelectPlan = crystalSelectPlan;
}

inline void MathematicalCrystalChemistry::Runtime::Selection::CrystalSelector::setCrystalSelectPlan(CrystalSelectPlan&& crystalSelectPlan)
{
	m_crystalSelectPlan = std::move(crystalSelectPlan);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_SELECTION_CRYSTALSELECTOR_H
