#ifndef MATHTOOLKIT_UNITCONVERSION_ENERGYCASTING_H
#define MATHTOOLKIT_UNITCONVERSION_ENERGYCASTING_H

#include <ratio>
#include <stdexcept>

#include "ReportException.h"

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Units

namespace MathToolkit
{
	namespace UnitConversion
	{
		namespace EnergyCasting
		{
			namespace Unit
			{
				struct AtomicUnit
				{
					static constexpr double getUnitEnergyInSI() { return (4.3597447222071 * static_cast<double>(std::atto::num) / static_cast<double>(std::atto::den)); }
				};

				template <typename Magnitude = std::ratio<1>>
				struct Joule
				{
					static constexpr double getUnitEnergyInSI() { return (static_cast<double>(Magnitude::num) / static_cast<double>(Magnitude::den)); }
				};

				template <typename Magnitude = std::ratio<1>>
				struct Electronvolt
				{
					static constexpr double getUnitEnergyInSI()
					{
						constexpr double electronVolt = 0.1602176634 * (static_cast<double>(std::atto::num) / static_cast<double>(std::atto::den));
						return (electronVolt * static_cast<double>(Magnitude::num) / static_cast<double>(Magnitude::den));
					}
				};
			}
		}
	}
}

// Units
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Methods

namespace MathToolkit
{
	namespace UnitConversion
	{
		namespace EnergyCasting
		{
			template <typename from, typename to>
			inline constexpr double cast(const double value)
			{
				return ((value * from::getUnitEnergyInSI()) / to::getUnitEnergyInSI());
			}

			template <typename from, typename to>
			inline constexpr double getScalingMultiplier()
			{
				return (from::getUnitEnergyInSI() / to::getUnitEnergyInSI());
			}
		}
	}
}

// Methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !MATHTOOLKIT_UNITCONVERSION_ENERGYCASTING_H
