#ifndef OPERATINGSYSTEM_IO_FILEREADER_H
#define OPERATINGSYSTEM_IO_FILEREADER_H

#include <regex>
#include <vector>

#include "FileBase.h"


namespace OperatingSystem
{
	namespace IO
	{
		class FileReader :public FileBase
		{
			using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operators

		public:
			FileReader() = default;
			explicit FileReader(const std::filesystem::path& filePath);
			explicit FileReader(std::filesystem::path&& filePath);

			FileReader(const FileReader&) = default;
			FileReader(FileReader&&) noexcept = default;
			FileReader& operator=(const FileReader&) = default;
			FileReader& operator=(FileReader&&) noexcept = default;

			virtual ~FileReader() = default;

		// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			std::string readFile() const;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Static methods

			template <typename T>
			static bool tryReadData(const std::string& inputText, const std::string& dataFlag, T&);

			template <typename T>
			static void readData(const std::string& inputText, const std::string& dataFlag, T&);

			template <typename T>
			static void translateString(const std::string&, T&);


			static std::vector<std::vector<std::string>> readDataEnumeration(const std::string& inputText, const std::string& dataFlag);
			static double readFraction(const std::string&);

		// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			static std::string findDataLine(const std::string& inputText, const std::string& dataFlag);

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Utility

template <typename T>
inline void OperatingSystem::IO::FileReader::readData(const std::string& inputText, const std::string& dataFlag, T& val)
{
	if (!(tryReadData(inputText, dataFlag, val)))
	{
		std::string errorMessage;
		{
			errorMessage += "Could not read ";
			errorMessage += dataFlag;
			errorMessage += ".";
		}

		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException("OperatingSystem::IO::FileReader::readData<T>", errorMessage));
	}
}

// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline std::string OperatingSystem::IO::FileReader::findDataLine(const std::string& inputText, const std::string& dataFlag)
{
	size_type initialPos = inputText.find(dataFlag);
	size_type endPos = inputText.find("\n", initialPos);

	if (initialPos < endPos)
		return std::string{ inputText, initialPos, (endPos - initialPos) };
	else
		return std::string{};
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !OPERATINGSYSTEM_IO_FILEREADER_H
