#ifndef OPERATINGSYSTEM_IO_FILEWRITER_H
#define OPERATINGSYSTEM_IO_FILEWRITER_H

#include "FileBase.h"


namespace OperatingSystem
{
	namespace IO
	{
		enum class FileCreateOptions
		{
			none,
			skip_existing,
			overwrite_existing,
		};


		class FileWriter :public FileBase
		{
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operators

		public:
			FileWriter() = default;
			explicit FileWriter(const std::filesystem::path& filePath);
			explicit FileWriter(std::filesystem::path&& filePath);

			FileWriter(const FileWriter&) = default;
			FileWriter(FileWriter&&) noexcept = default;
			FileWriter& operator=(const FileWriter&) = default;
			FileWriter& operator=(FileWriter&&) noexcept = default;

			virtual ~FileWriter() = default;

		// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			bool isCreatable(FileCreateOptions createOption) const;
			void clearFile() const;

			virtual void createFile(FileCreateOptions createOption, const std::string& message = std::string{}) const;
			virtual void write(const std::string& message) const;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Static methods

			static std::string toString(const double) noexcept;

		// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			void overCreate(const std::string& message) const;

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			static std::mutex m_outputMutex;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline bool OperatingSystem::IO::FileWriter::isCreatable(FileCreateOptions createOption) const
{
	if (createOption == FileCreateOptions::none || createOption == FileCreateOptions::skip_existing)
	{
		if (std::filesystem::is_regular_file(filePath()))
			return false;
		else
			return std::filesystem::is_directory(filePath().parent_path());
	}

	else
		return std::filesystem::is_directory(filePath().parent_path());
}

inline void OperatingSystem::IO::FileWriter::clearFile() const
{
	overCreate(std::string{});
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !OPERATINGSYSTEM_IO_FILEWRITER_H
