#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_GEOMETRICALCONSTRAINTPARAMETERS_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_GEOMETRICALCONSTRAINTPARAMETERS_H

#include <stdexcept>
#include <string>

#include "ReportException.h"


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace GeometricalConstraints
		{
			class GeometricalConstraintParameters
			{
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				GeometricalConstraintParameters();
				virtual ~GeometricalConstraintParameters() = default;

				GeometricalConstraintParameters(const GeometricalConstraintParameters&) = default;
				GeometricalConstraintParameters(GeometricalConstraintParameters&&) noexcept = default;
				GeometricalConstraintParameters& operator=(const GeometricalConstraintParameters&) = default;
				GeometricalConstraintParameters& operator=(GeometricalConstraintParameters&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void initialize();
				void initializeFromInputFileTexts(const std::string& inputTexts);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				double ionicBondCutoffLengthRatio() const;
				double metallicBondCutoffLengthRatio() const;
				double anionicConstraintCutoffLengthRatio() const;
				double nonIonicBondCutoffLengthRatio() const;
				double nonMetallicBondCutoffLengthRatio() const;

				double nonIonicBondRangeRatio() const;
				double nonMetallicBondRangeRatio() const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				double m_ionicBondCutoffLengthRatio;
				double m_metallicBondCutoffLengthRatio;
				double m_anionicConstraintCutoffLengthRatio;
				double m_nonIonicBondCutoffLengthRatio;
				double m_nonMetallicBondCutoffLengthRatio;

				double m_nonIonicBondRangeRatio;
				double m_nonMetallicBondRangeRatio;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline double MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters::ionicBondCutoffLengthRatio() const
{
	return m_ionicBondCutoffLengthRatio;
}

inline double MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters::metallicBondCutoffLengthRatio() const
{
	return m_metallicBondCutoffLengthRatio;
}

inline double MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters::anionicConstraintCutoffLengthRatio() const
{
	return m_anionicConstraintCutoffLengthRatio;
}

inline double MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters::nonIonicBondCutoffLengthRatio() const
{
	return m_nonIonicBondCutoffLengthRatio;
}

inline double MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters::nonMetallicBondCutoffLengthRatio() const
{
	return m_nonMetallicBondCutoffLengthRatio;
}

inline double MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters::nonIonicBondRangeRatio() const
{
	return m_nonIonicBondRangeRatio;
}

inline double MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters::nonMetallicBondRangeRatio() const
{
	return m_nonMetallicBondRangeRatio;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_GEOMETRICALCONSTRAINTPARAMETERS_H
