#ifndef MATHEMATICALCRYSTALCHEMISTRY_DESIGN_GENERATION_INITIALSTRUCTUREGENERATIONOPTIONS_H
#define MATHEMATICALCRYSTALCHEMISTRY_DESIGN_GENERATION_INITIALSTRUCTUREGENERATIONOPTIONS_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <vector>
#include <map>

#include "ReportException.h"

#include "RandomStructureGenerationParameters.h"


namespace MathematicalCrystalChemistry
{
	namespace Design
	{
		namespace Generation
		{
			class InitialStructureGenerationOptions
			{
				using size_type = std::size_t;

			public:
				enum class GenerationType
				{
					random,
				};

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				InitialStructureGenerationOptions();
				virtual ~InitialStructureGenerationOptions() = default;

				InitialStructureGenerationOptions(const InitialStructureGenerationOptions&) = default;
				InitialStructureGenerationOptions(InitialStructureGenerationOptions&&) = default;
				InitialStructureGenerationOptions& operator=(const InitialStructureGenerationOptions&) = default;
				InitialStructureGenerationOptions& operator=(InitialStructureGenerationOptions&&) = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void initialize();
				void initializeFromFiles(const std::vector<std::filesystem::path>& filePaths);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				std::map<GenerationType, size_type> structureGenerating() const;

				const RandomStructureGenerationParameters& randomStructureGenerationParameters() const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods
				
			private:
				void readStructureGenerating(const std::string& inputTexts);
				GenerationType translateIntoGenerationType(const std::string&) const;

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::map<GenerationType, size_type> m_structureGenerating;

				RandomStructureGenerationParameters m_randomStructureGenerationParameters;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline std::map<MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerationOptions::GenerationType, std::size_t> MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerationOptions::structureGenerating() const
{
	return m_structureGenerating;
}

inline const MathematicalCrystalChemistry::Design::Generation::RandomStructureGenerationParameters& MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerationOptions::randomStructureGenerationParameters() const
{
	return m_randomStructureGenerationParameters;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerationOptions::GenerationType MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerationOptions::translateIntoGenerationType(const std::string& name) const
{
	if (name == "RANDOM" || name == "Random" || name == "random")
		return GenerationType::random;

	else
	{
		std::string errorMessage;
		{
			errorMessage += "Generation typename of \"";
			errorMessage += name;
			errorMessage += "\" is out of definitions.";
		}

		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "designateStructureGenerator", errorMessage));
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_DESIGN_GENERATION_INITIALSTRUCTUREGENERATIONOPTIONS_H
