#ifndef MATHEMATICALCRYSTALCHEMISTRY_DESIGN_GENERATION_INITIALSTRUCTUREGENERATOR_H
#define MATHEMATICALCRYSTALCHEMISTRY_DESIGN_GENERATION_INITIALSTRUCTUREGENERATOR_H

#include "ModelAtomCollection.h"
#include "ModelCrystalStructure.h"

#include "RandomStructureGenerator.h"
#include "InitialStructureGenerationOptions.h"


namespace MathematicalCrystalChemistry
{
	namespace Design
	{
		namespace Generation
		{
			class InitialStructureGenerator
			{
				using ModelAtomCollection = MathematicalCrystalChemistry::StructureModel::Components::ModelAtomCollection;
				using ModelCrystalStructure = MathematicalCrystalChemistry::StructureModel::Components::ModelCrystalStructure;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructor and destructor

			public:
				InitialStructureGenerator();
				virtual ~InitialStructureGenerator() = default;

				InitialStructureGenerator(const InitialStructureGenerator&) = default;
				InitialStructureGenerator(InitialStructureGenerator&&) = default;
				InitialStructureGenerator& operator=(const InitialStructureGenerator&) = default;
				InitialStructureGenerator& operator=(InitialStructureGenerator&&) = default;

			// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				ModelCrystalStructure generate(const ModelAtomCollection&) const;

				void setGenerationType(const InitialStructureGenerationOptions::GenerationType) noexcept;
				void setInitialStructureGenerationOptions(const InitialStructureGenerationOptions&);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				InitialStructureGenerationOptions::GenerationType m_generationType;

				Internal::RandomStructureGenerator m_randomStructureGenerator;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

inline MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerator::InitialStructureGenerator()
	: m_generationType{ InitialStructureGenerationOptions::GenerationType::random }
	, m_randomStructureGenerator{}
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline MathematicalCrystalChemistry::StructureModel::Components::ModelCrystalStructure MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerator::generate(const ModelAtomCollection& modelAtomCollection) const
{
	if (m_generationType == InitialStructureGenerationOptions::GenerationType::random)
		return m_randomStructureGenerator.generate(modelAtomCollection);

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "generate", "Generation type is invalid."));
	}
}

inline void MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerator::setGenerationType(const InitialStructureGenerationOptions::GenerationType val) noexcept
{
	m_generationType = val;
}

inline void MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerator::setInitialStructureGenerationOptions(const InitialStructureGenerationOptions& initialStructureGenerationOptions)
{
	m_randomStructureGenerator.setRandomStructureGenerationParameters(initialStructureGenerationOptions.randomStructureGenerationParameters());
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_DESIGN_GENERATION_INITIALSTRUCTUREGENERATOR_H
