#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_INTERNAL_INTERATOMICCONSTRAINTBASE_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_INTERNAL_INTERATOMICCONSTRAINTBASE_H

#include <memory>
#include <functional>

#include "ReportException.h"
#include "LinearAlgebra.h"
#include "PeriodicReferenceWrapper.h"
#include "UnitCell.h"

#include "ModelAtom.h"
#include "ConstraintKey.h"


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace GeometricalConstraints
		{
			namespace Internal
			{
				class InteratomicConstraintBase
				{
				protected:
					using size_type = std::size_t;
					using NumericalVector = MathToolkit::Generic::NumericalVector<double>;

					using ModelAtom = MathematicalCrystalChemistry::StructureModel::Components::ModelAtom;
					using Ligand = ChemToolkit::Crystallography::PeriodicReferenceWrapper<ModelAtom, ChemToolkit::Crystallography::UnitCell>;
					using ConstraintKey = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::ConstraintKey;

// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Protected constructors and public destructor

				protected:
					InteratomicConstraintBase(ModelAtom&, const Ligand&);
					InteratomicConstraintBase(ModelAtom&, Ligand&&);
					InteratomicConstraintBase(const Ligand&, const Ligand&);
					explicit InteratomicConstraintBase(const std::shared_ptr<InteratomicConstraintBase>& reversePtr);

				public:
					virtual ~InteratomicConstraintBase() = default;

				// Protected constructors and public destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Public methods

					ModelAtom& modelAtomRef() const noexcept;
					const Ligand& ligand() const noexcept;

				// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Protected methods

				protected:
					std::shared_ptr<InteratomicConstraintBase> reversePtr() const;
					void setReversePtr(const std::shared_ptr<InteratomicConstraintBase>& reversePtr) noexcept;

				// Protected methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

				private:
					std::weak_ptr<InteratomicConstraintBase> m_reversePtr;
					std::reference_wrapper<ModelAtom> m_modelAtomRef;
					Ligand m_ligand;


				private:
					InteratomicConstraintBase() = delete;
					InteratomicConstraintBase(const InteratomicConstraintBase&) = delete;
					InteratomicConstraintBase(InteratomicConstraintBase&&) noexcept = delete;
					InteratomicConstraintBase& operator=(const InteratomicConstraintBase&) = delete;
					InteratomicConstraintBase& operator=(InteratomicConstraintBase&&) noexcept = delete;
				};
			}
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline MathematicalCrystalChemistry::StructureModel::Components::ModelAtom& MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::InteratomicConstraintBase::modelAtomRef() const noexcept
{
	return m_modelAtomRef.get();
}

inline const ChemToolkit::Crystallography::PeriodicReferenceWrapper<MathematicalCrystalChemistry::StructureModel::Components::ModelAtom, ChemToolkit::Crystallography::UnitCell>& MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::InteratomicConstraintBase::ligand() const noexcept
{
	return m_ligand;
}

// Public method
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Protected methods

inline std::shared_ptr<MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::InteratomicConstraintBase> MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::InteratomicConstraintBase::reversePtr() const
{
	return m_reversePtr.lock();
}

inline void MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::InteratomicConstraintBase::setReversePtr(const std::shared_ptr<InteratomicConstraintBase>& reversePtr) noexcept
{
	m_reversePtr = reversePtr;
}

// Protected methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_INTERNAL_INTERATOMICCONSTRAINTBASE_H
