#ifndef CHEMTOOLKIT_CRYSTALLOGRAPHY_SPACEGROUPS_INTERNATIONALTABLES_H
#define CHEMTOOLKIT_CRYSTALLOGRAPHY_SPACEGROUPS_INTERNATIONALTABLES_H

#include <stdexcept>
#include <string>

#include "ReportException.h"


namespace ChemToolkit
{
	namespace Crystallography
	{
		namespace SpaceGroups
		{
			class InternationalTables final
			{
				using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				InternationalTables() = default;
				~InternationalTables() = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				static double spaceGroupPrecision();
				static double getDefaultSpaceGroupPrecision();

				static void setSpaceGroupPrecision();
				static void setSpaceGroupPrecision(const double spaceGroupPrecision);

				static std::string transformIntoShortSymbol(const size_type spaceGroupNumber);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				static double m_spaceGroupPrecision;
				static double m_defaultSpaceGroupPrecision;


			private:
				InternationalTables(const InternationalTables&) = default;
				InternationalTables(InternationalTables&&) noexcept = default;
				InternationalTables& operator=(const InternationalTables&) = default;
				InternationalTables& operator=(InternationalTables&&) noexcept = default;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline double ChemToolkit::Crystallography::SpaceGroups::InternationalTables::spaceGroupPrecision()
{
	return m_spaceGroupPrecision;
}

inline double ChemToolkit::Crystallography::SpaceGroups::InternationalTables::getDefaultSpaceGroupPrecision()
{
	return m_defaultSpaceGroupPrecision;
}

inline void ChemToolkit::Crystallography::SpaceGroups::InternationalTables::setSpaceGroupPrecision()
{
	m_spaceGroupPrecision = m_defaultSpaceGroupPrecision;
}

inline void ChemToolkit::Crystallography::SpaceGroups::InternationalTables::setSpaceGroupPrecision(const double value)
{
	if (0.0 < value)
		m_spaceGroupPrecision = value;

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException("ChemToolkit::Crystallography::SpaceGroups::InternationalTables::setSpaceGroupPrecision", "\"spaceGroupPrecision\" is not more than zero."));
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !CHEMTOOLKIT_CRYSTALLOGRAPHY_SPACEGROUPS_INTERNATIONALTABLES_H
