#ifndef MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_INTERNAL_ISFEASIBLEWHOLESTRUCTURE_H
#define MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_INTERNAL_ISFEASIBLEWHOLESTRUCTURE_H

#include <limits>

#include "ModelCrystalStructure.h"

#include "WholeStructuralConstraints.h"


namespace MathematicalCrystalChemistry
{
	namespace Analysis
	{
		class CrystalAnalyzer;


		namespace Internal
		{
			class IsFeasibleWholeStructure
			{
				friend CrystalAnalyzer;

				using size_type = std::size_t;
				using AffineMapping = MathToolkit::Generic::AffineMapping<double>;
				using ModelAtom = MathematicalCrystalChemistry::StructureModel::Components::ModelAtom;
				using ModelCrystalStructure = MathematicalCrystalChemistry::StructureModel::Components::ModelCrystalStructure;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			private:
				IsFeasibleWholeStructure() = default;

			public:
				virtual ~IsFeasibleWholeStructure() = default;

				IsFeasibleWholeStructure(const IsFeasibleWholeStructure&) = default;
				IsFeasibleWholeStructure(IsFeasibleWholeStructure&&) noexcept = default;
				IsFeasibleWholeStructure& operator=(const IsFeasibleWholeStructure&) = default;
				IsFeasibleWholeStructure& operator=(IsFeasibleWholeStructure&&) noexcept = default;

				// Constructors, destructor, and operators
	// **********************************************************************************************************************************************************************************************************************************************************************************************
	// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Public methods

				bool operator()(const ModelCrystalStructure&) const;

				void setWholeStructuralConstraints(const WholeStructuralConstraints&);
				void setWholeStructuralConstraints(WholeStructuralConstraints&&);

				// Public methods
	// **********************************************************************************************************************************************************************************************************************************************************************************************
	// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Private methods

			private:
				bool hasFeasibleSpaceGroupNumber(ModelCrystalStructure) const;

				// Public methods
	// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				WholeStructuralConstraints m_wholeStructuralConstraints;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline bool MathematicalCrystalChemistry::Analysis::Internal::IsFeasibleWholeStructure::operator()(const ModelCrystalStructure& modelCrystalStructure) const
{
	if (!(hasFeasibleSpaceGroupNumber(modelCrystalStructure)))
		return false;

	return true;
}

inline void MathematicalCrystalChemistry::Analysis::Internal::IsFeasibleWholeStructure::setWholeStructuralConstraints(const WholeStructuralConstraints& wholeStructuralConstraints)
{
	m_wholeStructuralConstraints = wholeStructuralConstraints;
}

inline void MathematicalCrystalChemistry::Analysis::Internal::IsFeasibleWholeStructure::setWholeStructuralConstraints(WholeStructuralConstraints&& wholeStructuralConstraints)
{
	m_wholeStructuralConstraints = std::move(wholeStructuralConstraints);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline bool MathematicalCrystalChemistry::Analysis::Internal::IsFeasibleWholeStructure::hasFeasibleSpaceGroupNumber(ModelCrystalStructure modelCrystalStructure) const
{
	if (m_wholeStructuralConstraints.feasibleSpaceGroupNumbers().empty())
		return true;

	else
	{
		modelCrystalStructure.conventionalizeStructure();

		for (const auto& feasibleSpaceGroupNumber : m_wholeStructuralConstraints.feasibleSpaceGroupNumbers())
		{
			if (modelCrystalStructure.spaceGroupNumber() == feasibleSpaceGroupNumber)
				return true;
		}

		return false;
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_INTERNAL_ISFEASIBLEWHOLESTRUCTURE_H
