#ifndef OPERATINGSYSTEM_IO_LOGWRITER_H
#define OPERATINGSYSTEM_IO_LOGWRITER_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <mutex>

#include "ReportException.h"


namespace OperatingSystem
{
	namespace IO
	{
		class LogWriter final
		{
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Destructor

		public:
			LogWriter();
			explicit LogWriter(const std::filesystem::path& directoryPath);
			~LogWriter() = default;

			LogWriter(const LogWriter&) = default;
			LogWriter(LogWriter&&) noexcept = default;
			LogWriter& operator=(const LogWriter&) = default;
			LogWriter& operator=(LogWriter&&) noexcept = default;

		// Destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			bool isAvailable() const;
			void setDirectoryPath(const std::filesystem::path& directoryPath);

			void write(const std::string&) const;
			void write(const std::exception&) const;
			void write(const std::invalid_argument&) const;
			void write(const std::logic_error&) const;
			void write(const std::runtime_error&) const;
			void write(const std::out_of_range&) const;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			void createFile(const std::string& outputText = std::string{}) const;

			std::string writeHeaderMessage() const;
			void outputMessage(const std::string& message) const;

			static std::string getLogFilename() noexcept;

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			std::filesystem::path m_logFilePath;

			static std::mutex m_logMutex;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline bool OperatingSystem::IO::LogWriter::isAvailable() const
{
	return std::filesystem::is_regular_file(m_logFilePath);
}

inline void OperatingSystem::IO::LogWriter::setDirectoryPath(const std::filesystem::path& directoryPath)
{
	if (std::filesystem::is_directory(directoryPath))
	{
		m_logFilePath = directoryPath;
		m_logFilePath /= getLogFilename();


		std::lock_guard logLockGuard{ m_logMutex };

		if (!(std::filesystem::is_regular_file(m_logFilePath)))
			createFile();
	}

	else
	{
		std::string errorMessage;
		{
			errorMessage += "\"";
			errorMessage += directoryPath.generic_string();
			errorMessage += "\" is not a path to a directory.";
		}

		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "setLogFilePath", errorMessage));
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline std::string OperatingSystem::IO::LogWriter::getLogFilename() noexcept
{
	return std::string{ "log.txt" };
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !OPERATINGSYSTEM_IO_LOGWRITER_H
