#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_MINIMUMDISTANCECONSTRAINT_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_MINIMUMDISTANCECONSTRAINT_H

#include "InteratomicConstraintBase.h"


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace GeometricalConstraints
		{
			namespace Internal
			{
				class MinimumDistanceConstraint :public InteratomicConstraintBase
				{
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Protected constructors and public destructor

				protected:
					MinimumDistanceConstraint(ModelAtom&, const Ligand&);
					MinimumDistanceConstraint(ModelAtom&, Ligand&&);
					MinimumDistanceConstraint(const Ligand&, const Ligand&);
					explicit MinimumDistanceConstraint(const std::shared_ptr<MinimumDistanceConstraint>& twinPtr);

				public:
					virtual ~MinimumDistanceConstraint() = default;

				// Protected constructors and public destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Public methods

					virtual void initialize() noexcept;
					double minimumDistance() const noexcept;

					bool isFeasible(const double errorRate) const;

				// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Protected methods

				protected:
					void setMinimumDistance(const double minimumDistance);
					void synchronizeMinimumDistance();

				// Protected methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

				private:
					double m_minimumDistance;


				private:
					MinimumDistanceConstraint() = delete;
					MinimumDistanceConstraint(const MinimumDistanceConstraint&) = delete;
					MinimumDistanceConstraint(MinimumDistanceConstraint&&) noexcept = delete;
					MinimumDistanceConstraint& operator=(const MinimumDistanceConstraint&) = delete;
					MinimumDistanceConstraint& operator=(MinimumDistanceConstraint&&) noexcept = delete;
				};
			}
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline void MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::MinimumDistanceConstraint::initialize() noexcept
{
	m_minimumDistance = 0.0;
}

inline double MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::MinimumDistanceConstraint::minimumDistance() const noexcept
{
	return m_minimumDistance;
}

inline bool MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::MinimumDistanceConstraint::isFeasible(const double errorRate) const
{
	if (errorRate < 0.0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "isFeasible", "\"errorRate\" is less than zero."));
	}

	else
	{
		NumericalVector displacement = ligand().coordinate() - modelAtomRef().coordinate();
		double distanceSquare = displacement.normSquare();
		double minBondLength = (1.0 - errorRate) * m_minimumDistance;

		if (distanceSquare < (minBondLength * minBondLength))
			return false;
		else
			return true;
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Protected methods

inline void MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::MinimumDistanceConstraint::setMinimumDistance(const double minimumDistance)
{
	if (minimumDistance < 0.0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "setMinimumDistance", "\"minimumDistance\" is less than zero."));
	}

	else
		m_minimumDistance = minimumDistance;
}

inline void MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal::MinimumDistanceConstraint::synchronizeMinimumDistance()
{
	std::shared_ptr<MinimumDistanceConstraint> reverseMinimumDistanceConstraintPtr = std::dynamic_pointer_cast<MinimumDistanceConstraint>(reversePtr());
	reverseMinimumDistanceConstraintPtr->m_minimumDistance = m_minimumDistance;
}

// Protected methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_GEOMETRICALCONSTRAINTS_MINIMUMDISTANCECONSTRAINT_H
