#ifndef MATHEMATICALCRYSTALCHEMISTRY_GENERIC_MODELCHEMICALELEMENTS_H
#define MATHEMATICALCRYSTALSCHEMISTRY_GENERIC_MODELCHEMICALELEMENTS_H

#include <stdexcept>
#include <string>
#include <vector>
#include <map>

#include "AtomCategory.h"
#include "CoordinationFeature.h"
#include "CoordinationFeatureSymbol.h"
#include "OxidationFeature.h"


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace ChemicalElements
		{
			class ModelChemicalElement
			{
				using size_type = std::size_t;

				using AtomCategory = MathematicalCrystalChemistry::StructureModel::Features::AtomCategory;
				using CoordinationFeature = MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeature;
				using CoordinationFeatureSymbol = MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol;
				using OxidationFeature = MathematicalCrystalChemistry::StructureModel::Features::OxidationFeature;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				explicit ModelChemicalElement(const std::string& elementSymbol);
				virtual ~ModelChemicalElement() = default;

				ModelChemicalElement(const ModelChemicalElement&) = default;
				ModelChemicalElement(ModelChemicalElement&&) noexcept = default;
				ModelChemicalElement& operator=(const ModelChemicalElement&) = default;
				ModelChemicalElement& operator=(ModelChemicalElement&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				std::string elementSymbol() const;
				AtomCategory atomCategory() const;
				const OxidationFeature& oxidationFeature() const;

				std::vector<CoordinationFeature> getFeasibleCoordinationFeatures(const size_type oxidationState) const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Static methods

			private:
				static std::vector<CoordinationFeatureSymbol> getFeasibleCoordinationFeatureSymbols(std::string elementSymbol, const size_type oxidationState);

			// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::string m_elementSymbol;

				AtomCategory m_atomCategory;
				std::map<size_type, std::vector<CoordinationFeature>> m_feasibleCoordinationFeatures;
				OxidationFeature m_oxidationFeature;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline std::string MathematicalCrystalChemistry::StructureModel::ChemicalElements::ModelChemicalElement::elementSymbol() const
{
	return m_elementSymbol;
}

inline MathematicalCrystalChemistry::StructureModel::Features::AtomCategory MathematicalCrystalChemistry::StructureModel::ChemicalElements::ModelChemicalElement::atomCategory() const
{
	return m_atomCategory;
}

inline const MathematicalCrystalChemistry::StructureModel::Features::OxidationFeature& MathematicalCrystalChemistry::StructureModel::ChemicalElements::ModelChemicalElement::oxidationFeature() const
{
	return m_oxidationFeature;
}

inline std::vector<MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeature> MathematicalCrystalChemistry::StructureModel::ChemicalElements::ModelChemicalElement::getFeasibleCoordinationFeatures(const size_type oxidationState) const
{
	auto iter = m_feasibleCoordinationFeatures.find(oxidationState);

	if (iter != m_feasibleCoordinationFeatures.end())
		return iter->second;
	else
		std::vector<MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeature>();
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods


// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************



#endif // !MATHEMATICALCRYSTALCHEMISTRY_GENERIC_MODELCHEMICALELEMENTS_H
