#ifndef OPERATINGSYSTEM_CONFIGURATION_MPICONTROLLER_H
#define OPERATINGSYSTEM_CONFIGURATION_MPICONTROLLER_H

#include <string>
#include <stdexcept>

#include "ReportException.h"


namespace OperatingSystem
{
	namespace Configuration
	{
		class MpiController final
		{
			using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		private:
			MpiController() = default;

		public:
			~MpiController() = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

		public:
			static size_type mpiRank() noexcept;
			static size_type mpiProcessing() noexcept;

			static bool isMpiDirectoryName(const std::string& directoryName) noexcept;
			static std::string getMpiDirectoryName() noexcept;

			static void setMpiRank(const std::size_t mpiRank) noexcept;
			static void setMpiProcessing(const std::size_t mpiProcessing);

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			static std::size_t m_mpiRank;
			static std::size_t m_mpiProcessing;


		private:
			MpiController(const MpiController&) = delete;
			MpiController(MpiController&&) = delete;
			MpiController& operator=(const MpiController&) = delete;
			MpiController& operator=(MpiController&&) = delete;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::size_t OperatingSystem::Configuration::MpiController::mpiRank() noexcept
{
	return m_mpiRank;
}

inline std::size_t OperatingSystem::Configuration::MpiController::mpiProcessing() noexcept
{
	return m_mpiProcessing;
}

inline std::string OperatingSystem::Configuration::MpiController::getMpiDirectoryName() noexcept
{
	std::string directoryName;
	{
		directoryName += "MPI_";
		directoryName += std::to_string(m_mpiRank);
	}

	return directoryName;
}

inline void OperatingSystem::Configuration::MpiController::setMpiRank(const std::size_t value) noexcept
{
	m_mpiRank = value;
}

inline void OperatingSystem::Configuration::MpiController::setMpiProcessing(const std::size_t value)
{
	if (0 < value)
		m_mpiProcessing = value;

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException("setMpiProcessing of OperatingSystem::Configuration::MpiController", "\"mpiProcessing\" is not more than zero."));
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !OPERATINGSYSTEM_CONFIGURATION_MPICONTROLLER_H
