#ifndef MATHTOOLKIT_GENERIC_NUMERICALSYMMETRICMATRIX_H
#define MATHTOOLKIT_GENERIC_NUMERICALSYMMETRICMATRIX_H

#include <initializer_list>
#include <utility>
#include <vector>

#include "NumericalSymmetricMatrixBase.h"
#include "NumericalVector.h"
#include "NumericalVectorSlice.h"


namespace MathToolkit
{
	namespace Generic
	{
		/*
		Column Major Layout for m_columnSize-by-m_rowSize matrix
		*/

		class NumericalSymmetricMatrix :public MathToolkit::Generic::Internal::NumericalSymmetricMatrixBase
		{
		public:
			using size_type = std::size_t;
			using value_type = double;
			using pointer = value_type*;
			using const_pointer = const value_type*;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, operators, and destructor

		public:
			NumericalSymmetricMatrix() = default;
			NumericalSymmetricMatrix(const size_type dimension, const value_type value = value_type{});

			NumericalSymmetricMatrix(const NumericalSymmetricMatrix&) = default;
			NumericalSymmetricMatrix(NumericalSymmetricMatrix&&) noexcept = default;
			NumericalSymmetricMatrix& operator=(const NumericalSymmetricMatrix&) = default;
			NumericalSymmetricMatrix& operator=(NumericalSymmetricMatrix&&) noexcept = default;
			NumericalSymmetricMatrix& operator=(const value_type scalar);

			virtual ~NumericalSymmetricMatrix() = default;

		// Constructors, operators, and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Unary operators

			NumericalSymmetricMatrix& operator+=(const NumericalSymmetricMatrix&);
			NumericalSymmetricMatrix& operator-=(const NumericalSymmetricMatrix&);
			NumericalSymmetricMatrix& operator*=(const value_type scalingMultiplier);
			NumericalSymmetricMatrix& operator/=(const value_type scalingDivider);

		// Unary operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Mathematical methods

			NumericalSymmetricMatrix& add(const value_type scalingMultiplier, const NumericalSymmetricMatrix&);
			NumericalSymmetricMatrix& add(const value_type scalingMultiplier, const NumericalSymmetricMatrix&, const value_type selfScalingMultiplier);

			NumericalSymmetricMatrix& invert();
			NumericalSymmetricMatrix getInverseMatrix() const;

			std::vector<NumericalVector<value_type>> getEigenVectors() const;
			std::vector<std::pair<value_type, NumericalVector<value_type>>> getEigenValueAndVectors() const;

		// Mathematical methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		};


		inline NumericalSymmetricMatrix getIdentitySymmetricMatrix(const std::size_t rank)
		{
			NumericalSymmetricMatrix identityMatrix(rank);
			{
				for (std::size_t index = 0; index < rank; ++index)
					identityMatrix(index, index) = 1.0;
			}

			return identityMatrix;
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors and operators

inline MathToolkit::Generic::NumericalSymmetricMatrix::NumericalSymmetricMatrix(const size_type dimension, const value_type value)
	: MathToolkit::Generic::Internal::NumericalSymmetricMatrixBase{ dimension, value }
{
}

inline MathToolkit::Generic::NumericalSymmetricMatrix& MathToolkit::Generic::NumericalSymmetricMatrix::operator=(const value_type scalar)
{
	this->m_valueArray = scalar;
	return *this;
}

// Constructors and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Unary operators

inline MathToolkit::Generic::NumericalSymmetricMatrix& MathToolkit::Generic::NumericalSymmetricMatrix::operator+=(const NumericalSymmetricMatrix& numericalMatrix)
{
	this->m_valueArray += numericalMatrix.m_valueArray;
	return *this;
}

inline MathToolkit::Generic::NumericalSymmetricMatrix& MathToolkit::Generic::NumericalSymmetricMatrix::operator-=(const NumericalSymmetricMatrix& numericalMatrix)
{
	this->m_valueArray -= numericalMatrix.m_valueArray;
	return *this;
}

inline MathToolkit::Generic::NumericalSymmetricMatrix& MathToolkit::Generic::NumericalSymmetricMatrix::operator*=(const value_type scalingMultiplier)
{
	this->m_valueArray *= scalingMultiplier;
	return *this;
}

inline MathToolkit::Generic::NumericalSymmetricMatrix& MathToolkit::Generic::NumericalSymmetricMatrix::operator/=(const value_type scalingDivider)
{
	const value_type scalingMultiplier = (1.0 / scalingDivider);
	this->m_valueArray *= scalingMultiplier;
	return *this;
}

// Unary operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Mathematical methods

inline MathToolkit::Generic::NumericalSymmetricMatrix& MathToolkit::Generic::NumericalSymmetricMatrix::add(const value_type scalingMultiplier, const NumericalSymmetricMatrix& numericalMatrix)
{
	auto iter = std::begin(numericalMatrix.m_valueArray);

	for (auto& value : this->m_valueArray)
	{
		value += (*iter) * scalingMultiplier;
		++iter;
	}

	return *this;
}

inline MathToolkit::Generic::NumericalSymmetricMatrix& MathToolkit::Generic::NumericalSymmetricMatrix::add(const value_type scalingMultiplier, const NumericalSymmetricMatrix& numericalMatrix, const value_type selfScalingMultiplier)
{
	auto iter = std::begin(numericalMatrix.m_valueArray);

	for (auto& value : this->m_valueArray)
	{
		value *= selfScalingMultiplier;
		value += (*iter) * scalingMultiplier;
		++iter;
	}

	return *this;
}

inline MathToolkit::Generic::NumericalSymmetricMatrix MathToolkit::Generic::NumericalSymmetricMatrix::getInverseMatrix() const
{
	NumericalSymmetricMatrix inverseMatrix{ *this };
	inverseMatrix.invert();
	return inverseMatrix;
}

inline std::vector<MathToolkit::Generic::NumericalVector<double>> MathToolkit::Generic::NumericalSymmetricMatrix::getEigenVectors() const
{
	std::vector<MathToolkit::Generic::NumericalVector<value_type>> eigenVectors;
	{
		for (const auto& eigenvalueAndVector : getEigenValueAndVectors())
			eigenVectors.push_back(eigenvalueAndVector.second);
	}

	return std::vector<NumericalVector<value_type>>{};
}

// Mathematical methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !MATHTOOLKIT_GENERIC_NUMERICALSYMMETRICMATRIX_H
