#ifndef MATHTOOLKIT_GENERIC_NUMERICALVECTOR_H
#define MATHTOOLKIT_GENERIC_NUMERICALVECTOR_H

#include <cmath>

#include "NumericalVectorBase.h"
#include "NumericalMatrixBase.h"
#include "NumericalVectorSlice.h"


namespace MathToolkit
{
	namespace Generic
	{
		template <typename T>
		class NumericalMatrix;

		template <typename T>
		class NumericalVector :public MathToolkit::Generic::Internal::NumericalVectorBase<T>
		{
			friend NumericalMatrix<T>;
			using NumericalMatrixBase = MathToolkit::Generic::Internal::NumericalMatrixBase<T>;

		public:
			using size_type = std::size_t;
			using value_type = double;
			using pointer = value_type*;
			using const_pointer = const value_type*;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operators

		public:
			NumericalVector();
			NumericalVector(std::initializer_list<value_type>);
			NumericalVector(const size_type, const value_type value = value_type{});
			explicit NumericalVector(const NumericalVectorSlice<T>&);

			template <typename U>
			NumericalVector(const NumericalVector<U>);

			NumericalVector(const NumericalVector&);
			NumericalVector(NumericalVector&&) noexcept = default;
			NumericalVector& operator=(const NumericalVector&);
			NumericalVector& operator=(NumericalVector&&) noexcept = default;
			NumericalVector& operator=(const NumericalVectorSlice<T>&);
			NumericalVector& operator=(const value_type scalar);

			template <typename U>
			NumericalVector& operator=(const NumericalVector<U>);

			virtual ~NumericalVector() = default;

		// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Unary operators

			NumericalVector& operator+=(const NumericalVector&);
			NumericalVector& operator-=(const NumericalVector&);
			NumericalVector& operator*=(const value_type scalingMultiplier);
			NumericalVector& operator/=(const value_type scalingDivider);

		// Unary operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Arithmetic methods

			NumericalVector& add(const value_type scalingMultiplier, const NumericalVector&);
			NumericalVector& add(const value_type scalingMultiplier, const NumericalVector&, const value_type selfScalingMultiplier);
			NumericalVector& add(const NumericalMatrixBase&, const NumericalVector&);
			NumericalVector& transform(const NumericalMatrixBase&);

		// Arithmetic methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Utility

			double norm() const;
			double normSquare() const;

		// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors and operators

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>::NumericalVector()
	: MathToolkit::Generic::Internal::NumericalVectorBase<T>{}
{
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>::NumericalVector(std::initializer_list<value_type> init)
	: MathToolkit::Generic::Internal::NumericalVectorBase<T>(init)
{
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>::NumericalVector(const size_type size, const value_type value)
	: MathToolkit::Generic::Internal::NumericalVectorBase<T>(size, value)
{
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>::NumericalVector(const NumericalVectorSlice<T>& numericalVectorSlice)
	: MathToolkit::Generic::Internal::NumericalVectorBase<T>{}
{
	this->m_valueArray.resize(numericalVectorSlice.size());
	auto thisIter = std::begin(this->m_valueArray);

	for (size_type index = 0; index < numericalVectorSlice.size(); ++index)
	{
		(*thisIter) = numericalVectorSlice[index];
		++thisIter;
	}
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>::NumericalVector(const NumericalVector& numericalVector)
	: MathToolkit::Generic::Internal::NumericalVectorBase<T>(dynamic_cast<const MathToolkit::Generic::Internal::NumericalVectorBase<T>&>(numericalVector))
{
}

template <typename T>
template <typename U>
inline MathToolkit::Generic::NumericalVector<T>::NumericalVector(const NumericalVector<U> nv)
	: MathToolkit::Generic::Internal::NumericalVectorBase<T>(nv)
{
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::operator=(const NumericalVector& numericalVector)
{
	MathToolkit::Generic::Internal::NumericalVectorBase<T>::operator=(dynamic_cast<const MathToolkit::Generic::Internal::NumericalVectorBase<T>&>(numericalVector));
	return *this;
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::operator=(const value_type scalar)
{
	this->m_valueArray = scalar;
	return *this;
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::operator=(const NumericalVectorSlice<T>& numericalVectorSlice)
{
	this->m_valueArray.resize(numericalVectorSlice.size());
	auto thisIter = std::begin(this->m_valueArray);
	{
		for (size_type index = 0; index < numericalVectorSlice.size(); ++index)
		{
			(*thisIter) = numericalVectorSlice[index];
			++thisIter;
		}
	}

	return *this;
}

template <typename T>
template <typename U>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::operator=(const NumericalVector<U> nv)
{
	MathToolkit::Generic::Internal::NumericalVectorBase<T>::operator=(dynamic_cast<const MathToolkit::Generic::Internal::NumericalVectorBase<T>&>(nv));
	return *this;
}

// Constructors and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Unary operators

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::operator+=(const NumericalVector& numericalVector)
{
	this->m_valueArray += numericalVector.m_valueArray;
	return *this;
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::operator-=(const NumericalVector& numericalVector)
{
	this->m_valueArray -= numericalVector.m_valueArray;
	return *this;
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::operator*=(const value_type scalingMultiplier)
{
	this->m_valueArray *= scalingMultiplier;
	return *this;
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::operator/=(const value_type scalingDivider)
{
	const value_type scalingMultiplier = (1.0 / scalingDivider);
	this->m_valueArray *= scalingMultiplier;
	return *this;
}

// Unary operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Arithmetic methods

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::add(const value_type scalingMultiplier, const NumericalVector& numericalVector)
{
	auto iter = std::begin(numericalVector.m_valueArray);
	{
		for (auto& value : this->m_valueArray)
		{
			value += (*iter) * scalingMultiplier;
			++iter;
		}
	}

	return *this;
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::add(const value_type scalingMultiplier, const NumericalVector& numericalVector, const value_type selfScalingMultiplier)
{
	auto iter = std::begin(numericalVector.m_valueArray);
	{
		for (auto& value : this->m_valueArray)
		{
			value *= selfScalingMultiplier;
			value += (*iter) * scalingMultiplier;
			++iter;
		}
	}

	return *this;
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::add(const NumericalMatrixBase& numericalMatrixBase, const NumericalVector& numericalVector)
{
	auto matrixIter = numericalMatrixBase.begin();
	{
		for (const auto val : numericalVector)
		{
			auto thisIter = std::begin(this->m_valueArray);

			for (size_type rowIndex = 0; rowIndex < numericalMatrixBase.columnSize(); ++rowIndex)
			{
				(*thisIter) += (*matrixIter) * val;
				++thisIter;
				++matrixIter;
			}
		}
	}

	return *this;
}

template <typename T>
inline MathToolkit::Generic::NumericalVector<T>& MathToolkit::Generic::NumericalVector<T>::transform(const NumericalMatrixBase& numericalMatrixBase)
{
	std::valarray<value_type> newValueArray(0.0, numericalMatrixBase.columnSize());
	{
		auto matrixIter = numericalMatrixBase.begin();
		{
			for (const auto val : this->m_valueArray)
			{
				auto newIter = std::begin(newValueArray);

				for (size_type rowIndex = 0; rowIndex < numericalMatrixBase.columnSize(); ++rowIndex)
				{
					(*newIter) += (*matrixIter) * val;
					++newIter;
					++matrixIter;
				}
			}
		}
	}

	this->m_valueArray.swap(newValueArray);
	return *this;
}

// Arithmetic methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Utility

template <typename T>
inline double MathToolkit::Generic::NumericalVector<T>::norm() const
{
	return std::sqrt(normSquare());
}

template <>
inline double MathToolkit::Generic::NumericalVector<double>::normSquare() const
{
	double result{ 0.0 };
	{
		for (const auto value : this->m_valueArray)
			result += (value * value);
	}

	return result;
}


template <>
inline double MathToolkit::Generic::NumericalVector<std::complex<double>>::normSquare() const
{
	double result{ 0.0 };
	{
		for (const auto value : this->m_valueArray)
		{
			result += (value.real() * value.real());
			result += (value.imag() * value.imag());
		}
	}

	return result;
}

// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !MATHTOOLKIT_GENERIC_NUMERICALVECTOR_H
