#ifndef MATHTOOLKIT_GENERIC_INTERNAL_NUMERICALVECTORBASE_H
#define MATHTOOLKIT_GENERIC_INTERNAL_NUMERICALVECTORBASE_H

#include <string>
#include <stdexcept>
#include <type_traits>
#include <algorithm>
#include <complex>
#include <valarray>

#include "ReportException.h"


namespace MathToolkit
{
	namespace Generic
	{
		namespace Internal
		{
			template <typename T>
			class NumericalMatrixBase;

			template <typename T>
			class NumericalVectorBase
			{
				friend NumericalMatrixBase<T>;

				using size_type = std::size_t;
				using value_type = T;
				using pointer = value_type*;
				using const_pointer = const value_type*;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			protected:
				NumericalVectorBase();
				NumericalVectorBase(std::initializer_list<value_type>);
				NumericalVectorBase(const size_type, const value_type);

			public:
				NumericalVectorBase(const NumericalVectorBase&) = default;
				NumericalVectorBase(NumericalVectorBase&&) noexcept = default;
				NumericalVectorBase& operator=(const NumericalVectorBase&) = default;
				NumericalVectorBase& operator=(NumericalVectorBase&&) noexcept = default;

				virtual ~NumericalVectorBase() = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				value_type operator[](const size_type index) const;
				value_type& operator[](const size_type index);
				value_type at(const size_type index) const;
				value_type& at(const size_type index);

				pointer begin();
				const_pointer begin() const;
				pointer end();
				const_pointer end() const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Utility

				bool isEmpty() const;
				size_type size() const;

			// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Customize methods

				void swap(NumericalVectorBase&);
				void resize(const size_type, const value_type value = value_type{});
				void clear();

				void pushBack(const value_type value);
				void pushBack(const NumericalVectorBase&);

			// Customize methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			protected:
				std::valarray<value_type> m_valueArray;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

template <typename T>
inline T MathToolkit::Generic::Internal::NumericalVectorBase<T>::operator[](const size_type index) const
{
	return m_valueArray[index];
}

template <typename T>
inline T& MathToolkit::Generic::Internal::NumericalVectorBase<T>::operator[](const size_type index)
{
	return m_valueArray[index];
}

template <typename T>
inline T MathToolkit::Generic::Internal::NumericalVectorBase<T>::at(size_type index) const
{
	if (index < m_valueArray.size())
		return m_valueArray[index];

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::out_of_range(reportException(typeid(*this), "at", "The \"index\" is out of range."));
	}
}

template <typename T>
inline T& MathToolkit::Generic::Internal::NumericalVectorBase<T>::at(size_type index)
{
	if (index < m_valueArray.size())
		return m_valueArray[index];

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::out_of_range(reportException(typeid(*this), "at", "The \"index\" is out of range."));
	}
}

template <typename T>
inline T* MathToolkit::Generic::Internal::NumericalVectorBase<T>::begin()
{
	return std::begin(m_valueArray);
}

template <typename T>
inline const T* MathToolkit::Generic::Internal::NumericalVectorBase<T>::begin() const
{
	return std::begin(m_valueArray);
}

template <typename T>
inline T* MathToolkit::Generic::Internal::NumericalVectorBase<T>::end()
{
	return std::end(m_valueArray);
}

template <typename T>
inline const T* MathToolkit::Generic::Internal::NumericalVectorBase<T>::end() const
{
	return std::end(m_valueArray);
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Utility

template <typename T>
inline bool MathToolkit::Generic::Internal::NumericalVectorBase<T>::isEmpty() const
{
	return !(static_cast<bool>(m_valueArray.size()));
}

template <typename T>
inline std::size_t MathToolkit::Generic::Internal::NumericalVectorBase<T>::size() const
{
	return m_valueArray.size();
}

// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Customize methods

template <typename T>
inline void MathToolkit::Generic::Internal::NumericalVectorBase<T>::swap(NumericalVectorBase& numericalVectorBase)
{
	m_valueArray.swap(numericalVectorBase.m_valueArray);
}

template <typename T>
inline void MathToolkit::Generic::Internal::NumericalVectorBase<T>::resize(const size_type size, const value_type value)
{
	m_valueArray.resize(size, value);
}

template <typename T>
inline void MathToolkit::Generic::Internal::NumericalVectorBase<T>::clear()
{
	std::valarray<value_type> va;
	m_valueArray.swap(va);
}

template <typename T>
inline void MathToolkit::Generic::Internal::NumericalVectorBase<T>::pushBack(const value_type value)
{
	std::valarray<value_type> newValueArray(m_valueArray.size() + 1);
	{
		std::copy(std::begin(m_valueArray), std::end(m_valueArray), std::begin(newValueArray));
		newValueArray[m_valueArray.size()] = value;
	}

	m_valueArray.swap(newValueArray);
}

template <typename T>
inline void MathToolkit::Generic::Internal::NumericalVectorBase<T>::pushBack(const NumericalVectorBase& numericalVectorBase)
{
	std::valarray<value_type> newValueArray(m_valueArray.size() + numericalVectorBase.size());
	{
		std::copy(std::begin(m_valueArray), std::end(m_valueArray), std::begin(newValueArray));
		std::copy(std::begin(numericalVectorBase), std::end(numericalVectorBase), (std::begin(newValueArray) + m_valueArray.size()));
	}

	m_valueArray.swap(newValueArray);
}

// Customize methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !MATHTOOLKIT_GENERIC_INTERNAL_NUMERICALVECTORBASE_H
