#ifndef MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_OXIDATIONFEATURE_H
#define MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_OXIDATIONFEATURE_H

#include <iterator>
#include <limits>
#include <map>
#include <string>
#include <set>


namespace MathematicalCrystalChemistry
{
	namespace StructureModel
	{
		namespace Features
		{
			class OxidationFeature
			{
				using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				explicit OxidationFeature(const std::string& elementSymbol);
				virtual ~OxidationFeature() = default;

				OxidationFeature(const OxidationFeature&) = default;
				OxidationFeature(OxidationFeature&&) noexcept = default;
				OxidationFeature& operator=(const OxidationFeature&) = default;
				OxidationFeature& operator=(OxidationFeature&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				const std::set<int>& feasibleOxidationStates() const;
				const std::map<int, double>& ionizationPotentials() const;
				double electronegativity() const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				double getIonizationPotential(const int oxidationState) const;

				int minOxidationState() const;
				int maxOxidationState() const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				static std::set<int> getFeasibleOxidationStates(const std::string& elementSymbol);
				static std::map<int, double> getIonizationPotentials(const std::string& elementSymbol);
				static double getElectronegativity(const std::string& elementSymbol);

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::set<int> m_feasibleOxidationStates;
				std::map<int, double> m_ionizationPotentials;
				double m_electronegativity;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const std::set<int>& MathematicalCrystalChemistry::StructureModel::Features::OxidationFeature::feasibleOxidationStates() const
{
	return m_feasibleOxidationStates;
}

inline const std::map<int, double>& MathematicalCrystalChemistry::StructureModel::Features::OxidationFeature::ionizationPotentials() const
{
	return m_ionizationPotentials;
}

inline double MathematicalCrystalChemistry::StructureModel::Features::OxidationFeature::electronegativity() const
{
	return m_electronegativity;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline double MathematicalCrystalChemistry::StructureModel::Features::OxidationFeature::getIonizationPotential(const int oxidationState) const
{
	if (m_ionizationPotentials.empty())
		return std::numeric_limits<double>::max();

	else
	{
		auto iter = m_ionizationPotentials.find(oxidationState + 1);

		if (iter == m_ionizationPotentials.end())
			return std::numeric_limits<double>::max();
		else
			return iter->second;
	}
}

inline int MathematicalCrystalChemistry::StructureModel::Features::OxidationFeature::minOxidationState() const
{
	auto iter = m_feasibleOxidationStates.begin();
	return (*iter);
}

inline int MathematicalCrystalChemistry::StructureModel::Features::OxidationFeature::maxOxidationState() const
{
	auto iter = std::prev(m_feasibleOxidationStates.end());
	return (*iter);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_STRUCTUREMODEL_FEATURES_OXIDATIONFEATURE_H
