#ifndef MATHEMATICALCRYSTALCHEMISTRY_DESIGN_GENERATION_INTERNAL_RANDOMSTRUCTUREGENERATOR_H
#define MATHEMATICALCRYSTALCHEMISTRY_DESIGN_GENERATION_INTERNAL_RANDOMSTRUCTUREGENERATOR_H

#include "RandomValueGenerator.h"

#include "UnitCell.h"
#include "ModelAtom.h"
#include "ModelAtomCollection.h"
#include "ModelCrystalStructure.h"

#include "RandomStructureGenerationParameters.h"


namespace MathematicalCrystalChemistry
{
	namespace Design
	{
		namespace Generation
		{
			namespace Internal
			{
				class RandomStructureGenerator
				{
					using UnitCell = ChemToolkit::Crystallography::UnitCell;
					using ModelAtom = MathematicalCrystalChemistry::StructureModel::Components::ModelAtom;
					using ModelCrystalStructure = MathematicalCrystalChemistry::StructureModel::Components::ModelCrystalStructure;

					using RandomValueGenerator = MathToolkit::Numerics::RandomValueGenerator;
					using ModelAtomCollection = MathematicalCrystalChemistry::StructureModel::Components::ModelAtomCollection;

// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Constructors, destructor, and operators

				public:
					RandomStructureGenerator();
					explicit RandomStructureGenerator(const RandomStructureGenerationParameters&);
					virtual ~RandomStructureGenerator() = default;

					RandomStructureGenerator(const RandomStructureGenerator&) = default;
					RandomStructureGenerator(RandomStructureGenerator&&) = default;
					RandomStructureGenerator& operator=(const RandomStructureGenerator&) = default;
					RandomStructureGenerator& operator=(RandomStructureGenerator&&) = default;

				// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Public methods

					ModelCrystalStructure generate(const ModelAtomCollection&) const;

					void setRandomStructureGenerationParameters(const RandomStructureGenerationParameters&);

				// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Private methods

				private:
					UnitCell generateUnitCell() const;
					void distributeAtoms(ModelCrystalStructure&) const;

				// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

				private:
					RandomValueGenerator m_randomValueGenerator;
					RandomStructureGenerationParameters m_randomStructureGenerationParameters;
				};
			}
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline MathematicalCrystalChemistry::StructureModel::Components::ModelCrystalStructure MathematicalCrystalChemistry::Design::Generation::Internal::RandomStructureGenerator::generate(const ModelAtomCollection& modelAtomCollection) const
{
	ModelCrystalStructure modelCrystalStructure{ generateUnitCell() };
	{
		for (const auto& modelAtom : modelAtomCollection)
			modelCrystalStructure.atomicArrangement().push_back(modelAtom);

		distributeAtoms(modelCrystalStructure);
		modelCrystalStructure.expandStructure(m_randomStructureGenerationParameters.initialPackingFraction());
	}

	return modelCrystalStructure;
}

inline void MathematicalCrystalChemistry::Design::Generation::Internal::RandomStructureGenerator::setRandomStructureGenerationParameters(const RandomStructureGenerationParameters& randomStructureGenerationParameters)
{
	m_randomStructureGenerationParameters = randomStructureGenerationParameters;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline void MathematicalCrystalChemistry::Design::Generation::Internal::RandomStructureGenerator::distributeAtoms(ModelCrystalStructure& modelCrystalStructure) const
{
	for (auto& modelAtom : modelCrystalStructure.atomicArrangement())
	{
		MathToolkit::Generic::NumericalVector<double> fractionalCoordinate(3);
		{
			fractionalCoordinate[0] = m_randomValueGenerator.getUniformDistributedValue(0.0, 1.0);
			fractionalCoordinate[1] = m_randomValueGenerator.getUniformDistributedValue(0.0, 1.0);
			fractionalCoordinate[2] = m_randomValueGenerator.getUniformDistributedValue(0.0, 1.0);
		}

		modelAtom.coordinate() = modelCrystalStructure.unitCell().basisVectors() * fractionalCoordinate;
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !MATHEMATICALCRYSTALCHEMISTRY_DESIGN_GENERATION_INTERNAL_RANDOMSTRUCTUREGENERATOR_H
