#ifndef MATHTOOLKIT_NUMERICS_RANDOMVALUEGENERATOR_H
#define MATHTOOLKIT_NUMERICS_RANDOMVALUEGENERATOR_H

#include <string>
#include <stdexcept>
#include <random>

#include "ReportException.h"


namespace MathToolkit
{
	namespace Numerics
	{
		class RandomValueGenerator
		{
			using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operators

		public:
			RandomValueGenerator();
			virtual ~RandomValueGenerator() = default;

			RandomValueGenerator(const RandomValueGenerator&);
			RandomValueGenerator(RandomValueGenerator&&) noexcept;
			RandomValueGenerator& operator=(const RandomValueGenerator&);
			RandomValueGenerator& operator=(RandomValueGenerator&&) noexcept;

		// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Access methods

			void setMaxRandomValueGenerating() noexcept;
			void setMaxRandomValueGenerating(const size_type);

			static size_type defaultMaxRandomValueGenerating();

		// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			double getUniformDistributedValue(const double minValue, const double maxValue) const;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			size_type m_maxRandomValueGenerating;

			mutable std::random_device m_seedGenerator;
			mutable std::default_random_engine m_engine{};
			mutable std::uniform_real_distribution<> m_distributor{};
			mutable size_type m_randomValueGenerating{ 0 };

			static size_type m_defaultMaxRandomValueGenerating;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline void MathToolkit::Numerics::RandomValueGenerator::setMaxRandomValueGenerating() noexcept
{
	m_maxRandomValueGenerating = m_defaultMaxRandomValueGenerating;
}

inline void MathToolkit::Numerics::RandomValueGenerator::setMaxRandomValueGenerating(const size_type val)
{
	if (0 < val)
		m_maxRandomValueGenerating = val;
	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "setMaxRandomValueGenerating", "\"maxRandomValueGenerating\" is zero."));
	}
}

inline std::size_t MathToolkit::Numerics::RandomValueGenerator::defaultMaxRandomValueGenerating()
{
	return m_defaultMaxRandomValueGenerating;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline double MathToolkit::Numerics::RandomValueGenerator::getUniformDistributedValue(const double minValue, const double maxValue) const
{
	if (m_randomValueGenerating < m_maxRandomValueGenerating)
		return minValue + (maxValue - minValue) * m_distributor(m_engine);

	else
	{
		m_engine = std::default_random_engine(m_seedGenerator());
		m_randomValueGenerating = 0;

		return minValue + (maxValue - minValue) * m_distributor(m_engine);
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHTOOLKIT_NUMERICS_RANDOMVALUEGENERATOR_H
