#ifndef OPERATINGSYSTEM_CONFIGURATION_REPORTEXCEPTION_H
#define OPERATINGSYSTEM_CONFIGURATION_REPORTEXCEPTION_H

#include <string>
#include <typeinfo>


namespace OperatingSystem
{
	namespace Configuration
	{
		class ReportException
		{
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		public:
			ReportException() = default;
			virtual ~ReportException() = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			std::string operator()(const std::string& information) const;
			std::string operator()(const std::string& functionName, const std::string& information) const;
			std::string operator()(const std::type_info&, const std::string& functionName, const std::string& information) const;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			std::string writeErrorLocation(const std::string& functionName) const;
			std::string writeErrorLocation(const std::type_info&, const std::string& functionName) const;

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************


		private:
			ReportException(const ReportException&) = delete;
			ReportException(ReportException&&) = delete;
			ReportException& operator=(const ReportException&) = delete;
			ReportException& operator=(ReportException&&) = delete;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::string OperatingSystem::Configuration::ReportException::operator()(const std::string& information) const
{
	std::string message;
	{
		message += information;
		message += "\n";
	}

	return message;
}

inline std::string OperatingSystem::Configuration::ReportException::operator()(const std::string& functionName, const std::string& information) const
{
	std::string message;
	{
		message += information;
		message += "\n";
		message += writeErrorLocation(functionName);
	}

	return message;
}

inline std::string OperatingSystem::Configuration::ReportException::operator()(const std::type_info& typeInfo, const std::string& functionName, const std::string& information) const
{
	std::string message;
	{
		message += information;
		message += "\n";
		message += writeErrorLocation(typeInfo, functionName);
	}

	return message;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline std::string OperatingSystem::Configuration::ReportException::writeErrorLocation(const std::string& functionName) const
{
	std::string message;
	{
		message += "\tError location: ";
		message += functionName;
		message += "\n";
	}

	return message;
}

inline std::string OperatingSystem::Configuration::ReportException::writeErrorLocation(const std::type_info& typeInfo, const std::string& functionName) const
{
	std::string message;
	{
		message += "\tError location: ";
		{
			if (functionName == "constructor" || functionName == "Constructor")
			{
				message += '\"';
				message += functionName;
			}

			else
			{
				message += "Member function of \"";
				message += functionName;
			}
		}

		message += "\" in ";
		message += typeInfo.name();
		message += "\n";
	}

	return message;
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !OPERATINGSYSTEM_CONFIGURATION_REPORTEXCEPTION_H

