#ifndef MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_SELECTION_INTERNAL_SELECTCRYSTALS_H
#define MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_SELECTION_INTERNAL_SELECTCRYSTALS_H

#include "CrystallographicInformation.h"
#include "ModelCrystalStructure.h"

#include "CrystalAnalyzer.h"
#include "StructureClassifier.h"

#include "CrystalSelectPlan.h"


namespace MathematicalCrystalChemistry
{
	namespace Runtime
	{
		namespace Selection
		{
			namespace Internal
			{
				class SelectCrystals
				{
					using size_type = std::size_t;
					using CrystallographicInformation = ChemToolkit::Crystallography::SpaceGroups::CrystallographicInformation;

					using ModelCrystalStructure = MathematicalCrystalChemistry::StructureModel::Components::ModelCrystalStructure;
					using CrystalAnalyzer = MathematicalCrystalChemistry::Analysis::CrystalAnalyzer;
					using StructureClassifier = MathematicalCrystalChemistry::IO::StructureClassifier;

// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Constructor, destructor, and operators

				public:
					explicit SelectCrystals(const size_type threadRank);
					SelectCrystals(const size_type threadRank, const CrystalSelectPlan&);
					virtual ~SelectCrystals() = default;

					SelectCrystals(const SelectCrystals&) = default;
					SelectCrystals(SelectCrystals&&) noexcept = default;
					SelectCrystals& operator=(const SelectCrystals&) = default;
					SelectCrystals& operator=(SelectCrystals&&) noexcept = default;

				// Constructor, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Public methods

					void operator()() const;

					void setCrystalSelectPlan(const CrystalSelectPlan&);

				// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

				private:
					size_type m_threadRank;
					std::filesystem::path m_cifDataDirectoryPath;

					CrystalAnalyzer m_crystalAnalyzer;
					StructureClassifier m_structureClassifier;
				};
			}
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline void MathematicalCrystalChemistry::Runtime::Selection::Internal::SelectCrystals::setCrystalSelectPlan(const CrystalSelectPlan& crystalSelectPlan)
{
	m_cifDataDirectoryPath = crystalSelectPlan.cifDataDirectoryPath();

	m_crystalAnalyzer.setChemicalCompositionConstraints(crystalSelectPlan.crystalAnalysisOptions().chemicalCompositionConstraints());
	m_crystalAnalyzer.setCoordinationPolyhedraConstraints(crystalSelectPlan.crystalAnalysisOptions().coordinationPolyhedraConstraints());
	m_crystalAnalyzer.setWholeStructuralConstraints(crystalSelectPlan.crystalAnalysisOptions().wholeStructuralConstraints());

	m_structureClassifier.setOutputDirectoryPath(crystalSelectPlan.outputDirectoryPath());
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_SELECTION_INTERNAL_SELECTCRYSTALS_H
