#ifndef MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_CREATION_STRUCTURECREATIONPLAN_H
#define MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_CREATION_STRUCTURECREATIONPLAN_H

#include <string>
#include <filesystem>

#include "ChemicalComposition.h"

#include "ChemicalParameters.h"
#include "StructureCreationReportOptions.h"
#include "TargetModelAtomCompositions.h"

#include "GeometricalConstraintParameters.h"
#include "InitialStructureGenerationOptions.h"
#include "StructuralOptimizationParameters.h"


namespace MathematicalCrystalChemistry
{
	namespace Runtime
	{
		namespace Creation
		{
			class StructureCreationPlan
			{
				using size_type = std::size_t;
				using ChemicalComposition = ChemToolkit::Generic::ChemicalComposition;
				using ChemicalParameters = ChemToolkit::Generic::ChemicalParameters;

				using GeometricalConstraintParameters = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters;
				using InitialStructureGenerationOptions = MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerationOptions;
				using StructuralOptimizationParameters = MathematicalCrystalChemistry::Design::Optimization::StructuralOptimizationParameters;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				StructureCreationPlan();
				virtual ~StructureCreationPlan() = default;

				StructureCreationPlan(const StructureCreationPlan&) = default;
				StructureCreationPlan(StructureCreationPlan&&) noexcept = default;
				StructureCreationPlan& operator=(const StructureCreationPlan&) = default;
				StructureCreationPlan& operator=(StructureCreationPlan&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void initialize();
				void initializeFromFiles(const std::vector<std::filesystem::path>& filePaths);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				const ChemicalParameters& chemicalParameters() const;
				const GeometricalConstraintParameters& geometricalConstraintParameters() const;

				const StructureCreationReportOptions& structureCreationReportOptions() const;
				const TargetModelAtomCompositions& targetModelAtomCompositions() const;

				const InitialStructureGenerationOptions& initialStructureGenerationOptions() const;
				const StructuralOptimizationParameters& structuralOptimizationParameters() const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				ChemicalParameters m_chemicalParameters;
				GeometricalConstraintParameters m_geometricalConstraintParameters;

				StructureCreationReportOptions m_structureCreationReportOptions;
				TargetModelAtomCompositions m_targetModelAtomCompositions;

				InitialStructureGenerationOptions m_initialStructureGenerationOptions;
				StructuralOptimizationParameters m_structuralOptimizationParameters;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const ChemToolkit::Generic::ChemicalParameters& MathematicalCrystalChemistry::Runtime::Creation::StructureCreationPlan::chemicalParameters() const
{
	return m_chemicalParameters;
}

inline const MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::GeometricalConstraintParameters& MathematicalCrystalChemistry::Runtime::Creation::StructureCreationPlan::geometricalConstraintParameters() const
{
	return m_geometricalConstraintParameters;
}

inline const MathematicalCrystalChemistry::Runtime::Creation::StructureCreationReportOptions& MathematicalCrystalChemistry::Runtime::Creation::StructureCreationPlan::structureCreationReportOptions() const
{
	return m_structureCreationReportOptions;
}

inline const MathematicalCrystalChemistry::Runtime::Creation::TargetModelAtomCompositions& MathematicalCrystalChemistry::Runtime::Creation::StructureCreationPlan::targetModelAtomCompositions() const
{
	return m_targetModelAtomCompositions;
}

inline const MathematicalCrystalChemistry::Design::Generation::InitialStructureGenerationOptions& MathematicalCrystalChemistry::Runtime::Creation::StructureCreationPlan::initialStructureGenerationOptions() const
{
	return m_initialStructureGenerationOptions;
}

inline const MathematicalCrystalChemistry::Design::Optimization::StructuralOptimizationParameters& MathematicalCrystalChemistry::Runtime::Creation::StructureCreationPlan::structuralOptimizationParameters() const
{
	return m_structuralOptimizationParameters;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_CREATION_STRUCTURECREATIONPLAN_H
