#ifndef MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_CREATION_STRUCTURECREATIONREPORTOPTIONS_H
#define MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_CREATION_STRUCTURECREATIONREPORTOPTIONS_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <vector>

#include "ReportException.h"

#include "StructureDesignRecordOptions.h"


namespace MathematicalCrystalChemistry
{
	namespace Runtime
	{
		namespace Creation
		{
			class StructureCreationReportOptions
			{
				using size_type = std::size_t;
				using StructureDesignRecordOptions = MathematicalCrystalChemistry::Design::Recording::StructureDesignRecordOptions;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				StructureCreationReportOptions();
				virtual ~StructureCreationReportOptions() = default;

				StructureCreationReportOptions(const StructureCreationReportOptions&) = default;
				StructureCreationReportOptions(StructureCreationReportOptions&&) noexcept = default;
				StructureCreationReportOptions& operator=(const StructureCreationReportOptions&) = default;
				StructureCreationReportOptions& operator=(StructureCreationReportOptions&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void initialize();
				void initializeFromInputFileTexts(const std::string& inputTexts);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				const std::filesystem::path& workDirectoryPath() const noexcept;
				const std::filesystem::path& designedStructuresDirectoryPath() const noexcept;
				bool needInfeasibleStructureData() const noexcept;

				const StructureDesignRecordOptions& structureDesignRecordOptions() const noexcept;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				bool translateIntoBool(const std::string&) const;

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::filesystem::path m_workDirectoryPath;
				std::filesystem::path m_designedStructuresDirectoryPath;
				bool m_needInfeasibleStructureData;

				StructureDesignRecordOptions m_structureDesignRecordOptions;
			};
		}
	}
}


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const std::filesystem::path& MathematicalCrystalChemistry::Runtime::Creation::StructureCreationReportOptions::workDirectoryPath() const noexcept
{
	return m_workDirectoryPath;
}

inline const std::filesystem::path& MathematicalCrystalChemistry::Runtime::Creation::StructureCreationReportOptions::designedStructuresDirectoryPath() const noexcept
{
	return m_designedStructuresDirectoryPath;
}

inline bool MathematicalCrystalChemistry::Runtime::Creation::StructureCreationReportOptions::needInfeasibleStructureData() const noexcept
{
	return m_needInfeasibleStructureData;
}

inline const MathematicalCrystalChemistry::Design::Recording::StructureDesignRecordOptions& MathematicalCrystalChemistry::Runtime::Creation::StructureCreationReportOptions::structureDesignRecordOptions() const noexcept
{
	return m_structureDesignRecordOptions;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline bool MathematicalCrystalChemistry::Runtime::Creation::StructureCreationReportOptions::translateIntoBool(const std::string& text) const
{
	if ((text == "ON") || (text == "On") || (text == "on"))
		return true;

	else if ((text == "OFF") || (text == "Off") || (text == "off"))
		return false;

	else
	{
		std::string errorMessage;
		{
			errorMessage += "\"";
			errorMessage += text;
			errorMessage += "\" is not a switch flag.";
		}


		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "translateIntoBool", errorMessage));
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_CREATION_STRUCTURECREATIONREPORTOPTIONS_H
