#ifndef MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_CREATION_STRUCTUREDESIGNER_H
#define MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_CREATION_STRUCTUREDESIGNER_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <map>
#include <queue>

#include "ReportException.h"

#include "StructureCreationPlan.h"


namespace MathematicalCrystalChemistry
{
	namespace Runtime
	{
		namespace Creation
		{
			class StructureCreator
			{
				using size_type = std::size_t;
				using ModelAtomCollection = MathematicalCrystalChemistry::StructureModel::Components::ModelAtomCollection;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors and destructor

			public:
				StructureCreator() = default;
				explicit StructureCreator(const StructureCreationPlan&);
				explicit StructureCreator(StructureCreationPlan&&);

				virtual ~StructureCreator() = default;

			// Constructors and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void execute() const;

				void setStructureCreationPlan(const StructureCreationPlan&);
				void setStructureCreationPlan(StructureCreationPlan&&);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				void initializeDirectories() const;
				void initializeChemicalParameters() const;
				void initializeGeometricalConstraintParameters() const;

				bool isValidModelAtomCollection(const ModelAtomCollection&) const;

			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private utility

				size_type getMaxStructureGenerating(const size_type totalStructureGenerating) const noexcept;

			// Private utility
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				StructureCreationPlan m_structureCreationPlan;


			private:
				StructureCreator(const StructureCreator&) = delete;
				StructureCreator(StructureCreator&&) = delete;
				StructureCreator& operator=(const StructureCreator&) = delete;
				StructureCreator& operator=(StructureCreator&&) = delete;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline void MathematicalCrystalChemistry::Runtime::Creation::StructureCreator::setStructureCreationPlan(const StructureCreationPlan& plan)
{
	m_structureCreationPlan = plan;
}

inline void MathematicalCrystalChemistry::Runtime::Creation::StructureCreator::setStructureCreationPlan(StructureCreationPlan&& plan)
{
	m_structureCreationPlan = std::move(plan);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

inline bool MathematicalCrystalChemistry::Runtime::Creation::StructureCreator::isValidModelAtomCollection(const ModelAtomCollection& modelAtomCollection) const
{
	for (const auto& modelAtom : modelAtomCollection)
	{
		if (!(modelAtom.coordinationFeature().symbol().isValid()))
			return false;
	}

	return true;
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_RUNTIME_CREATION_STRUCTUREDESIGNER_H
