#ifndef MATHEMATICALCRYSTALCHEMISTRY_DESIGN_OPTIMIZATION_INTERNAL_STRUCTURECUSTOMIZERBASE_H
#define MATHEMATICALCRYSTALCHEMISTRY_DESIGN_OPTIMIZATION_INTERNAL_STRUCTURECUSTOMIZERBASE_H

#include <stdexcept>

#include "ReportException.h"


namespace MathematicalCrystalChemistry
{
	namespace Design
	{
		namespace Optimization
		{
			namespace Internal
			{
				class StructureCustomizerBase
				{
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Constructors, destructor, and operators

				protected:
					StructureCustomizerBase();

				public:
					virtual ~StructureCustomizerBase() = default;

					StructureCustomizerBase(const StructureCustomizerBase&) = default;
					StructureCustomizerBase(StructureCustomizerBase&&) noexcept = default;
					StructureCustomizerBase& operator=(const StructureCustomizerBase&) = default;
					StructureCustomizerBase& operator=(StructureCustomizerBase&&) noexcept = default;

				// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
				// Static methods

					static double commonPressure() noexcept;
					static double commonAttractiveForceConstant() noexcept;
					static double commonRepulsiveForceConstant() noexcept;

					static double defaultCommonPressure() noexcept;
					static double defaultCommonAttractiveForceConstant() noexcept;
					static double defaultCommonRepulsiveForceConstant() noexcept;

					static void setCommonPressure();
					static void setCommonPressure(const double);
					static void setCommonAttractiveForceConstant();
					static void setCommonAttractiveForceConstant(const double commonAttractiveForceConstant);
					static void setCommonRepulsiveForceConstant();
					static void setCommonRepulsiveForceConstant(const double commonRepulsiveForceConstant);

				// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

				private:
					static double m_commonPressure;
					static double m_commonAttractiveForceConstant;
					static double m_commonRepulsiveForceConstant;

					static double m_defaultCommonPressure;
					static double m_defaultCommonAttractiveForceConstant;
					static double m_defaultCommonRepulsiveForceConstant;
				};
			}
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

inline double MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::commonPressure() noexcept
{
	return m_commonPressure;
}

inline double MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::commonAttractiveForceConstant() noexcept
{
	return m_commonAttractiveForceConstant;
}

inline double MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::commonRepulsiveForceConstant() noexcept
{
	return m_commonRepulsiveForceConstant;
}

inline double MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::defaultCommonPressure() noexcept
{
	return m_defaultCommonPressure;
}

inline double MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::defaultCommonAttractiveForceConstant() noexcept
{
	return m_defaultCommonAttractiveForceConstant;
}

inline double MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::defaultCommonRepulsiveForceConstant() noexcept
{
	return m_defaultCommonRepulsiveForceConstant;
}

inline void MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::setCommonPressure()
{
	m_commonPressure = m_defaultCommonPressure;
}

inline void MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::setCommonPressure(const double val)
{
	if (val < 0.0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException("setCommonPressure of MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase", "\"pressure\" is less than zero."));
	}

	else
		m_commonPressure = val;
}

inline void MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::setCommonAttractiveForceConstant()
{
	m_commonAttractiveForceConstant = m_defaultCommonAttractiveForceConstant;
}

inline void MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::setCommonAttractiveForceConstant(const double val)
{
	if (val < 0.0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException("setCommonAttractiveForceConstant of MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase", "\"commonAttractiveForceConstant\" is not more than zero."));
	}

	else
		m_commonAttractiveForceConstant = val;
}

inline void MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::setCommonRepulsiveForceConstant()
{
	m_commonRepulsiveForceConstant = m_defaultCommonRepulsiveForceConstant;
}

inline void MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase::setCommonRepulsiveForceConstant(const double val)
{
	if (0.0 < val)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException("setCommonRepulsiveForceConstant of MathematicalCrystalChemistry::Design::Optimization::Internal::StructureCustomizerBase", "\"commonRepulsiveForceConstant\" is not less than zero."));
	}

	else
		m_commonRepulsiveForceConstant = val;
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_DESIGN_OPTIMIZATION_INTERNAL_STRUCTURECUSTOMIZERBASE_H
