#ifndef MATHEMATICALCRYSTALCHEMISTRY_DESIGN_RECORDING_STRUCTUREDESIGNRECORDOPTIONS_H
#define MATHEMATICALCRYSTALCHEMISTRY_DESIGN_RECORDING_STRUCTUREDESIGNRECORDOPTIONS_H

#include <stdexcept>
#include <string>
#include <filesystem>

#include "ReportException.h"


namespace MathematicalCrystalChemistry
{
	namespace Design
	{
		namespace Recording
		{
			class StructureDesignRecordOptions
			{
				using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operators

			public:
				StructureDesignRecordOptions();
				virtual ~StructureDesignRecordOptions() = default;

				StructureDesignRecordOptions(const StructureDesignRecordOptions&) = default;
				StructureDesignRecordOptions(StructureDesignRecordOptions&&) noexcept = default;
				StructureDesignRecordOptions& operator=(const StructureDesignRecordOptions&) = default;
				StructureDesignRecordOptions& operator=(StructureDesignRecordOptions&&) noexcept = default;

			// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void initialize();
				void initializeFromInputFileTexts(const std::string& inputTexts);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				size_type mdRecordFrequency() const;
				size_type chemicalBondsRecordFrequency() const;

				bool needMdRecord() const;
				bool needChemicalBondsRecord() const;

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				size_type m_mdRecordFrequency;
				size_type m_chemicalBondsRecordFrequency;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline bool MathematicalCrystalChemistry::Design::Recording::StructureDesignRecordOptions::needMdRecord() const
{
	return static_cast<bool>(m_mdRecordFrequency);
}

inline bool MathematicalCrystalChemistry::Design::Recording::StructureDesignRecordOptions::needChemicalBondsRecord() const
{
	return static_cast<bool>(m_chemicalBondsRecordFrequency);
}

inline std::size_t MathematicalCrystalChemistry::Design::Recording::StructureDesignRecordOptions::mdRecordFrequency() const
{
	return m_mdRecordFrequency;
}

inline std::size_t MathematicalCrystalChemistry::Design::Recording::StructureDesignRecordOptions::chemicalBondsRecordFrequency() const
{
	return m_chemicalBondsRecordFrequency;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_DESIGN_RECORDING_STRUCTUREDESIGNRECORDOPTIONS_H
