#ifndef CHEMTOOLKIT_CRYSTALLOGRAPHY_SPACEGROUPS_SYMMETRYOPERATIONTABLE_H
#define CHEMTOOLKIT_CRYSTALLOGRAPHY_SPACEGROUPS_SYMMETRYOPERATIONTABLE_H

#include <string>
#include <stdexcept>
#include <vector>

#include "LinearAlgebra.h"


namespace ChemToolkit
{
	namespace Crystallography
	{
		namespace SpaceGroups
		{
			class SymmetryOperationTable
			{
				using size_type = std::size_t;
				using NumericalMatrix = MathToolkit::Generic::NumericalMatrix<double>;
				using AffineMapping = MathToolkit::Generic::AffineMapping<double>;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructor and destructor

			public:
				virtual ~SymmetryOperationTable() = default;

			// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				static std::vector<AffineMapping> getSymmetryOperations(const std::string& symbol);
				static bool isValidSymmetryOperationSymbol(const std::string& symbol);

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Private methods

			private:
				static std::vector<AffineMapping> translateIntoSymmetryOperations(const std::string& symbol);

				static std::vector<AffineMapping> getRotationMappings(const size_type multiplicity);
				static std::vector<AffineMapping> getReflectionMappings();
				static std::vector<AffineMapping> getInversionMappings();
				static std::vector<AffineMapping> getRotoinversionMappings(const size_type multiplicity);


			// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				SymmetryOperationTable() = default;

				SymmetryOperationTable(const SymmetryOperationTable&) = delete;
				SymmetryOperationTable(SymmetryOperationTable&&) = delete;
				SymmetryOperationTable& operator=(const SymmetryOperationTable&) = delete;
				SymmetryOperationTable& operator=(SymmetryOperationTable&&) = delete;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::vector<MathToolkit::Generic::AffineMapping<double>> ChemToolkit::Crystallography::SpaceGroups::SymmetryOperationTable::getSymmetryOperations(const std::string& symbol)
{
	std::vector<AffineMapping> affineMappings = translateIntoSymmetryOperations(symbol);

	if (affineMappings.empty())
	{
		std::string errorMessage;
		{
			errorMessage += "Symmetry symbol of \"";
			errorMessage += symbol;
			errorMessage += "\" is out of definitions.";
		}

		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException("ChemToolkit::Crystallography::SymmetryProperties::SymmetryOperationTable::getSymmetryOperation", errorMessage));
	}

	else
		return affineMappings;
}

inline bool ChemToolkit::Crystallography::SpaceGroups::SymmetryOperationTable::isValidSymmetryOperationSymbol(const std::string& symbol)
{
	std::vector<AffineMapping> linearMappings = translateIntoSymmetryOperations(symbol);

	if (linearMappings.empty())
		return false;
	else
		return true;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CHEMTOOLKIT_CRYSTALLOGRAPHY_SPACEGROUPS_SYMMETRYOPERATIONTABLE_H
