#ifndef OPERATINGSYSTEM_CONFIGURATION_THREADCONTROLLER_H
#define OPERATINGSYSTEM_CONFIGURATION_THREADCONTROLLER_H

#include <string>
#include <stdexcept>


namespace OperatingSystem
{
	namespace Configuration
	{
		class ThreadController final
		{
			using size_type = std::size_t;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		private:
			ThreadController() = default;

		public:
			~ThreadController() = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			static std::size_t maxThreading();
			static std::size_t mklThreading();
			static std::size_t mklDomainThreading(const std::string& domainName);
			static std::size_t mklStriping();
			static bool isMklDynamic();

			static std::size_t defaultMaxThreading();
			static std::size_t defaultMaxMklThreading();

			static void setMaxThreading(std::size_t maxThreading = 0);
			static void setMklThreading(std::size_t mklThreading = 0);
			static void setLocalMklThreading(std::size_t localMklThreading = 0);
			static void setMklDomainThreading(const std::string& domainName, std::size_t mklDomainThreading = 0);
			static void setMklStriping(std::size_t mklStriping = 0);
			static void setMklDynamic(const bool);

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			static int translateMklDomainName(const std::string& domainName);

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			static std::size_t m_maxThreading;

			static std::size_t m_defaultMaxThreading;
			static std::size_t m_defaultMaxMklThreading;


		private:
			ThreadController(const ThreadController&) = delete;
			ThreadController(ThreadController&&) = delete;
			ThreadController& operator=(const ThreadController&) = delete;
			ThreadController& operator=(ThreadController&&) = delete;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::size_t OperatingSystem::Configuration::ThreadController::maxThreading()
{
	return m_maxThreading;
}

inline std::size_t OperatingSystem::Configuration::ThreadController::defaultMaxThreading()
{
	return m_defaultMaxThreading;
}

inline std::size_t OperatingSystem::Configuration::ThreadController::defaultMaxMklThreading()
{
	return m_defaultMaxMklThreading;
}

inline void OperatingSystem::Configuration::ThreadController::setMaxThreading(std::size_t maxThreading)
{
	if (maxThreading == 0)
		m_maxThreading = m_defaultMaxThreading;
	else
		m_maxThreading = maxThreading;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !OPERATINGSYSTEM_CONFIGURATION_THREADCONTROLLER_H
