#ifndef MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_WHOLESTRUCTURALCONSTRAINTS_H
#define MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_WHOLESTRUCTURALCONSTRAINTS_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <vector>
#include <map>

#include "AtomCategory.h"

#include "CrystalCategory.h"


namespace MathematicalCrystalChemistry
{
	namespace Analysis
	{
		class WholeStructuralConstraints
		{
			using size_type = std::size_t;
			using AtomCategory = MathematicalCrystalChemistry::StructureModel::Features::AtomCategory;
			using CrystalCategory = MathematicalCrystalChemistry::StructureModel::Features::CrystalCategory;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		public:
			WholeStructuralConstraints();
			virtual ~WholeStructuralConstraints() = default;

			WholeStructuralConstraints(const WholeStructuralConstraints&) = default;
			WholeStructuralConstraints(WholeStructuralConstraints&&) noexcept = default;
			WholeStructuralConstraints& operator=(const WholeStructuralConstraints&) = default;
			WholeStructuralConstraints& operator=(WholeStructuralConstraints&&) noexcept = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			void initialize();
			void initializeFromInputFileTexts(const std::string& inputTexts);

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Access methods

			const std::vector<size_type>& feasibleSpaceGroupNumbers() const noexcept;
			double feasibleGeometricalConstraintErrorRate() const noexcept;

		// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			std::vector<size_type> m_feasibleSpaceGroupNumbers;
			double m_feasibleGeometricalConstraintErrorRate;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const std::vector<std::size_t>& MathematicalCrystalChemistry::Analysis::WholeStructuralConstraints::feasibleSpaceGroupNumbers() const noexcept
{
	return m_feasibleSpaceGroupNumbers;
}

inline double MathematicalCrystalChemistry::Analysis::WholeStructuralConstraints::feasibleGeometricalConstraintErrorRate() const noexcept
{
	return m_feasibleGeometricalConstraintErrorRate;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************


#endif // !MATHEMATICALCRYSTALCHEMISTRY_ANALYSIS_WHOLESTRUCTURALCONSTRAINTS_H
