#include "AtomCategoryComposition.h"

#include <vector>
#include <stdexcept>
#include <numeric>

#include "ReportException.h"

using namespace MathematicalCrystalChemistry::StructureModel::Features;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

std::string AtomCategoryComposition::name() const
{
	if (!(isEmpty()))
	{
		std::string compositionName;
		{
			for (const auto& symbolAndCount : m_dictionary)
			{
				compositionName += symbolAndCount.first;
				compositionName += "_";
				compositionName += std::to_string(symbolAndCount.second);
				compositionName += "_";
			}

			compositionName.pop_back();
		}

		return compositionName;
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "name", "This is invalid."));
	}
}

std::string AtomCategoryComposition::getSystemName() const
{
	if (!(isEmpty()))
	{
		std::string name;
		{
			for (const auto& numAndCount : m_dictionary)
			{
				name += numAndCount.first;
				name += "-";
			}

			name.pop_back();
		}

		return name;
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "getAtomCategorySystemName", "This is invalid."));
	}
}

AtomCategoryComposition AtomCategoryComposition::getReducedAtomCategoryComposition() const
{
	if (!(isEmpty()))
	{
		AtomCategoryComposition reducedAtomCategoryComposition;
		reducedAtomCategoryComposition.m_dictionary = m_dictionary;
		{
			int greatestCommonDivider{ 0 };
			{
				std::vector<int> atomCounts;
				{
					for (const auto& symbolAndCount : m_dictionary)
						atomCounts.push_back(static_cast<int>(symbolAndCount.second));
				}

				greatestCommonDivider = std::accumulate(atomCounts.begin(), atomCounts.end(), 0, [](const int a, const int b) { return std::gcd(a, b); });
			}

			for (auto& symbolAndCount : reducedAtomCategoryComposition.m_dictionary)
				symbolAndCount.second /= greatestCommonDivider;
		}

		return reducedAtomCategoryComposition;
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "getReducedAtomCategoryComposition", "This is invalid."));
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
