#include "AtomicEnvironment.h"

#include <algorithm>
#include <map>

using namespace MathematicalCrystalChemistry::StructureModel::Components;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

AtomicEnvironment::AtomicEnvironment()
	: m_elementSymbol{}
	, m_atomCategorySymbol{}
	, m_siteSymmetrySymbol{}
	, m_wyckoffSymbol{}
	, m_siteIndex{ 0 }
	, m_ionicBondFingerprint{}
	, m_metallicBondFingerprint{}
	, m_cationicConstraintFingerprint{}
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

std::size_t AtomicEnvironment::getHashValue() const
{
	size_type hashValue = 0;
	{
		hashValue ^= std::hash<std::string>{}(m_atomCategorySymbol);
		hashValue ^= std::hash<std::string>{}(m_siteSymmetrySymbol);

		for (const auto& symbol : m_ionicBondFingerprint)
			hashValue ^= std::hash<std::string>{}(symbol);

		for (const auto& symbol : m_metallicBondFingerprint)
			hashValue ^= std::hash<std::string>{}(symbol);

		for (const auto& symbolAndBridges : m_cationicConstraintFingerprint)
		{
			hashValue ^= std::hash<std::string>{}(symbolAndBridges.first);

			for (const auto& symbol : symbolAndBridges.second)
				hashValue ^= std::hash<std::string>{}(symbol);
		}
	}

	return hashValue;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

std::string AtomicEnvironment::getReportText() const
{
	std::string reportText;
	{
		reportText += m_elementSymbol;
		reportText += "   ";
		reportText += m_atomCategorySymbol;
		reportText += "   ";
		reportText += std::to_string(m_siteIndex);
		reportText += "   ";
		reportText += m_siteSymmetrySymbol;
		reportText += "   ";
		reportText += getChemicalBondReportText(m_ionicBondFingerprint);
		reportText += "   ";
		reportText += getChemicalBondReportText(m_metallicBondFingerprint);
		reportText += "   ";
		reportText += getCationicConstraintReportText();
		reportText += "\n";
	}

	return reportText;
}

std::string AtomicEnvironment::getFingerprint() const
{
	std::string fingerprintString;
	{
		fingerprintString += "AtomCategorySymbol:_";
		fingerprintString += m_atomCategorySymbol;
		fingerprintString += "__SiteSymmetry:_";
		fingerprintString += m_siteSymmetrySymbol;
		fingerprintString += writeConstraintsFingerprint();
	}

	return fingerprintString;
}

std::string AtomicEnvironment::getPlainFingerprint() const
{
	std::string fingerprintString;
	{
		fingerprintString += "AtomCategorySymbol:_";
		fingerprintString += m_atomCategorySymbol;
		fingerprintString += writeConstraintsFingerprint();
	}

	return fingerprintString;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

std::string AtomicEnvironment::writeConstraintsFingerprint() const
{
	std::string fingerprintString;
	{
		{
			fingerprintString += "__IonicBondedAtoms:_";

			for (const auto& symbol : m_ionicBondFingerprint)
			{
				fingerprintString += symbol;
				fingerprintString += "-";
			}

			fingerprintString.pop_back();
		}
		{
			fingerprintString += "__MetallicBondedAtoms:_";

			for (const auto& symbol : m_metallicBondFingerprint)
			{
				fingerprintString += symbol;
				fingerprintString += "-";
			}

			fingerprintString.pop_back();
		}
		{
			fingerprintString += "__CationicConstraintedAtoms:_";

			for (const auto& symbolAndBridges : m_cationicConstraintFingerprint)
			{
				fingerprintString += "CBA[";
				{
					for (const auto& symbol : symbolAndBridges.second)
					{
						fingerprintString += symbol;
						fingerprintString += ",";
					}

					fingerprintString.pop_back();
					fingerprintString += "]=";
				}

				fingerprintString += symbolAndBridges.first;
				fingerprintString += "-";
			}

			fingerprintString.pop_back();
		}
	}

	return fingerprintString;
}

std::string AtomicEnvironment::getChemicalBondReportText(const std::vector<std::string>& bondedAtoms) const
{
	std::map<std::string, size_type> chemicalBondComposition;
	{
		for (const auto& elementSymbol : bondedAtoms)
		{
			auto iter = chemicalBondComposition.find(elementSymbol);

			if (iter == chemicalBondComposition.end())
				chemicalBondComposition.emplace(elementSymbol, 1);
			else
				++iter->second;
		}
	}

	std::string reportText;
	{
		for (const auto& symbolAndNum : chemicalBondComposition)
		{
			reportText += symbolAndNum.first;
			reportText += std::to_string(symbolAndNum.second);
		}
	}

	return reportText;
}

std::string AtomicEnvironment::getCationicConstraintReportText() const
{
	std::string reportText;
	{
		for (const auto& symbolAndBridges : m_cationicConstraintFingerprint)
		{
			std::map<std::string, size_type> bridgeAtomComposition;
			{
				for (const auto& symbol : symbolAndBridges.second)
				{
					auto iter = bridgeAtomComposition.find(symbol);

					if (iter == bridgeAtomComposition.end())
						bridgeAtomComposition.emplace(symbol, 1);
					else
						++iter->second;
				}
			}

			reportText += "---CBA[";
			{
				for (const auto& symbolAndNum : bridgeAtomComposition)
				{
					reportText += symbolAndNum.first;
					reportText += std::to_string(symbolAndNum.second);
				}
			}
			reportText += "]-";
			reportText += symbolAndBridges.first;
		}
	}

	return reportText;
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private access methods

void AtomicEnvironment::setIonicBondFingerprint(const std::vector<Ligand>& ligands)
{
	m_ionicBondFingerprint.clear();

	for (const auto& ligand : ligands)
		m_ionicBondFingerprint.push_back(ligand.get().coordinationFeature().atomCategory().symbol());

	std::sort(m_ionicBondFingerprint.begin(), m_ionicBondFingerprint.end());
}

void AtomicEnvironment::setMetallicBondFingerprint(const std::vector<Ligand>& ligands)
{
	m_metallicBondFingerprint.clear();

	for (const auto& ligand : ligands)
		m_metallicBondFingerprint.push_back(ligand.get().coordinationFeature().atomCategory().symbol());

	std::sort(m_metallicBondFingerprint.begin(), m_metallicBondFingerprint.end());
}

void AtomicEnvironment::setCationicConstraintFingerprint(const std::vector<std::pair<Ligand, std::vector<Ligand>>>& ligandsAndBridges)
{
	m_cationicConstraintFingerprint.clear();

	for (const auto& ligandAndBridges : ligandsAndBridges)
	{
		std::vector<std::string> bridgeSymbols;
		{
			for (const auto& bridgingAnion : ligandAndBridges.second)
				bridgeSymbols.push_back(bridgingAnion.get().coordinationFeature().atomCategory().symbol());
		}

		m_cationicConstraintFingerprint.push_back(std::make_pair(ligandAndBridges.first.get().coordinationFeature().atomCategory().symbol(), std::move(bridgeSymbols)));
	}

	std::sort(m_cationicConstraintFingerprint.begin(), m_cationicConstraintFingerprint.end());
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
