#include "ChemicalComposition.h"

#include <vector>
#include <stdexcept>
#include <numeric>

#include "ReportException.h"

using namespace ChemToolkit::Generic;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

std::string ChemicalComposition::name() const
{
	if (!(isEmpty()))
	{
		std::string compositionName;
		{
			for (const auto& symbolAndCount : m_dictionary)
			{
				compositionName += symbolAndCount.first;
				compositionName += "_";
				compositionName += std::to_string(symbolAndCount.second);
				compositionName += "_";
			}

			compositionName.pop_back();
		}

		return compositionName;
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "name", "This is invalid."));
	}
}

std::string ChemicalComposition::getChemicalFormula() const
{
	if (!(isEmpty()))
	{
		std::map<std::string, size_type> reducedDictionary;
		{
			int greatestCommonDivider{ 0 };
			{
				std::vector<int> atomCounts;
				{
					for (const auto& symbolAndCount : m_dictionary)
						atomCounts.push_back(static_cast<int>(symbolAndCount.second));
				}

				greatestCommonDivider = std::accumulate(atomCounts.begin(), atomCounts.end(), 0, [](const int a, const int b) { return std::gcd(a, b); });
			}

			reducedDictionary = m_dictionary;
			{
				for (auto& numAndCount : reducedDictionary)
					numAndCount.second /= greatestCommonDivider;
			}
		}


		std::string chemicalFormula;
		{
			for (const auto& symbolAndCount : reducedDictionary)
			{
				chemicalFormula += symbolAndCount.first;
				chemicalFormula += "_";
				chemicalFormula += std::to_string(symbolAndCount.second);
				chemicalFormula += "_";
			}

			chemicalFormula.pop_back();
		}

		return chemicalFormula;
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "getChemicalFormula", "This is invalid."));
	}
}

std::string ChemicalComposition::getChemicalSystemName() const
{
	if (!(isEmpty()))
	{
		std::string name;
		{
			for (const auto& numAndCount : m_dictionary)
			{
				name += numAndCount.first;
				name += "-";
			}

			name.pop_back();
		}

		return name;
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "getChemicalSystemName", "This is invalid."));
	}
}

ChemicalComposition ChemicalComposition::getReducedChemicalComposition() const
{
	if (!(isEmpty()))
	{
		ChemicalComposition reducedChemicalComposition;
		reducedChemicalComposition.m_dictionary = m_dictionary;
		{
			int greatestCommonDivider{ 0 };
			{
				std::vector<int> atomCounts;
				{
					for (const auto& symbolAndCount : m_dictionary)
						atomCounts.push_back(static_cast<int>(symbolAndCount.second));
				}

				greatestCommonDivider = std::accumulate(atomCounts.begin(), atomCounts.end(), 0, [](const int a, const int b) { return std::gcd(a, b); });
			}

			for (auto& symbolAndCount : reducedChemicalComposition.m_dictionary)
				symbolAndCount.second /= greatestCommonDivider;
		}

		return reducedChemicalComposition;
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "getReducedChemicalComposition", "This is invalid."));
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
