#include "ChemicalCompositionConstraints.h"

#include <limits>

#include "FileReader.h"
#include "PeriodicTable.h"

using namespace MathematicalCrystalChemistry::Analysis;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

ChemicalCompositionConstraints::ChemicalCompositionConstraints()
	: m_minNumAtoms{ 0 }
	, m_maxNumAtoms{ std::numeric_limits<size_type>::max() }
	, m_feasibleCrystalCategories{}
	, m_feasibleAtomCategories{}
	, m_necessaryElementSymbols{}
	, m_infeasibleElementSymbols{}
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void ChemicalCompositionConstraints::initialize()
{
	m_minNumAtoms = 0;
	m_maxNumAtoms = std::numeric_limits<size_type>::max();

	m_feasibleCrystalCategories.clear();
	m_feasibleAtomCategories.clear();
	m_necessaryElementSymbols.clear();
	m_infeasibleElementSymbols.clear();
}

void ChemicalCompositionConstraints::initializeFromInputFileTexts(const std::string& inputTexts)
{
	initialize();

	OperatingSystem::IO::FileReader::tryReadData(inputTexts, "Minimum.Number.of.Atoms.Per.Unit.Cell", m_minNumAtoms);
	OperatingSystem::IO::FileReader::tryReadData(inputTexts, "Maximum.Number.of.Atoms.Per.Unit.Cell", m_maxNumAtoms);
	OperatingSystem::IO::FileReader::tryReadData(inputTexts, "Necessary.Chemical.Elements", m_necessaryElementSymbols);
	OperatingSystem::IO::FileReader::tryReadData(inputTexts, "Infeasible.Chemical.Elements", m_infeasibleElementSymbols);

	readFeasibleCrystalCategories(inputTexts);
	readFeasibleAtomCategories(inputTexts);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void ChemicalCompositionConstraints::readFeasibleCrystalCategories(const std::string& inputTexts)
{
	std::vector<std::string> categoryNames;
	OperatingSystem::IO::FileReader::tryReadData(inputTexts, "Feasible.Crystal.Categories", categoryNames);

	for (const auto& categoryName : categoryNames)
		m_feasibleCrystalCategories.push_back(CrystalCategory{ categoryName });
}

void ChemicalCompositionConstraints::readFeasibleAtomCategories(const std::string& inputTexts)
{
	std::vector<std::string> categoryNames;
	OperatingSystem::IO::FileReader::tryReadData(inputTexts, "Feasible.Atom.Categories", categoryNames);

	for (const auto& categoryName : categoryNames)
		m_feasibleAtomCategories.push_back(AtomCategory{ categoryName });
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
