#include "ConstraintComposition.h"

#include <regex>

#include "ReportException.h"

using namespace MathematicalCrystalChemistry::StructureModel::Features;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

ConstraintComposition::ConstraintComposition()
	: m_isUnconstrained{ false }
	, m_symbolComposition{}
{
}

ConstraintComposition::ConstraintComposition(const std::string& name)
	: m_isUnconstrained{ false }
	, m_symbolComposition{ getSymbolComposition(name) }
{
	if (name == "*")
		m_isUnconstrained = true;
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

std::string ConstraintComposition::name() const
{
	if (m_isUnconstrained)
		return std::string{ "*" };
	else
	{
		std::string constraintCompositionName;
		{
			for (const auto& symbolAndNum : m_symbolComposition)
			{
				constraintCompositionName += symbolAndNum.first;
				constraintCompositionName += std::to_string(symbolAndNum.second);
			}
		}

		return constraintCompositionName;
	}
}

std::string ConstraintComposition::getSystemName() const
{
	if (m_isUnconstrained)
		return std::string{ "*" };
	else
	{
		if (m_symbolComposition.empty())
			return std::string{};

		else
		{
			std::string name;
			{
				for (const auto& symbolAndNum : m_symbolComposition)
				{
					name += symbolAndNum.first;
					name += "-";
				}

				name.pop_back();
			}

			return name;
		}
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

std::map<std::string, std::size_t> ConstraintComposition::getSymbolComposition(const std::string& name)
{
	if (name.empty() || name == "*")
		return std::map<std::string, std::size_t>{};

	else
	{
		std::map<std::string, std::size_t> dictionary;
		{
			std::regex totalPat("([[:alpha:]]+[\\d]+)*([[:alpha:]]+[\\d]+){1}");
			std::regex pat("([[:alpha:]]+)([\\d]+)");

			for (std::sregex_iterator iter(name.begin(), name.end(), pat); iter != std::sregex_iterator{}; ++iter)
				dictionary.emplace(iter->str(1), static_cast<size_type>(std::stoi(iter->str(2))));
		}

		return dictionary;
	}
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
