#include "ConstraintKey.h"

using namespace MathematicalCrystalChemistry::StructureModel::GeometricalConstraints;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

ConstraintKey::ConstraintKey(ModelAtom& modelAtom, const Ligand& ligand)
	: m_modelAtomRef{ std::ref(modelAtom) }
	, m_ligand{ ligand }
	, m_hashValue{}
{
	m_hashValue = std::hash<const ModelAtom*>{}(&(m_modelAtomRef.get()));
	m_hashValue ^= m_ligand.getHashValue();
}

ConstraintKey::ConstraintKey(ModelAtom& modelAtom, Ligand&& ligand)
	: m_modelAtomRef{ std::ref(modelAtom) }
	, m_ligand{ std::move(ligand) }
	, m_hashValue{}
{
	m_hashValue = std::hash<const ModelAtom*>{}(&(m_modelAtomRef.get()));
	m_hashValue ^= m_ligand.getHashValue();
}

ConstraintKey::ConstraintKey(const ConstraintKey& constraintKey)
	: m_modelAtomRef{ constraintKey.m_modelAtomRef }
	, m_ligand{ constraintKey.m_ligand }
	, m_hashValue{}
{
	m_hashValue = std::hash<const ModelAtom*>{}(&(m_modelAtomRef.get()));
	m_hashValue ^= m_ligand.getHashValue();
}

ConstraintKey::ConstraintKey(ConstraintKey&& constraintKey) noexcept
	: m_modelAtomRef{ constraintKey.m_modelAtomRef }
	, m_ligand{ std::move(constraintKey.m_ligand) }
	, m_hashValue{}
{
	m_hashValue = std::hash<const ModelAtom*>{}(&(m_modelAtomRef.get()));
	m_hashValue ^= m_ligand.getHashValue();
}

ConstraintKey& ConstraintKey::operator=(const ConstraintKey& constraintKey)
{
	m_modelAtomRef = constraintKey.m_modelAtomRef;
	m_ligand = constraintKey.m_ligand;

	m_hashValue = std::hash<const ModelAtom*>{}(&(m_modelAtomRef.get()));
	m_hashValue ^= m_ligand.getHashValue();

	return *this;
}

ConstraintKey& ConstraintKey::operator=(ConstraintKey&& constraintKey) noexcept
{
	m_modelAtomRef = constraintKey.m_modelAtomRef;
	m_ligand = std::move(constraintKey.m_ligand);

	m_hashValue = std::hash<const ModelAtom*>{}(&(m_modelAtomRef.get()));
	m_hashValue ^= m_ligand.getHashValue();

	return *this;
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
