#include "CoordinationCenter.h"

#include "LengthCasting.h"

#include "IonicBond.h"
#include "MetallicBond.h"

using namespace MathematicalCrystalChemistry::StructureModel::Components::Internal;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

CoordinationCenter::CoordinationCenter(const std::string& elementSymbol)
	: CoordinationCenterBase{ elementSymbol }
	, AtomBase{ elementSymbol }
	, m_localCoordinateSystem{}
	, m_ionicLigands{}
	, m_metallicLigands{}
{
}

CoordinationCenter::CoordinationCenter(const std::string& elementSymbol, const CoordinationFeatureSymbol& coordinationFeatureSymbol)
	: CoordinationCenterBase{ elementSymbol, coordinationFeatureSymbol }
	, AtomBase{ elementSymbol }
	, m_localCoordinateSystem{}
	, m_ionicLigands{}
	, m_metallicLigands{}
{
}

CoordinationCenter::CoordinationCenter(const std::string& elementSymbol, const NumericalVector& cartesianCoordinate)
	: CoordinationCenterBase{ elementSymbol }
	, AtomBase{ elementSymbol, cartesianCoordinate }
	, m_localCoordinateSystem{}
	, m_ionicLigands{}
	, m_metallicLigands{}
{
}

CoordinationCenter::CoordinationCenter(const std::string& elementSymbol, const CoordinationFeatureSymbol& coordinationFeatureSymbol, const NumericalVector& cartesianCoordinate)
	: CoordinationCenterBase{ elementSymbol, coordinationFeatureSymbol }
	, AtomBase{ elementSymbol, cartesianCoordinate }
	, m_localCoordinateSystem{}
	, m_ionicLigands{}
	, m_metallicLigands{}
{
}

CoordinationCenter::CoordinationCenter(const CoordinationCenter& coordinationCenter)
	: CoordinationCenterBase{ dynamic_cast<const CoordinationCenterBase&>(coordinationCenter) }
	, AtomBase{ coordinationCenter.elementSymbol(), coordinationCenter.coordinate() }
	, m_localCoordinateSystem{}
	, m_ionicLigands{}
	, m_metallicLigands{}
{
}

CoordinationCenter::CoordinationCenter(CoordinationCenter&& coordinationCenter) noexcept
	: CoordinationCenterBase{ dynamic_cast<const CoordinationCenterBase&>(coordinationCenter) }
	, AtomBase{ coordinationCenter.elementSymbol(), coordinationCenter.coordinate()}
	, m_localCoordinateSystem{}
	, m_ionicLigands{}
	, m_metallicLigands{}
{
	coordinationCenter.m_localCoordinateSystem.clear();
	coordinationCenter.m_ionicLigands.clear();
	coordinationCenter.m_metallicLigands.clear();
}

CoordinationCenter& CoordinationCenter::operator=(const CoordinationCenter& coordinationCenter)
{
	CoordinationCenterBase::operator=(dynamic_cast<const CoordinationCenterBase&>(coordinationCenter));
	AtomBase::operator=(dynamic_cast<const AtomBase&>(coordinationCenter));
	m_localCoordinateSystem.clear();
	m_ionicLigands.clear();
	m_metallicLigands.clear();

	return *this;
}

CoordinationCenter& CoordinationCenter::operator=(CoordinationCenter&& coordinationCenter) noexcept
{
	CoordinationCenterBase::operator=(dynamic_cast<const CoordinationCenterBase&>(coordinationCenter));
	AtomBase::operator=(dynamic_cast<const AtomBase&>(coordinationCenter));
	m_localCoordinateSystem.clear();
	m_ionicLigands.clear();
	m_metallicLigands.clear();

	coordinationCenter.m_localCoordinateSystem.clear();
	coordinationCenter.m_ionicLigands.clear();
	coordinationCenter.m_metallicLigands.clear();

	return *this;
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
