#include "CoordinationFeatureSpecifyOptions.h"

#include "LengthCasting.h"
#include "FileReader.h"

using namespace MathematicalCrystalChemistry::Runtime::Analysis;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

CoordinationFeatureSpecifyOptions::CoordinationFeatureSpecifyOptions()
	: m_cifFilesDirectoryPath{}
	, m_workDirectoryPath{}
	, m_outputDirectoryPath{}
	, m_needInfeasibleResult{ false }
	, m_upperIonicBondRadiusNumberExtensionWidth{ 0 }
	, m_repulsiveIonicRadiusNumberExtensionWidth{ 0 }
	, m_maxReoptimizing{ 0 }
	, m_maxAtomFractuationRatio{ 0.0 }
	, m_initialMinimumConstraintCutoffRatio{ 0.0 }
	, m_initialIonicBondCutoffRatio{ 0.0 }
	, m_initialMetallicBondCutoffRatio{ 0.0 }
	, m_chemicalParameters{}
	, m_geometricalConstraintParameters{}
	, m_structuralOptimizationParameters{}
	, m_structureDesignRecordOptions{}
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void CoordinationFeatureSpecifyOptions::initialize() noexcept
{
	m_cifFilesDirectoryPath.clear();
	m_workDirectoryPath.clear();
	m_outputDirectoryPath.clear();

	m_needInfeasibleResult = false;
	m_upperIonicBondRadiusNumberExtensionWidth = 0;
	m_repulsiveIonicRadiusNumberExtensionWidth = 0;
	m_maxReoptimizing = 0;
	m_maxAtomFractuationRatio = 0.0;
	m_initialMinimumConstraintCutoffRatio = 0.0;
	m_initialIonicBondCutoffRatio = 0.0;
	m_initialMetallicBondCutoffRatio = 0.0;

	m_chemicalParameters.initialize();
	m_geometricalConstraintParameters.initialize();
	m_structuralOptimizationParameters.initialize();
	m_structureDesignRecordOptions.initialize();
}

void CoordinationFeatureSpecifyOptions::initializeFromFiles(const std::vector<std::filesystem::path>& filePaths)
{
	initialize();

	std::string inputTexts;
	{
		for (const auto& filePath : filePaths)
		{
			OperatingSystem::IO::FileReader fileReader{ filePath };
			inputTexts += fileReader.readFile();
		}
	}

	using OperatingSystem::IO::FileReader;

	FileReader::tryReadData(inputTexts, "Cif.Data.Directory.Path", m_cifFilesDirectoryPath);
	FileReader::tryReadData(inputTexts, "Work.Directory.Path", m_workDirectoryPath);
	FileReader::tryReadData(inputTexts, "Output.Directory.Path", m_outputDirectoryPath);

	{
		std::string infeasibleResultReportSwitch;
		FileReader::tryReadData(inputTexts, "Report.Infeasible.Result", infeasibleResultReportSwitch);
		readInfeasibleResultNecessity(infeasibleResultReportSwitch);
	}
	FileReader::tryReadData(inputTexts, "Extension.Width.of.Upper.Ionic.Bond.Radius.Number", m_upperIonicBondRadiusNumberExtensionWidth);
	FileReader::tryReadData(inputTexts, "Extension.Width.of.Repulsive.Ionic.Radius.Number", m_repulsiveIonicRadiusNumberExtensionWidth);
	FileReader::tryReadData(inputTexts, "Maximum.Number.of.Reoptimization.Trials", m_maxReoptimizing);
	{
		double angstromFractuation = 0.0;
		FileReader::tryReadData(inputTexts, "Maximum.Atom.Fractuation.Ratio", angstromFractuation);
		m_maxAtomFractuationRatio = MathToolkit::UnitConversion::LengthCasting::cast<MathToolkit::UnitConversion::LengthCasting::Unit::Angstrom, MathToolkit::UnitConversion::LengthCasting::Unit::AtomicUnit>(angstromFractuation);
	}
	FileReader::tryReadData(inputTexts, "Initial.Minimum.Constraint.Cutoff.Ratio", m_initialMinimumConstraintCutoffRatio);
	FileReader::tryReadData(inputTexts, "Initial.Ionic.Bond.Cutoff.Ratio", m_initialIonicBondCutoffRatio);
	FileReader::tryReadData(inputTexts, "Initial.Metallic.Bond.Cutoff.Ratio", m_initialMetallicBondCutoffRatio);


	m_chemicalParameters.initializeFromInputFileTexts(inputTexts);
	m_geometricalConstraintParameters.initializeFromInputFileTexts(inputTexts);
	m_structuralOptimizationParameters.initializeFromInputFileTexts(inputTexts);
	m_structureDesignRecordOptions.initializeFromInputFileTexts(inputTexts);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
