#include "CoordinationFeatureSymbol.h"

#include <regex>

#include "ReportException.h"
#include "AtomCategory.h"

using namespace MathematicalCrystalChemistry::StructureModel::Features;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

std::string CoordinationFeatureSymbol::m_symbolPat{ "([[:alpha:]]+)[\\-]{1}([[:upper:]]{1,2})([[:lower:]]{1,2})[\\^]{1}[\\{]{1}([\\*[:alnum:]]+)*[\\}]{1}[\\-]{1}([[:upper:]]{1,2})[\\-]{1}([[:upper:]]{1,2})([[:lower:]]{1,2})[\\^]{1}[\\{]{1}([\\*[:alnum:]]+)*[\\}]{1}" };


CoordinationFeatureSymbol::CoordinationFeatureSymbol()
	: m_atomCategorySymbol{}
	, m_upperIonicBondRadiusSymbol{ "Z" }
	, m_lowerIonicBondRadiusSymbol{ "z" }
	, m_repulsiveIonicRadiusSymbol{ "Z" }
	, m_upperMetallicBondRadiusSymbol{ "Z" }
	, m_lowerMetallicBondRadiusSymbol{ "z" }
	, m_ionicBondCompositionSymbol{}
	, m_metallicBondCompositionSymbol{}
{
}

CoordinationFeatureSymbol::CoordinationFeatureSymbol(const std::string& name)
	: m_atomCategorySymbol{ readAtomCategorySymbol(name) }
	, m_upperIonicBondRadiusSymbol{ readUpperIonicBondRadiusSymbol(name) }
	, m_lowerIonicBondRadiusSymbol{ readLowerIonicBondRadiusSymbol(name) }
	, m_repulsiveIonicRadiusSymbol{ readRepulsiveIonicRadiusSymbol(name) }
	, m_upperMetallicBondRadiusSymbol{ readUpperMetallicBondRadiusSymbol(name) }
	, m_lowerMetallicBondRadiusSymbol{ readLowerMetallicBondRadiusSymbol(name) }
	, m_ionicBondCompositionSymbol{ readIonicBondCompositionSymbol(name) }
	, m_metallicBondCompositionSymbol{ readMetallicBondCompositionSymbol(name) }
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

void CoordinationFeatureSymbol::setAtomCategorySymbol(const std::string& symbol)
{
	m_atomCategorySymbol = symbol;
}

void CoordinationFeatureSymbol::setUpperIonicBondRadiusSymbol(const std::string& symbol)
{
	m_upperIonicBondRadiusSymbol = symbol;
}

void CoordinationFeatureSymbol::setLowerIonicBondRadiusSymbol(const std::string& symbol)
{
	m_lowerIonicBondRadiusSymbol = symbol;
}

void CoordinationFeatureSymbol::setRepulsiveIonicRadiusSymbol(const std::string& symbol)
{
	m_repulsiveIonicRadiusSymbol = symbol;
}

void CoordinationFeatureSymbol::setUpperMetallicBondRadiusSymbol(const std::string& symbol)
{
	m_upperMetallicBondRadiusSymbol = symbol;
}

void CoordinationFeatureSymbol::setLowerMetallicBondRadiusSymbol(const std::string& symbol)
{
	m_lowerMetallicBondRadiusSymbol = symbol;
}

void CoordinationFeatureSymbol::setIonicBondCompositionSymbol(const std::string& symbol)
{
	m_ionicBondCompositionSymbol = symbol;
}

void CoordinationFeatureSymbol::setMetallicBondCompositionSymbol(const std::string& symbol)
{
	m_metallicBondCompositionSymbol = symbol;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void CoordinationFeatureSymbol::initializeFromElementSymbol(const std::string& elementSymbol)
{
	clear();
	m_atomCategorySymbol = AtomCategory::toAtomCategory(elementSymbol).symbol();
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

std::string CoordinationFeatureSymbol::readAtomCategorySymbol(const std::string& name)
{
	std::regex pat(m_symbolPat);
	std::smatch sm;
	{
		if (std::regex_match(name, sm, pat))
			return sm.str(1);
		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"";
				errorMessage += name;
				errorMessage += "\" is not a name of coordination feature symbol.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException("readUpperIonicBondRadiusSymbol of MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol", errorMessage));
		}
	}
}

std::string CoordinationFeatureSymbol::readUpperIonicBondRadiusSymbol(const std::string& name)
{
	std::regex pat(m_symbolPat);
	std::smatch sm;
	{
		if (std::regex_match(name, sm, pat))
			return sm.str(2);
		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"";
				errorMessage += name;
				errorMessage += "\" is not a name of coordination feature symbol.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException("readUpperIonicBondRadiusSymbol of MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol", errorMessage));
		}
	}
}

std::string CoordinationFeatureSymbol::readLowerIonicBondRadiusSymbol(const std::string& name)
{
	std::regex pat(m_symbolPat);
	std::smatch sm;
	{
		if (std::regex_match(name, sm, pat))
			return sm.str(3);
		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"";
				errorMessage += name;
				errorMessage += "\" is not a name of coordination feature symbol.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException("readLowerIonicBondRadiusSymbol of MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol", errorMessage));
		}
	}
}

std::string CoordinationFeatureSymbol::readIonicBondCompositionSymbol(const std::string& name)
{
	std::regex pat(m_symbolPat);
	std::smatch sm;
	{
		if (std::regex_match(name, sm, pat))
			return sm.str(4);
		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"";
				errorMessage += name;
				errorMessage += "\" is not a name of coordination feature symbol.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException("readIonicBondCompositionSymbol of MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol", errorMessage));
		}
	}
}

std::string CoordinationFeatureSymbol::readRepulsiveIonicRadiusSymbol(const std::string& name)
{
	std::regex pat(m_symbolPat);
	std::smatch sm;
	{
		if (std::regex_match(name, sm, pat))
			return sm.str(5);
		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"";
				errorMessage += name;
				errorMessage += "\" is not a name of coordination feature symbol.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException("readRepulsiveIonicRadiusSymbol of MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol", errorMessage));
		}
	}
}

std::string CoordinationFeatureSymbol::readUpperMetallicBondRadiusSymbol(const std::string& name)
{
	std::regex pat(m_symbolPat);
	std::smatch sm;
	{
		if (std::regex_match(name, sm, pat))
			return sm.str(6);
		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"";
				errorMessage += name;
				errorMessage += "\" is not a name of coordination feature symbol.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException("readUpperMetallicBondRadiusSymbol of MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol", errorMessage));
		}
	}
}

std::string CoordinationFeatureSymbol::readLowerMetallicBondRadiusSymbol(const std::string& name)
{
	std::regex pat(m_symbolPat);
	std::smatch sm;
	{
		if (std::regex_match(name, sm, pat))
			return sm.str(7);
		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"";
				errorMessage += name;
				errorMessage += "\" is not a name of coordination feature symbol.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException("readLowerMetallicBondRadiusSymbol of MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol", errorMessage));
		}
	}
}

std::string CoordinationFeatureSymbol::readMetallicBondCompositionSymbol(const std::string& name)
{
	std::regex pat(m_symbolPat);
	std::smatch sm;
	{
		if (std::regex_match(name, sm, pat))
			return sm.str(8);
		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"";
				errorMessage += name;
				errorMessage += "\" is not a name of coordination feature symbol.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException("readMetallicBondCompositionSymbol of MathematicalCrystalChemistry::StructureModel::Features::CoordinationFeatureSymbol", errorMessage));
		}
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
