#include "CrystalPrototypeDesigner.h"

#include <iostream>
#include <regex>

#include "ThreadController.h"

#include "CoordinationFeatureSpecifyOptions.h"
#include "CoordinationFeatureSpecifier.h"

#include "StructureCreationPlan.h"
#include "StructureCreator.h"

#include "CrystalSelectPlan.h"
#include "CrystalSelector.h"

using namespace OperatingSystem::SystemManagement;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

CrystalPrototypeDesigner::CrystalPrototypeDesigner(int argc, char* argv[])
	: m_commandLineArgument{}
	, m_analysisOptionFilePaths{}
	, m_designOptionFilePaths{}
	, m_searchOptionFilePaths{}
{
	for (int index = 0; index < argc; ++index)
		m_commandLineArgument.push_back(std::string{ argv[index] });
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

int CrystalPrototypeDesigner::execute()
{
	std::cout << "\nHello, world! This is \"CrystalPrototypeDesigner.exe\".\n\n";

	initializeExecutionEnvironment();
	readAnalysisOptionFilePaths();
	readCreationOptionFilePaths();
	readSearchOptionFilePaths();


	if (!(m_analysisOptionFilePaths.empty()))
	{
		MathematicalCrystalChemistry::Runtime::Analysis::CoordinationFeatureSpecifyOptions coordinationFeatureSpecifyOptions;
		coordinationFeatureSpecifyOptions.initializeFromFiles(m_analysisOptionFilePaths);

		MathematicalCrystalChemistry::Runtime::Analysis::CoordinationFeatureSpecifier coordinationFeatureSpecifier{ std::move(coordinationFeatureSpecifyOptions) };
		coordinationFeatureSpecifier.specify();
	}

	if (!(m_designOptionFilePaths.empty()))
	{
		MathematicalCrystalChemistry::Runtime::Creation::StructureCreationPlan structureCreationPlan;
		structureCreationPlan.initializeFromFiles(m_designOptionFilePaths);

		MathematicalCrystalChemistry::Runtime::Creation::StructureCreator structureCreator{ std::move(structureCreationPlan) };
		structureCreator.execute();
	}

	if (!(m_searchOptionFilePaths.empty()))
	{
		MathematicalCrystalChemistry::Runtime::Selection::CrystalSelectPlan crystalSelectPlan;
		crystalSelectPlan.initializeFromFiles(m_searchOptionFilePaths);

		MathematicalCrystalChemistry::Runtime::Selection::CrystalSelector crystalSelector{ std::move(crystalSelectPlan) };
		crystalSelector.execute();
	}

	return 0;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void CrystalPrototypeDesigner::initializeExecutionEnvironment()
{
	size_type threadNumber = 0;
	{
		std::vector<std::string> threadStrings = readCommandLineArgument(std::string{ "-t" });

		if (threadStrings.size() == 1)
			threadNumber = static_cast<size_type>(std::stoi(threadStrings.front()));
		else
		{
			if (1 < threadStrings.size())
				throw std::runtime_error("Could not read the number of thread.");
		}
	}

	OperatingSystem::Configuration::ThreadController::setMaxThreading(threadNumber);
	OperatingSystem::Configuration::ThreadController::setMklThreading(1);
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private utility

std::vector<std::string> CrystalPrototypeDesigner::readCommandLineArgument(const std::string& flag) const
{
	std::vector<std::string> argument;
	{
		auto flagIter = m_commandLineArgument.begin();
		{
			for (flagIter; flagIter != m_commandLineArgument.end(); ++flagIter)
			{
				if ((*flagIter) == flag)
					break;
			}
		}

		if (flagIter != m_commandLineArgument.end())
		{
			for (auto iter = std::next(flagIter); iter != m_commandLineArgument.end(); ++iter)
			{
				std::regex pat("[\\-]{1}([[:alpha:]]+)");
				std::smatch sm;

				if (std::regex_match((*iter), sm, pat))
					break;
				else
					argument.push_back((*iter));
			}
		}
	}

	return argument;
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
