#include "CrystalSelectPlan.h"

#include "FileReader.h"

using namespace MathematicalCrystalChemistry::Runtime::Selection;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void CrystalSelectPlan::initialize()
{
	m_cifDataDirectoryPath.clear();
	m_outputDirectoryPath.clear();

	m_chemicalParameters.initialize();
	m_crystalAnalysisOptions.initialize();
	m_geometricalConstraintParameters.initialize();
	m_structuralOptimizationParameters.initialize();
}

void CrystalSelectPlan::initializeFromFiles(const std::vector<std::filesystem::path>& filePaths)
{
	std::string inputTexts;
	{
		for (const std::filesystem::path& filePath : filePaths)
		{
			OperatingSystem::IO::FileReader fileReader{ filePath };
			inputTexts += fileReader.readFile();

			if (inputTexts.back() != '\n')
				inputTexts += '\n';
		}
	}

	readIOPaths(inputTexts);
	m_chemicalParameters.initializeFromInputFileTexts(inputTexts);
	m_crystalAnalysisOptions.initializeFromFiles(filePaths);
	m_geometricalConstraintParameters.initializeFromInputFileTexts(inputTexts);
	m_structuralOptimizationParameters.initializeFromInputFileTexts(inputTexts);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

void CrystalSelectPlan::readIOPaths(const std::string& fileText)
{
	using OperatingSystem::IO::FileReader;
	{
		std::string cifDataDirectoryPathString;
		FileReader::readData(fileText, "Cif.Data.Directory.Path", cifDataDirectoryPathString);

		m_cifDataDirectoryPath = std::filesystem::path{ cifDataDirectoryPathString };
	}
	{
		std::string outputDirectoryPathString;
		FileReader::readData(fileText, "Output.Directory.Path", outputDirectoryPathString);

		m_outputDirectoryPath = std::filesystem::path{ outputDirectoryPathString };
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
