#include "FileBase.h"

#include "ReportException.h"

using namespace OperatingSystem::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

std::mutex FileBase::m_filesystemMutex;


FileBase::FileBase(const std::filesystem::path& filePath)
	: m_filePath{ filePath }
{
}

FileBase::FileBase(std::filesystem::path&& filePath)
	: m_filePath{ std::move(filePath) }
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods on file system

void FileBase::copyFile(const std::filesystem::path& copyFilePath, FileCopyOptions copyOption) const
{
	if (isValid())
	{
		if (isValidFilePath(copyFilePath))
		{
			std::lock_guard<std::mutex> guard{ m_filesystemMutex };

			switch (copyOption)
			{
			case FileCopyOptions::none:
				std::filesystem::copy_file(m_filePath, copyFilePath, std::filesystem::copy_options::none);
				break;
			case FileCopyOptions::skip_existing:
				std::filesystem::copy_file(m_filePath, copyFilePath, std::filesystem::copy_options::skip_existing);
				break;
			case FileCopyOptions::overwrite_existing:
				std::filesystem::copy_file(m_filePath, copyFilePath, std::filesystem::copy_options::overwrite_existing);
				break;
			case FileCopyOptions::update_existing:
				std::filesystem::copy_file(m_filePath, copyFilePath, std::filesystem::copy_options::update_existing);
				break;
			default:
				OperatingSystem::Configuration::ReportException reportException;
				throw std::invalid_argument(reportException(typeid(*this), "copyFile", "\"copyOption\" is invalid"));
			}
		}

		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"copyFilePath\" of \"";
				errorMessage += copyFilePath.generic_string();
				errorMessage += "\" is not valid.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "copyFile", errorMessage));
		}
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "copyFile", "This is invalid."));
	}
}

void FileBase::renameFile(const std::filesystem::path& newFilePath, FileRenameOptions renameOption)
{
	if (isValid())
	{
		if (isValidFilePath(newFilePath))
		{
			if (renameOption == FileRenameOptions::overwrite_existing)
			{
				std::lock_guard<std::mutex> guard{ m_filesystemMutex };

				if (std::filesystem::exists(newFilePath))
					std::filesystem::remove_all(newFilePath);

				std::filesystem::rename(m_filePath, newFilePath);
				m_filePath = newFilePath;
			}

			else if (renameOption == FileRenameOptions::none)
			{
				if (std::filesystem::is_regular_file(newFilePath))
				{
					std::string errorMessage;
					{
						errorMessage += "New file path of \"";
						errorMessage += newFilePath.generic_string();
						errorMessage += "\" is the path to a existing regular file.";
					}

					OperatingSystem::Configuration::ReportException reportException;
					throw std::runtime_error(reportException(typeid(*this), "renameFile", errorMessage));
				}

				else
				{
					std::lock_guard<std::mutex> guard{ m_filesystemMutex };

					if (std::filesystem::exists(newFilePath))
						std::filesystem::remove_all(newFilePath);

					std::filesystem::rename(m_filePath, newFilePath);
					m_filePath = newFilePath;
				}
			}

			else if (renameOption == FileRenameOptions::skip_existing)
			{
				if (!(std::filesystem::is_regular_file(newFilePath)))
				{
					std::lock_guard<std::mutex> guard{ m_filesystemMutex };

					if (std::filesystem::exists(newFilePath))
						std::filesystem::remove_all(newFilePath);

					std::filesystem::rename(m_filePath, newFilePath);
					m_filePath = newFilePath;
				}
			}

			else
			{
				OperatingSystem::Configuration::ReportException reportException;
				throw std::invalid_argument(reportException(typeid(*this), "renameFile", "\"renameOption\" is invalid."));
			}
		}

		else
		{
			std::string errorMessage;
			{
				errorMessage += "\"newFilePath\" of \"";
				errorMessage += newFilePath.generic_string();
				errorMessage += "\" is not valid.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "renameFile", errorMessage));
		}
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "renameFile", "This is invalid."));
	}
}

// Public methods on file system
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
