#include "FileReader.h"

#include <fstream>

#include "ReportException.h"

using namespace OperatingSystem::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

FileReader::FileReader(const std::filesystem::path& filePath)
	: FileBase{ filePath }
{
}

FileReader::FileReader(std::filesystem::path&& filePath)
	: FileBase{ std::move(filePath) }
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

std::string FileReader::readFile() const
{
	if (isValid())
	{
		std::ifstream inputFileStream;
		inputFileStream.open(filePath(), std::ios_base::in);

		if (inputFileStream)
		{
			std::string inputString;
			{
				for (std::string stringLine; std::getline(inputFileStream, stringLine);)
				{
					inputString += stringLine;
					inputString += "\n";
				}
			}

			inputFileStream.close();
			return inputString;
		}

		else
		{
			std::string errorMessage;
			{
				errorMessage += "Message : Could not open the file of \"";
				errorMessage += filePath().generic_string();
				errorMessage += "\".";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "readFile", errorMessage));
		}
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "readFile", "This is invalid."));
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

template <>
bool FileReader::tryReadData<std::string>(const std::string& inputText, const std::string& dataFlag, std::string& outputText)
{
	std::string dataLine = findDataLine(inputText, dataFlag);

	std::string patString = dataFlag;
	patString += "[\\s]+([\\-\\_\\/\\+\\.\\:[:alnum:]]+){1}[\\s]*";
	std::regex pat(patString);
	std::smatch sm;


	if (std::regex_match(dataLine, sm, pat))
	{
		outputText = sm.str(1);
		return true;
	}

	else
		return false;
}

template <>
bool FileReader::tryReadData<std::filesystem::path>(const std::string& inputText, const std::string& dataFlag, std::filesystem::path& outputText)
{
	std::string dataLine = findDataLine(inputText, dataFlag);

	std::string patString = dataFlag;
	patString += "[\\s]+([\\-\\_\\/\\+\\.\\:[:alnum:]]+){1}[\\s]*";
	std::regex pat(patString);
	std::smatch sm;


	if (std::regex_match(dataLine, sm, pat))
	{
		outputText = std::filesystem::path{ sm.str(1) };
		return true;
	}

	else
		return false;
}

template <>
bool FileReader::tryReadData<std::size_t>(const std::string& inputText, const std::string& dataFlag, std::size_t& outputInteger)
{
	std::string dataLine = findDataLine(inputText, dataFlag);

	std::string patString = dataFlag;
	patString += "[\\s]+([\\d]+){1}[\\s]*";
	std::regex pat(patString);
	std::smatch sm;


	if (std::regex_match(dataLine, sm, pat))
	{
		outputInteger = static_cast<std::size_t>(std::stoi(sm.str(1)));
		return true;
	}

	else
		return false;
}

template <>
bool FileReader::tryReadData<int>(const std::string& inputText, const std::string& dataFlag, int& outputInteger)
{
	std::string dataLine = findDataLine(inputText, dataFlag);

	std::string patString = dataFlag;
	patString += "[\\s]+([\\-\\d]+){1}[\\s]*";
	std::regex pat(patString);
	std::smatch sm;


	if (std::regex_match(dataLine, sm, pat))
	{
		outputInteger = std::stoi(sm.str(1));
		return true;
	}

	else
		return false;
}

template <>
bool FileReader::tryReadData<double>(const std::string& inputText, const std::string& dataFlag, double& outputFloatingPointNumber)
{
	std::string dataLine = findDataLine(inputText, dataFlag);

	std::string patString = dataFlag;
	patString += "[\\s]+([e\\-\\d\\.]+){1}([\\(\\)\\d]+)*[\\s]*";
	std::regex pat(patString);
	std::smatch sm;


	if (std::regex_match(dataLine, sm, pat))
	{
		std::stringstream outputStream{ sm.str(1) };
		outputStream >> outputFloatingPointNumber;
		return true;
	}

	else
		return false;
}

template <>
bool FileReader::tryReadData<std::vector<std::string>>(const std::string& inputText, const std::string& dataFlag, std::vector<std::string>& outputTexts)
{
	outputTexts.clear();
	std::string dataLine = findDataLine(inputText, dataFlag);

	std::string totalPatString = dataFlag;
	totalPatString += "[\\s]+([\\:\\_\\-\\/\\+\\.[:alnum:]]+[\\s]+)*([\\:\\_\\-\\/\\+\\.[:alnum:]]+){1}[\\s]*";
	std::regex totalPat(totalPatString);
	std::smatch totalSm;


	if (std::regex_match(dataLine, totalSm, totalPat))
	{
		std::regex pat("([\\:\\_\\-\\/\\+\\.[:alnum:]]+)");

		for (std::sregex_iterator iter(dataLine.begin(), dataLine.end(), pat); iter != std::sregex_iterator{}; ++iter)
			outputTexts.push_back(iter->str(1));

		outputTexts.erase(outputTexts.begin());
		return true;
	}

	else
		return false;
}

template <>
bool FileReader::tryReadData<std::vector<std::size_t>>(const std::string& inputText, const std::string& dataFlag, std::vector<std::size_t>& outputIntegers)
{
	outputIntegers.clear();
	std::string dataLine = findDataLine(inputText, dataFlag);

	std::string totalPatString = dataFlag;
	totalPatString += "[\\s]+([\\d]+[\\s]+)*([\\d]+){1}[\\s]*";
	std::regex totalPat(totalPatString);
	std::smatch totalSm;


	if (std::regex_match(dataLine, totalSm, totalPat))
	{
		std::regex pat("([\\d]+)");

		for (std::sregex_iterator iter(dataLine.begin(), dataLine.end(), pat); iter != std::sregex_iterator{}; ++iter)
			outputIntegers.push_back(static_cast<std::size_t>(std::stoi(iter->str(1))));

		return true;
	}

	else
		return false;
}

template <>
bool FileReader::tryReadData<std::vector<int>>(const std::string& inputText, const std::string& dataFlag, std::vector<int>& outputIntegers)
{
	outputIntegers.clear();
	std::string dataLine = findDataLine(inputText, dataFlag);

	std::string totalPatString = dataFlag;
	totalPatString += "[\\s]+([\\-\\d]+[\\s]+)*([-\\d]+){1}[\\s]*";
	std::regex totalPat(totalPatString);
	std::smatch totalSm;


	if (std::regex_match(dataLine, totalSm, totalPat))
	{
		std::regex pat("([\\-\\d]+)");

		for (std::sregex_iterator iter(dataLine.begin(), dataLine.end(), pat); iter != std::sregex_iterator{}; ++iter)
			outputIntegers.push_back(std::stoi(iter->str(1)));

		return true;
	}

	else
		return false;
}

template <>
bool FileReader::tryReadData<std::vector<double>>(const std::string& inputText, const std::string& dataFlag, std::vector<double>& outputFloatingPointNumbers)
{
	outputFloatingPointNumbers.clear();
	std::string dataLine = findDataLine(inputText, dataFlag);

	std::string totalPatString = dataFlag;
	totalPatString += "[\\s]+([e\\-\\d\\.]+)*([e\\-\\d\\.]+){1}[\\s]*";
	std::regex totalPat(totalPatString);
	std::smatch totalSm;


	if (std::regex_match(dataLine, totalSm, totalPat))
	{
		std::regex pat("([e\\-\\d\\.]+)");

		for (std::sregex_iterator iter(dataLine.begin(), dataLine.end(), pat); iter != std::sregex_iterator{}; ++iter)
		{
			double value = 0.0;
			{
				std::stringstream outputStream{ iter->str(1) };
				outputStream >> value;
			}

			outputFloatingPointNumbers.push_back(value);
		}

		return true;
	}

	else
		return false;
}

template <>
void FileReader::translateString<int>(const std::string& inputText, int& value)
{
	std::regex pat("([\\-\\d]+)");
	std::smatch sm;

	if (std::regex_match(inputText, sm, pat))
		value = std::stoi(sm.str(1));

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException("OperatingSystem::IO::FileReader::translateString<int>", "\"inputText\" is not a integer number."));
	}
}

template <>
void FileReader::translateString<double>(const std::string& inputText, double& value)
{
	std::regex pat("([e\\-\\.\\d]+)([\\(\\)\\d]+)*");
	std::smatch sm;

	if (std::regex_match(inputText, sm, pat))
	{
		std::stringstream valueStream{ sm.str(1) };
		valueStream >> value;
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException("OperatingSystem::IO::FileReader::translateString<double>", "\"inputText\" is not a floating-point number."));
	}
}

double FileReader::readFraction(const std::string& inputText)
{
	std::regex integerPat{ "([\\-\\d]+)" };
	std::regex fractionPat{ "([\\-\\d]+)/([\\d]+)" };
	std::smatch sm;

	if (std::regex_match(inputText, sm, integerPat))
		return static_cast<double>(std::stoi(sm.str()));

	else if (std::regex_match(inputText, sm, fractionPat))
		return (static_cast<double>(std::stoi(sm.str(1))) / static_cast<double>(std::stoi(sm.str(2))));

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException("OperatingSystem::IO::FileReader::translateIntoTermValue", "\"termText\" is invalid."));
	}
}

std::vector<std::vector<std::string>> FileReader::readDataEnumeration(const std::string& inputText, const std::string& dataFlag)
{
	size_type initialPos = 0;
	{
		std::string initialDataFlag;
		{
			initialDataFlag += "<";
			initialDataFlag += dataFlag;
		}

		initialPos = inputText.find(initialDataFlag);
	}

	size_type endPos = 0;
	{
		std::string endDataFlag;
		{
			endDataFlag += dataFlag;
			endDataFlag += ">";
		}

		endPos = inputText.find(endDataFlag);
	}


	std::vector<std::vector<std::string>> dataEnumeration;
	{
		if (initialPos != std::string::npos && endPos != std::string::npos && initialPos < endPos)
		{
			size_type tempPos = inputText.find_first_of("\n", initialPos);
			std::string enumerationText{ inputText, (tempPos + 1), (endPos - tempPos - 1) };
			std::stringstream enumerationStream{ enumerationText };

			std::regex totalPat("[\\s]*([\\*\\.\\-\\_\\^\\{\\}[:alnum:]]+[\\s]+)*([\\*\\.\\-\\_\\^\\{\\}[:alnum:]]+){1}[\\s]*[\\n]*");
			std::regex pat("([\\*\\.\\-\\_\\^\\{\\}[:alnum:]]+)");


			for (std::string stringLine; std::getline(enumerationStream, stringLine);)
			{
				std::smatch totalSm;

				if (std::regex_match(stringLine, totalSm, totalPat))
				{
					std::vector<std::string> lineData;
					{
						for (std::sregex_iterator iter(stringLine.begin(), stringLine.end(), pat); iter != std::sregex_iterator{}; ++iter)
							lineData.push_back(iter->str(1));
					}
					
					dataEnumeration.push_back(std::move(lineData));
				}
			}
		}
	}

	return dataEnumeration;
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
