#include "FileWriter.h"

#include <sstream>
#include <fstream>

#include "ReportException.h"

using namespace OperatingSystem::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

std::mutex FileWriter::m_outputMutex;


FileWriter::FileWriter(const std::filesystem::path& filePath)
	: FileBase{ filePath }
{
}

FileWriter::FileWriter(std::filesystem::path&& filePath)
	: FileBase{ std::move(filePath) }
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void FileWriter::createFile(FileCreateOptions createOption, const std::string& message) const
{
	if (std::filesystem::is_directory(filePath().parent_path()))
	{
		if (createOption == FileCreateOptions::none)
		{
			if (std::filesystem::is_regular_file(filePath()))
			{
				std::string errorMessage;
				{
					errorMessage += "\"";
					errorMessage += filePath().string();
					errorMessage += "\" is a path to a existing file.";
				}

				OperatingSystem::Configuration::ReportException reportException;
				throw std::runtime_error(reportException(typeid(*this), "createFile", errorMessage));
			}

			else
				overCreate(message);
		}

		else if (createOption == FileCreateOptions::skip_existing)
		{
			if (!(std::filesystem::is_regular_file(filePath())))
				overCreate(message);
		}

		else if (createOption == FileCreateOptions::overwrite_existing)
			overCreate(message);

		else
		{
			OperatingSystem::Configuration::ReportException reportException;
			throw std::invalid_argument(reportException(typeid(*this), "createFile", "\"createOption\" is invalid."));
		}
	}

	else
	{
		std::string errorMessage;
		{
			errorMessage += "Parent directory path of \"";
			errorMessage += filePath().generic_string();
			errorMessage += "\" does not exist.";
		}

		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "createFile", errorMessage));
	}
}

void FileWriter::write(const std::string& message) const
{
	if (!(message.empty()))
	{
		if (isValid())
		{
			//std::lock_guard<std::mutex> guard{ m_outputMutex };

			std::ofstream outputFileStream;
			outputFileStream.open(filePath(), std::ios_base::app);

			if (outputFileStream)
			{
				outputFileStream << message;
				outputFileStream.close();
			}

			else
			{
				std::string errorMessage;
				{
					errorMessage += "Message : Could not open the file of \"";
					errorMessage += filePath().generic_string();
					errorMessage += "\".";
				}

				OperatingSystem::Configuration::ReportException reportException;
				throw std::runtime_error(reportException(typeid(*this), "write", errorMessage));
			}
		}

		else
		{
			std::string errorMessage;
			{
				errorMessage += "Message : The file path of \"";
				errorMessage += filePath().generic_string();
				errorMessage += "\" is invalid.";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "write", errorMessage));
		}
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

std::string FileWriter::toString(const double value) noexcept
{
	std::string outputText;
	{
		std::stringstream outputStream;
		outputStream.precision(12);
		outputStream.flags(std::ios_base::fixed);
		outputStream << value;

		outputStream >> outputText;
	}

	return outputText;
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void FileWriter::overCreate(const std::string& message) const
{
	//std::lock_guard<std::mutex> guard{ m_outputMutex };

	std::ofstream outputFileStream;
	outputFileStream.open(filePath(), std::ios_base::out);

	if (outputFileStream)
	{
		if (message.empty())
			outputFileStream.close();

		else
		{
			outputFileStream << message;
			outputFileStream.close();
		}
	}

	else
	{
		std::string errorMessage;
		{
			errorMessage += "Could not create the file of \"";
			errorMessage += filePath().generic_string();
			errorMessage += "\".";
		}

		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "createFile", errorMessage));
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
