#include "GeometricalConstraintParameters.h"

#include "FileReader.h"

#include "IonicBond.h"
#include "MetallicBond.h"
#include "AnionicConstraint.h"
#include "CationicConstraint.h"
#include "NonIonicBond.h"
#include "NonMetallicBond.h"

using namespace MathematicalCrystalChemistry::StructureModel::GeometricalConstraints;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

GeometricalConstraintParameters::GeometricalConstraintParameters()
	: m_ionicBondCutoffLengthRatio{ MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::IonicBond::defaultCutoffRatio() }
	, m_metallicBondCutoffLengthRatio{ MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::MetallicBond::defaultCutoffRatio() }
	, m_anionicConstraintCutoffLengthRatio{ MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::AnionicConstraint::defaultCutoffRatio() }
	, m_nonIonicBondCutoffLengthRatio{ MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::NonIonicBond::defaultCutoffRatio() }
	, m_nonMetallicBondCutoffLengthRatio{ MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::NonMetallicBond::defaultCutoffRatio() }
	, m_nonIonicBondRangeRatio{ MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::NonIonicBond::defaultNonBondingRangeRatio() }
	, m_nonMetallicBondRangeRatio{ MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::NonMetallicBond::defaultNonBondingRangeRatio() }
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void GeometricalConstraintParameters::initialize()
{
	m_ionicBondCutoffLengthRatio = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::IonicBond::defaultCutoffRatio();
	m_metallicBondCutoffLengthRatio = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::MetallicBond::defaultCutoffRatio();
	m_anionicConstraintCutoffLengthRatio = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::AnionicConstraint::defaultCutoffRatio();
	m_nonIonicBondCutoffLengthRatio = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::NonIonicBond::defaultCutoffRatio();
	m_nonMetallicBondCutoffLengthRatio = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::NonMetallicBond::defaultCutoffRatio();

	m_nonIonicBondRangeRatio = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::NonIonicBond::defaultNonBondingRangeRatio();
	m_nonMetallicBondRangeRatio = MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::NonMetallicBond::defaultNonBondingRangeRatio();
}

void GeometricalConstraintParameters::initializeFromInputFileTexts(const std::string& inputTexts)
{
	initialize();
	using OperatingSystem::IO::FileReader;

	FileReader::tryReadData(inputTexts, "Ionic.Bond.Cutoff.Length.Ratio", m_ionicBondCutoffLengthRatio);
	FileReader::tryReadData(inputTexts, "Metallic.Bond.Cutoff.Length.Ratio", m_metallicBondCutoffLengthRatio);
	FileReader::tryReadData(inputTexts, "Anionic.Constraint.Cutoff.Length.Ratio", m_anionicConstraintCutoffLengthRatio);
	FileReader::tryReadData(inputTexts, "NonIonic.Bond.Cutoff.Length.Ratio", m_nonIonicBondCutoffLengthRatio);
	FileReader::tryReadData(inputTexts, "NonMetallic.Bond.Cutoff.Length.Ratio", m_nonMetallicBondCutoffLengthRatio);

	FileReader::tryReadData(inputTexts, "NonIonic.Bond.Range.Ratio", m_nonIonicBondRangeRatio);
	FileReader::tryReadData(inputTexts, "NonMetallic.Bond.Range.Ratio", m_nonMetallicBondRangeRatio);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
