#include "IonicBond.h"

using namespace MathematicalCrystalChemistry::StructureModel::GeometricalConstraints;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private constructors

double IonicBond::m_defaultCutoffRatio{ 2.0 };


IonicBond::IonicBond(ModelAtom& modelAtom, const Ligand& li)
	: MinMaxDistanceConstraint{ modelAtom, li }
{
	const double minimumDistance = modelAtomRef().coordinationFeature().lowerIonicBondRadius() + ligand().get().coordinationFeature().lowerIonicBondRadius();
	const double maximumDistance = modelAtomRef().coordinationFeature().upperIonicBondRadius() + ligand().get().coordinationFeature().upperIonicBondRadius();
	MinMaxDistanceConstraint::setMinMaxDistances(minimumDistance, maximumDistance);
}

IonicBond::IonicBond(ModelAtom& modelAtom, Ligand&& li)
	: MinMaxDistanceConstraint{ modelAtom, std::move(li) }
{
	const double minimumDistance = modelAtomRef().coordinationFeature().lowerIonicBondRadius() + ligand().get().coordinationFeature().lowerIonicBondRadius();
	const double maximumDistance = modelAtomRef().coordinationFeature().upperIonicBondRadius() + ligand().get().coordinationFeature().upperIonicBondRadius();
	MinMaxDistanceConstraint::setMinMaxDistances(minimumDistance, maximumDistance);
}

IonicBond::IonicBond(const Ligand& la, const Ligand& lb)
	: MinMaxDistanceConstraint{ la, lb }
{
	const double minimumDistance = modelAtomRef().coordinationFeature().lowerIonicBondRadius() + ligand().get().coordinationFeature().lowerIonicBondRadius();
	const double maximumDistance = modelAtomRef().coordinationFeature().upperIonicBondRadius() + ligand().get().coordinationFeature().upperIonicBondRadius();
	MinMaxDistanceConstraint::setMinMaxDistances(minimumDistance, maximumDistance);
}

IonicBond::IonicBond(const std::shared_ptr<IonicBond>& reversePtr)
	: MinMaxDistanceConstraint{ std::dynamic_pointer_cast<MinMaxDistanceConstraint>(reversePtr) }
{
}

// Private constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
