#include "LogWriter.h"

#include <fstream>

#include "Clock.h"

using namespace OperatingSystem::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

std::mutex LogWriter::m_logMutex;


LogWriter::LogWriter()
	: m_logFilePath{}
{
}

LogWriter::LogWriter(const std::filesystem::path& directoryPath)
	: m_logFilePath{ directoryPath }
{
	m_logFilePath /= getLogFilename();
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void LogWriter::write(const std::string& message) const
{
	if (!(message.empty()))
	{
		std::string outputText{ "\n" };
		{
			outputText += OperatingSystem::Configuration::Clock::writeZonedTime();
			outputText += "\n\tInformation: ";
			outputText += message;

			if(outputText.back() != '\n')
				outputText += '\n';
		}

		outputMessage(outputText);
	}
}

void OperatingSystem::IO::LogWriter::write(const std::exception& e) const
{
	if (auto invalidArgumentPtr = dynamic_cast<const std::invalid_argument*>(&e))
		write(*invalidArgumentPtr);
	else if (auto logicErrorPtr = dynamic_cast<const std::logic_error*>(&e))
		write(*logicErrorPtr);
	else if (auto runtimeErrorPtr = dynamic_cast<const std::runtime_error*>(&e))
		write(*runtimeErrorPtr);
	else if (auto outOfRangePtr = dynamic_cast<const std::out_of_range*>(&e))
		write(*outOfRangePtr);

	else
	{
		std::string outputText{ "\n" };
		{
			outputText += OperatingSystem::Configuration::Clock::writeZonedTime();
			outputText += "\n\tException: ";
			outputText += e.what();

			if (outputText.back() != '\n')
				outputText += '\n';
		}

		outputMessage(outputText);
	}
}

void OperatingSystem::IO::LogWriter::write(const std::invalid_argument& invalidArgument) const
{
	std::string errorMessage{ "\n" };
	{
		errorMessage += OperatingSystem::Configuration::Clock::writeZonedTime();
		errorMessage += "\n\tInvalid argument: ";
		errorMessage += invalidArgument.what();

		if (errorMessage.back() != '\n')
			errorMessage += '\n';
	}

	outputMessage(errorMessage);
}

void OperatingSystem::IO::LogWriter::write(const std::logic_error& logicError) const
{
	std::string errorMessage{ "\n" };
	{
		errorMessage += OperatingSystem::Configuration::Clock::writeZonedTime();
		errorMessage += "\n\tLogic error: ";
		errorMessage += logicError.what();

		if (errorMessage.back() != '\n')
			errorMessage += '\n';
	}

	outputMessage(errorMessage);
}

void OperatingSystem::IO::LogWriter::write(const std::runtime_error& runtimeError) const
{
	std::string errorMessage{ "\n" };
	{
		errorMessage += OperatingSystem::Configuration::Clock::writeZonedTime();
		errorMessage += "\n\tRuntime error: ";
		errorMessage += runtimeError.what();

		if (errorMessage.back() != '\n')
			errorMessage += '\n';
	}

	outputMessage(errorMessage);
}

void OperatingSystem::IO::LogWriter::write(const std::out_of_range& outOfRange) const
{
	std::string errorMessage{ "\n" };
	{
		errorMessage += OperatingSystem::Configuration::Clock::writeZonedTime();
		errorMessage += "\n\tOut of range: ";
		errorMessage += outOfRange.what();

		if (errorMessage.back() != '\n')
			errorMessage += '\n';
	}

	outputMessage(errorMessage);
}

// Public methods and static members
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void LogWriter::createFile(const std::string& outputText) const
{
	if (m_logFilePath.empty())
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "createFile", "Log file path is empty."));
	}

	else
	{
		std::ofstream outputFileStream;
		outputFileStream.open(m_logFilePath, std::ios_base::out);

		if (outputFileStream)
		{
			outputFileStream << writeHeaderMessage();

			if (!(outputText.empty()))
				outputFileStream << outputText;

			outputFileStream.close();
		}

		else
		{
			std::string errorMessage;
			{
				errorMessage += "Could not create the log file of \"";
				errorMessage += m_logFilePath.generic_string();
				errorMessage += "\".";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "createFile", errorMessage));
		}
	}
}

std::string LogWriter::writeHeaderMessage() const
{
	std::string headerMessage;
	{
		headerMessage += "----------------------------------------------------------------------------------------------------\n\n";
		headerMessage += OperatingSystem::Configuration::Clock::writeZonedTime();
		headerMessage += "\n\tLog file is created.\n\n";
		headerMessage += "----------------------------------------------------------------------------------------------------\n\n";
	}

	return headerMessage;
}

void LogWriter::outputMessage(const std::string& message) const
{
	if (isAvailable())
	{
		std::lock_guard logLockGuard{ m_logMutex };

		std::ofstream outputFileStream;
		outputFileStream.open(m_logFilePath, std::ios_base::app);

		if (outputFileStream)
		{
			outputFileStream << message;
			outputFileStream.close();
		}

		else
		{
			std::string errorMessage;
			{
				errorMessage += "Message : Could not open the file of \"";
				errorMessage += m_logFilePath.generic_string();
				errorMessage += "\".";
			}

			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "outputMessage", errorMessage));
		}
	}

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "outputMessage", "Log file has not yet been available."));
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
