#include "MetallicBond.h"

using namespace MathematicalCrystalChemistry::StructureModel::GeometricalConstraints;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private constructors

double MetallicBond::m_defaultCutoffRatio{ 2.0 };


MetallicBond::MetallicBond(ModelAtom& modelAtom, const Ligand& li)
	: MinMaxDistanceConstraint{ modelAtom, li }
{
	const double maximumDistance = modelAtomRef().coordinationFeature().upperMetallicBondRadius() + ligand().get().coordinationFeature().upperMetallicBondRadius();
	const double minimumDistance = modelAtomRef().coordinationFeature().lowerMetallicBondRadius() + ligand().get().coordinationFeature().lowerMetallicBondRadius();
	MinMaxDistanceConstraint::setMinMaxDistances(minimumDistance, maximumDistance);
}

MetallicBond::MetallicBond(ModelAtom& modelAtom, Ligand&& li)
	: MinMaxDistanceConstraint{ modelAtom, std::move(li) }
{
	const double maximumDistance = modelAtomRef().coordinationFeature().upperMetallicBondRadius() + ligand().get().coordinationFeature().upperMetallicBondRadius();
	const double minimumDistance = modelAtomRef().coordinationFeature().lowerMetallicBondRadius() + ligand().get().coordinationFeature().lowerMetallicBondRadius();
	MinMaxDistanceConstraint::setMinMaxDistances(minimumDistance, maximumDistance);
}

MetallicBond::MetallicBond(const Ligand& la, const Ligand& lb)
	: MinMaxDistanceConstraint{ la, lb }
{
	const double maximumDistance = modelAtomRef().coordinationFeature().upperMetallicBondRadius() + ligand().get().coordinationFeature().upperMetallicBondRadius();
	const double minimumDistance = modelAtomRef().coordinationFeature().lowerMetallicBondRadius() + ligand().get().coordinationFeature().lowerMetallicBondRadius();
	MinMaxDistanceConstraint::setMinMaxDistances(minimumDistance, maximumDistance);
}

MetallicBond::MetallicBond(const std::shared_ptr<MetallicBond>& reversePtr)
	: MinMaxDistanceConstraint{ std::dynamic_pointer_cast<MinMaxDistanceConstraint>(reversePtr) }
{
}

// Private constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
