#include "MinMaxDistanceConstraint.h"

#include <limits>

using namespace MathematicalCrystalChemistry::StructureModel::GeometricalConstraints::Internal;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Protected constructors

MinMaxDistanceConstraint::MinMaxDistanceConstraint(ModelAtom& modelAtom, const Ligand& ligand)
	: InteratomicConstraintBase{ modelAtom, ligand }
	, m_maximumDistance{ std::numeric_limits<double>::max() }
	, m_minimumDistance{ 0.0 }
{
}

MinMaxDistanceConstraint::MinMaxDistanceConstraint(ModelAtom& modelAtom, Ligand&& ligand)
	: InteratomicConstraintBase{ modelAtom, std::move(ligand) }
	, m_maximumDistance{ std::numeric_limits<double>::max() }
	, m_minimumDistance{ 0.0 }
{
}

MinMaxDistanceConstraint::MinMaxDistanceConstraint(const Ligand& la, const Ligand& lb)
	: InteratomicConstraintBase{ la, lb }
	, m_maximumDistance{ std::numeric_limits<double>::max() }
	, m_minimumDistance{ 0.0 }
{
}

MinMaxDistanceConstraint::MinMaxDistanceConstraint(const std::shared_ptr<MinMaxDistanceConstraint>& reversePtr)
	: InteratomicConstraintBase{ std::dynamic_pointer_cast<InteratomicConstraintBase>(reversePtr) }
	, m_maximumDistance{ reversePtr->m_maximumDistance }
	, m_minimumDistance{ reversePtr->m_minimumDistance }
{
}

// Protected constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void MinMaxDistanceConstraint::initialize() noexcept
{
	m_maximumDistance = std::numeric_limits<double>::max();
	m_minimumDistance = 0.0;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
