#include "NonIonicBond.h"

using namespace MathematicalCrystalChemistry::StructureModel::GeometricalConstraints;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

double NonIonicBond::m_defaultNonBondingRangeRatio{ 1.2 };
double NonIonicBond::m_defaultCutoffRatio{ 2.0 };


NonIonicBond::NonIonicBond(ModelAtom& modelAtom, const Ligand& li)
	: MinimumDistanceConstraint{ modelAtom, li }
	, m_nonBondingRangeRatio{ m_defaultNonBondingRangeRatio }
{
	MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperIonicBondRadius() + ligand().get().coordinationFeature().upperIonicBondRadius()));
}

NonIonicBond::NonIonicBond(ModelAtom& modelAtom, const Ligand& li, const double val)
	: MinimumDistanceConstraint{ modelAtom, li }
	, m_nonBondingRangeRatio{ val }
{
	if (0.0 < m_nonBondingRangeRatio)
		MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperIonicBondRadius() + ligand().get().coordinationFeature().upperIonicBondRadius()));
	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "constructor", "\"nonBondingRangeRatio\" is not more than zero."));
	}
}

NonIonicBond::NonIonicBond(ModelAtom& modelAtom, Ligand&& li)
	: MinimumDistanceConstraint{ modelAtom, std::move(li) }
	, m_nonBondingRangeRatio{ m_defaultNonBondingRangeRatio }
{
	MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperIonicBondRadius() + ligand().get().coordinationFeature().upperIonicBondRadius()));
}

NonIonicBond::NonIonicBond(ModelAtom& modelAtom, Ligand&& li, const double val)
	: MinimumDistanceConstraint{ modelAtom, std::move(li) }
	, m_nonBondingRangeRatio{ val }
{
	if (0.0 < m_nonBondingRangeRatio)
		MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperIonicBondRadius() + ligand().get().coordinationFeature().upperIonicBondRadius()));
	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "constructor", "\"nonBondingRangeRatio\" is not more than zero."));
	}
}

NonIonicBond::NonIonicBond(const Ligand& la, const Ligand& lb)
	: MinimumDistanceConstraint{ la, lb }
	, m_nonBondingRangeRatio{ m_defaultNonBondingRangeRatio }
{
	MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperIonicBondRadius() + ligand().get().coordinationFeature().upperIonicBondRadius()));
}

NonIonicBond::NonIonicBond(const Ligand& la, const Ligand& lb, const double val)
	: MinimumDistanceConstraint{ la, lb }
	, m_nonBondingRangeRatio{ val }
{
	if (0.0 < m_nonBondingRangeRatio)
		MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperIonicBondRadius() + ligand().get().coordinationFeature().upperIonicBondRadius()));
	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "constructor", "\"nonBondingRangeRatio\" is not more than zero."));
	}
}

NonIonicBond::NonIonicBond(const std::shared_ptr<NonIonicBond>& reversePtr)
	: MinimumDistanceConstraint{ std::static_pointer_cast<MinimumDistanceConstraint>(reversePtr) }
	, m_nonBondingRangeRatio{ reversePtr->m_nonBondingRangeRatio }
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
