#include "NonMetallicBond.h"

using namespace MathematicalCrystalChemistry::StructureModel::GeometricalConstraints;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

double NonMetallicBond::m_defaultNonBondingRangeRatio{ 1.2 };
double NonMetallicBond::m_defaultCutoffRatio{ 2.0 };


NonMetallicBond::NonMetallicBond(ModelAtom& modelAtom, const Ligand& li)
	: MinimumDistanceConstraint{ modelAtom, li }
	, m_nonBondingRangeRatio{ m_defaultNonBondingRangeRatio }
{
	MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperMetallicBondRadius() + ligand().get().coordinationFeature().upperMetallicBondRadius()));
}

NonMetallicBond::NonMetallicBond(ModelAtom& modelAtom, const Ligand& li, const double val)
	: MinimumDistanceConstraint{ modelAtom, li }
	, m_nonBondingRangeRatio{ val }
{
	if (0.0 < m_nonBondingRangeRatio)
		MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperMetallicBondRadius() + ligand().get().coordinationFeature().upperMetallicBondRadius()));
	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "constructor", "\"nonBondingRangeRatio\" is not more than zero."));
	}
}

NonMetallicBond::NonMetallicBond(ModelAtom& modelAtom, Ligand&& li)
	: MinimumDistanceConstraint{ modelAtom, std::move(li) }
	, m_nonBondingRangeRatio{ m_defaultNonBondingRangeRatio }
{
	MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperMetallicBondRadius() + ligand().get().coordinationFeature().upperMetallicBondRadius()));
}

NonMetallicBond::NonMetallicBond(ModelAtom& modelAtom, Ligand&& li, const double val)
	: MinimumDistanceConstraint{ modelAtom, std::move(li) }
	, m_nonBondingRangeRatio{ val }
{
	if (0.0 < m_nonBondingRangeRatio)
		MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperMetallicBondRadius() + ligand().get().coordinationFeature().upperMetallicBondRadius()));
	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "constructor", "\"nonBondingRangeRatio\" is not more than zero."));
	}
}

NonMetallicBond::NonMetallicBond(const Ligand& la, const Ligand& lb)
	: MinimumDistanceConstraint{ la, lb }
	, m_nonBondingRangeRatio{ m_defaultNonBondingRangeRatio }
{
	MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperMetallicBondRadius() + ligand().get().coordinationFeature().upperMetallicBondRadius()));
}

NonMetallicBond::NonMetallicBond(const Ligand& la, const Ligand& lb, const double val)
	: MinimumDistanceConstraint{ la, lb }
	, m_nonBondingRangeRatio{ val }
{
	if (0.0 < m_nonBondingRangeRatio)
		MinimumDistanceConstraint::setMinimumDistance(m_nonBondingRangeRatio * (modelAtomRef().coordinationFeature().upperMetallicBondRadius() + ligand().get().coordinationFeature().upperMetallicBondRadius()));
	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "constructor", "\"nonBondingRangeRatio\" is not more than zero."));
	}
}

NonMetallicBond::NonMetallicBond(const std::shared_ptr<NonMetallicBond>& reversePtr)
	: MinimumDistanceConstraint{ std::static_pointer_cast<MinimumDistanceConstraint>(reversePtr) }
	, m_nonBondingRangeRatio{ reversePtr->m_nonBondingRangeRatio }
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
