#include "NumericalMatrixBase.h"

using namespace MathToolkit::Generic::Internal;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

template <>
NumericalMatrixBase<double>::NumericalMatrixBase()
	: m_valueArray{}
	, m_columnSize{ 0 }
	, m_rowSize{ 0 }
{
}

template <>
NumericalMatrixBase<std::complex<double>>::NumericalMatrixBase()
	: m_valueArray{}
	, m_columnSize{ 0 }
	, m_rowSize{ 0 }
{
}

template <>
NumericalMatrixBase<double>::NumericalMatrixBase(const size_type columnSize, const size_type rowSize, const value_type value)
	: m_valueArray(value, (columnSize* rowSize))
	, m_columnSize{ columnSize }
	, m_rowSize{ rowSize }
{
	if (m_columnSize == 0 || m_rowSize == 0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "constructor", "\"columnSize\" and/or \"rowSize\" are zero."));
	}
}

template <>
NumericalMatrixBase<std::complex<double>>::NumericalMatrixBase(const size_type columnSize, const size_type rowSize, const value_type value)
	: m_valueArray(value, (columnSize* rowSize))
	, m_columnSize{ columnSize }
	, m_rowSize{ rowSize }
{
	if (m_columnSize == 0 || m_rowSize == 0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "constructor", "\"columnSize\" and/or \"rowSize\" are zero."));
	}
}

template <>
NumericalMatrixBase<double>::NumericalMatrixBase(const double scalingMultiplier, const NumericalVectorBase<value_type>& nvba, const NumericalVectorBase<value_type>& nvbb)
	: m_valueArray(0.0, (nvba.size()* nvbb.size()))
	, m_columnSize{ nvba.size() }
	, m_rowSize{ nvbb.size() }
{
	auto thisIter = std::begin(m_valueArray);

	for (auto rightVectorIter = nvbb.begin(); rightVectorIter != nvbb.end(); ++rightVectorIter)
	{
		for (auto leftVectorIter = nvba.begin(); leftVectorIter != nvba.end(); ++leftVectorIter)
		{
			(*thisIter) = (*leftVectorIter) * (*rightVectorIter);
			++thisIter;
		}
	}
}

template <>
NumericalMatrixBase<std::complex<double>>::NumericalMatrixBase(const double scalingMultiplier, const NumericalVectorBase<value_type>& nvba, const NumericalVectorBase<value_type>& nvbb)
	: m_valueArray(0.0, (nvba.size()* nvbb.size()))
	, m_columnSize{ nvba.size() }
	, m_rowSize{ nvbb.size() }
{
	auto thisIter = std::begin(m_valueArray);

	for (auto rightVectorIter = nvbb.begin(); rightVectorIter != nvbb.end(); ++rightVectorIter)
	{
		for (auto leftVectorIter = nvba.begin(); leftVectorIter != nvba.end(); ++leftVectorIter)
		{
			(*thisIter) = (*leftVectorIter) * (*rightVectorIter);
			++thisIter;
		}
	}
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
