#include "NumericalSymmetricMatrix.h"

#include <memory>
#include <mkl.h>

#include "NumericalMatrix.h"

using namespace MathToolkit::Generic;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Mathematical methods

NumericalSymmetricMatrix& NumericalSymmetricMatrix::invert()
{
	NumericalSymmetricMatrix inverseMatrix = getIdentitySymmetricMatrix(dimension());
	std::unique_ptr<lapack_int[]> ipiv{ new lapack_int[dimension()] };
	lapack_int n = static_cast<lapack_int>(dimension());
	{
		lapack_int info = LAPACKE_dspsv(LAPACK_COL_MAJOR, 'U', n, n, begin(), ipiv.get(), inverseMatrix.begin(), n);

		if (info != 0)
		{
			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "invert", "Could not find inverse matrix."));
		}
	}

	this->swap(inverseMatrix);
	return *this;	
}

std::vector<std::pair<double, NumericalVector<double>>> NumericalSymmetricMatrix::getEigenValueAndVectors() const
{
	NumericalSymmetricMatrix ap{ *this };
	std::unique_ptr<double[]> w{ new double[dimension()] };
	NumericalMatrix<double> z(dimension(), dimension());
	lapack_int n = static_cast<lapack_int>(dimension());
	{
		lapack_int info = LAPACKE_dspev(LAPACK_COL_MAJOR, 'V', 'U', n, ap.begin(), w.get(), z.begin(), n);

		if (info != 0)
		{
			OperatingSystem::Configuration::ReportException reportException;
			throw std::runtime_error(reportException(typeid(*this), "getEigenValueAndVectors", "Could not find eigenvalues and eigenvectors."));
		}
	}


	std::vector<std::pair<double, MathToolkit::Generic::NumericalVector<double>>> eigenvalueAndVectors;
	{
		for (size_type index = 0; index < dimension(); ++index)
			eigenvalueAndVectors.push_back(std::make_pair(w[index], z.columnVector(index)));
	}

	return eigenvalueAndVectors;
}

// Mathematical methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
