#include "RandomStructureGenerationParameters.h"

#include "FileReader.h"
#include "AngleCasting.h"

using namespace MathematicalCrystalChemistry::Design::Generation;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

RandomStructureGenerationParameters::RandomStructureGenerationParameters()
	: m_initialPackingFraction{ 0.7 }
	, m_maxInitialLatticeLengthRatio{ 3.0 }
	, m_maxInitialLatticeAngle{ MathToolkit::UnitConversion::AngleCasting::cast<MathToolkit::UnitConversion::AngleCasting::Unit::Radian, MathToolkit::UnitConversion::AngleCasting::Unit::Degree>(120) }
	, m_minInitialLatticeAngle{ MathToolkit::UnitConversion::AngleCasting::cast<MathToolkit::UnitConversion::AngleCasting::Unit::Radian, MathToolkit::UnitConversion::AngleCasting::Unit::Degree>(60) }
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void RandomStructureGenerationParameters::initialize()
{
	m_initialPackingFraction = 0.7;
	m_maxInitialLatticeLengthRatio = 3.0;
	m_maxInitialLatticeAngle = MathToolkit::UnitConversion::AngleCasting::cast<MathToolkit::UnitConversion::AngleCasting::Unit::Radian, MathToolkit::UnitConversion::AngleCasting::Unit::Degree>(120);
	m_minInitialLatticeAngle = MathToolkit::UnitConversion::AngleCasting::cast<MathToolkit::UnitConversion::AngleCasting::Unit::Radian, MathToolkit::UnitConversion::AngleCasting::Unit::Degree>(60);
}

void RandomStructureGenerationParameters::initializeFromInputFileTexts(const std::string& inputTexts)
{
	initialize();

	std::string parameterTexts;
	{
		for (const auto& lineData : OperatingSystem::IO::FileReader::readDataEnumeration(inputTexts, "Random.Structure.Generation.Parameters"))
		{
			for (const auto& text : lineData)
			{
				parameterTexts += text;
				parameterTexts += "  ";
			}

			parameterTexts.pop_back();
			parameterTexts.pop_back();
			parameterTexts += "\n";
		}
	}

	using FileReader = OperatingSystem::IO::FileReader;

	FileReader::tryReadData(parameterTexts, "Initial.Packing.Fraction", m_initialPackingFraction);
	FileReader::tryReadData(parameterTexts, "Maximum.Initial.Lattice.Length.Ratio", m_maxInitialLatticeLengthRatio);
	{
		double inputValue = 0.0;
		FileReader::tryReadData(parameterTexts, "Maximum.Initial.Lattice.Angle", inputValue);
		m_maxInitialLatticeAngle = MathToolkit::UnitConversion::AngleCasting::cast<MathToolkit::UnitConversion::AngleCasting::Unit::Degree, MathToolkit::UnitConversion::AngleCasting::Unit::Radian>(inputValue);
	}
	{
		double inputValue = 0.0;
		FileReader::tryReadData(parameterTexts, "Minimum.Initial.Lattice.Angle", inputValue);

		m_minInitialLatticeAngle = MathToolkit::UnitConversion::AngleCasting::cast<MathToolkit::UnitConversion::AngleCasting::Unit::Degree, MathToolkit::UnitConversion::AngleCasting::Unit::Radian>(inputValue);
	}

	validateParameters();
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void RandomStructureGenerationParameters::validateParameters() const
{
	if (m_initialPackingFraction <= 0.0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "validateParameters", "\"Initial.Packing.Fraction\" is not more than zero."));
	}

	if (m_maxInitialLatticeLengthRatio < 1.0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "validateParameters", "\"Maximum.Initial.Lattice.Length.Ratio\" is less than one."));
	}

	if (m_maxInitialLatticeAngle <= 0.0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "validateParameters", "\"Maximum.Initial.Lattice.Angle\" is not more than zero."));
	}

	if (m_minInitialLatticeAngle <= 0.0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "validateParameters", "\"Minimum.Initial.Lattice.Angle\" is not more than zero."));
	}

	if (m_maxInitialLatticeAngle < m_minInitialLatticeAngle)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "validateParameters", "\"Maximum.Initial.Lattice.Angle\" is less than \"Minimum.Initial.Lattice.Angle\"."));
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
