#include "SelectCrystals.h"

#include "LogWriter.h"

#include "CifFileExcavator.h"

using namespace MathematicalCrystalChemistry::Runtime::Selection::Internal;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

SelectCrystals::SelectCrystals(const size_type threadRank)
	: m_threadRank{ threadRank }
	, m_cifDataDirectoryPath{}
	, m_crystalAnalyzer{}
	, m_structureClassifier{}
{
}

SelectCrystals::SelectCrystals(const size_type threadRank, const CrystalSelectPlan& crystalSelectPlan)
	: m_threadRank{ threadRank }
	, m_cifDataDirectoryPath{ crystalSelectPlan.cifDataDirectoryPath() }
	, m_crystalAnalyzer{}
	, m_structureClassifier{ crystalSelectPlan.outputDirectoryPath(), false }
{
	m_crystalAnalyzer.setChemicalCompositionConstraints(crystalSelectPlan.crystalAnalysisOptions().chemicalCompositionConstraints());
	m_crystalAnalyzer.setCoordinationPolyhedraConstraints(crystalSelectPlan.crystalAnalysisOptions().coordinationPolyhedraConstraints());
	m_crystalAnalyzer.setWholeStructuralConstraints(crystalSelectPlan.crystalAnalysisOptions().wholeStructuralConstraints());
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void SelectCrystals::operator()() const
{
	OperatingSystem::IO::LogWriter logWriter;
	logWriter.setDirectoryPath(m_structureClassifier.outputDirectoryPath());

	ChemToolkit::Crystallography::FileIO::CifFileExcavator cifFileExcavator{ m_threadRank };
	cifFileExcavator.initialize(m_cifDataDirectoryPath);


	while (cifFileExcavator.cifFileReader())
	{
		std::string logMessage;
		{
			logMessage += "Thread ";
			logMessage += std::to_string(m_threadRank);
			logMessage += ": Excavate the cif file of ";
			logMessage += cifFileExcavator.cifFileReader().filePath().generic_string();
		}

		logWriter.write(logMessage);


		try
		{
			ModelCrystalStructure modelCrystalStructure;
			{
				CrystallographicInformation crystallographicInformation = cifFileExcavator.cifFileReader().readStructure();

				if (crystallographicInformation.hasPartiallyFilledAtomSite())
				{
					cifFileExcavator.excavateNextCifFile();
					continue;
				}

				else
				{
					crystallographicInformation.toCrystalStructure(modelCrystalStructure);

					modelCrystalStructure.removeSiteDuplications();
					modelCrystalStructure.assignCoordinationFeaturesFromElementSymbols();
					modelCrystalStructure.optimizeOxidationStates();
					modelCrystalStructure.assignCoordinationFeaturesFromElementSymbols();

					modelCrystalStructure.createTrackingAtomPairs();
					modelCrystalStructure.createGeometricalConstraints();
				}
			}
			
			if (m_crystalAnalyzer.isOptimalSolution(modelCrystalStructure))
			{
				/*
				std::filesystem::path copyCifFilePath = m_structureClassifier.outputDirectoryPath();
				copyCifFilePath /= cifFileExcavator.cifFileReader().filename();
				cifFileExcavator.cifFileReader().copyFile(copyCifFilePath, OperatingSystem::IO::FileCopyOptions::none);
				*/

				m_structureClassifier.classify(cifFileExcavator.cifFileReader().filePath());
			}

			cifFileExcavator.excavateNextCifFile();
		}

		catch (const std::exception& e)
		{
			logWriter.write(e);
			cifFileExcavator.excavateNextCifFile();
		}
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
