#include "SpglibWrapper.h"

#include <map>
#include <iostream>

#include "spglib.h"

#include "InternationalTables.h"
#include "PeriodicTable.h"

using namespace ChemToolkit::Crystallography::SpaceGroups;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

SpglibWrapper::SpglibWrapper()
	: m_spaceGroupPrecision{ InternationalTables::spaceGroupPrecision() }
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

SpglibWrapper::SpglibVariables SpglibWrapper::getSpglibVariablesWithSpaceGroupInfo(const UnitCell& unitCell, const std::vector<AtomBase>& atomBases) const
{
	SpglibVariables spglibVariables = getInputSpglibVariables(unitCell, atomBases);
	SpglibDataset* spglibDatasetPtr;
	{
		double precision = m_spaceGroupPrecision;

		for (size_type rep = 0; rep < 10; ++rep)
		{
			spglibDatasetPtr = spg_get_dataset(spglibVariables.lattice, &(spglibVariables.positions[0]), &(spglibVariables.types[0]), spglibVariables.numAtoms, precision);

			if (spglibDatasetPtr == nullptr)
				precision *= 0.9;
			else
				break;
		}

		if (spglibDatasetPtr == nullptr)
		{
			OperatingSystem::Configuration::ReportException reportException;
			throw std::logic_error(reportException(typeid(*this), "getSpglibVariablesWithSpaceGroupInfo", "Could not obtain \"spglibDataset\"."));
		}
	}

	spglibVariables.spaceGroupNumber = spglibDatasetPtr->spacegroup_number;

	spglibVariables.mappingToInputIndices.clear();
	{
		for (int index = 0; index < spglibVariables.numAtoms; ++index)
			spglibVariables.mappingToInputIndices.push_back(index);
	}

	spglibVariables.siteSymmetrySymbols.clear();
	spglibVariables.wyckoffSymbols.clear();
	spglibVariables.siteIndices.clear();
	{
		for (int index = 0; index < spglibVariables.numAtoms; ++index)
		{
			spglibVariables.siteSymmetrySymbols.push_back(spglibDatasetPtr->site_symmetry_symbols[index]);
			spglibVariables.wyckoffSymbols.push_back(toWyckoffSymbol(spglibDatasetPtr->wyckoffs[index]));
			spglibVariables.siteIndices.push_back(spglibDatasetPtr->equivalent_atoms[index]);
		}
	}

	spg_free_dataset(spglibDatasetPtr);
	return spglibVariables;
}

SpglibWrapper::SpglibVariables SpglibWrapper::getConventionalSpglibVariables(const UnitCell& unitCell, const std::vector<AtomBase>& atomBases) const
{
	SpglibVariables inputSpglibVariables = getInputSpglibVariables(unitCell, atomBases);
	SpglibDataset* spglibDatasetPtr;
	{
		double precision = m_spaceGroupPrecision;

		for (size_type rep = 0; rep < 10; ++rep)
		{
			spglibDatasetPtr = spg_get_dataset(inputSpglibVariables.lattice, &(inputSpglibVariables.positions[0]), &(inputSpglibVariables.types[0]), inputSpglibVariables.numAtoms, precision);

			if (spglibDatasetPtr == nullptr)
				precision *= 0.9;
			else
				break;
		}

		if (spglibDatasetPtr == nullptr)
		{
			OperatingSystem::Configuration::ReportException reportException;
			throw std::logic_error(reportException(typeid(*this), "getConventionalSpglibVariables", "Could not obtain \"spglibDataset\"."));
		}
	}

	SpglibVariables conventionalSpglibVariables;
	{
		conventionalSpglibVariables.numAtoms = spglibDatasetPtr->n_std_atoms;

		conventionalSpglibVariables.lattice[0][0] = spglibDatasetPtr->std_lattice[0][0];
		conventionalSpglibVariables.lattice[0][1] = spglibDatasetPtr->std_lattice[0][1];
		conventionalSpglibVariables.lattice[0][2] = spglibDatasetPtr->std_lattice[0][2];
		conventionalSpglibVariables.lattice[1][0] = spglibDatasetPtr->std_lattice[1][0];
		conventionalSpglibVariables.lattice[1][1] = spglibDatasetPtr->std_lattice[1][1];
		conventionalSpglibVariables.lattice[1][2] = spglibDatasetPtr->std_lattice[1][2];
		conventionalSpglibVariables.lattice[2][0] = spglibDatasetPtr->std_lattice[2][0];
		conventionalSpglibVariables.lattice[2][1] = spglibDatasetPtr->std_lattice[2][1];
		conventionalSpglibVariables.lattice[2][2] = spglibDatasetPtr->std_lattice[2][2];

		conventionalSpglibVariables.types = std::unique_ptr<int[]>{ new int[conventionalSpglibVariables.numAtoms] };
		{
			for (int index = 0; index < conventionalSpglibVariables.numAtoms; ++index)
				conventionalSpglibVariables.types[index] = spglibDatasetPtr->std_types[index];
		}

		conventionalSpglibVariables.positions = std::unique_ptr<double[][3]>{ new double[conventionalSpglibVariables.numAtoms][3] };
		{
			for (int index = 0; index < conventionalSpglibVariables.numAtoms; ++index)
			{
				conventionalSpglibVariables.positions[index][0] = spglibDatasetPtr->std_positions[index][0];
				conventionalSpglibVariables.positions[index][1] = spglibDatasetPtr->std_positions[index][1];
				conventionalSpglibVariables.positions[index][2] = spglibDatasetPtr->std_positions[index][2];
			}
		}

		conventionalSpglibVariables.spaceGroupNumber = spglibDatasetPtr->spacegroup_number;

		conventionalSpglibVariables.mappingToInputIndices.clear();
		{
			std::map<int, int> primitiveToInput;
			{
				for (int index = 0; index < inputSpglibVariables.numAtoms; ++index)
					primitiveToInput.emplace(spglibDatasetPtr->mapping_to_primitive[index], index);
			}

			for (int index = 0; index < conventionalSpglibVariables.numAtoms; ++index)
				conventionalSpglibVariables.mappingToInputIndices.push_back(primitiveToInput.at(spglibDatasetPtr->std_mapping_to_primitive[index]));
		}

		conventionalSpglibVariables.siteSymmetrySymbols.clear();
		conventionalSpglibVariables.wyckoffSymbols.clear();
		conventionalSpglibVariables.siteIndices.clear();
		{
			for (int index = 0; index < conventionalSpglibVariables.numAtoms; ++index)
			{
				conventionalSpglibVariables.siteSymmetrySymbols.push_back(spglibDatasetPtr->site_symmetry_symbols[conventionalSpglibVariables.mappingToInputIndices.at(index)]);
				conventionalSpglibVariables.wyckoffSymbols.push_back(toWyckoffSymbol(spglibDatasetPtr->wyckoffs[conventionalSpglibVariables.mappingToInputIndices.at(index)]));
				conventionalSpglibVariables.siteIndices.push_back(spglibDatasetPtr->equivalent_atoms[conventionalSpglibVariables.mappingToInputIndices.at(index)]);
			}
		}
	}

	spg_free_dataset(spglibDatasetPtr);
	return conventionalSpglibVariables;
}

void SpglibWrapper::findPrimitiveSpglibVariables(SpglibVariables& spglibVariables, const double spaceGroupPrecision) const
{
	std::vector<int> inputTypes;
	{
		for (int index = 0; index < spglibVariables.numAtoms; ++index)
			inputTypes.push_back(spglibVariables.types[index]);
	}

	int numPrimitiveAtoms = 0;
	{
		double precision = spaceGroupPrecision;

		for (size_type rep = 0; rep < 10; ++rep)
		{
			numPrimitiveAtoms = spg_find_primitive(spglibVariables.lattice, &(spglibVariables.positions[0]), &(spglibVariables.types[0]), spglibVariables.numAtoms, precision);

			if (numPrimitiveAtoms == 0)
				precision *= 0.9;
			else
				break;
		}
	}

	if (numPrimitiveAtoms == 0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::logic_error(reportException(typeid(*this), "findPrimitiveSpglibVariables", "Could not execute \"spg_find_primitive\" correctly."));
	}

	else
	{
		spglibVariables.numAtoms = numPrimitiveAtoms;
		spglibVariables.spaceGroupNumber = 1;
		spglibVariables.mappingToInputIndices.clear();
		{
			for (int index = 0; index < spglibVariables.numAtoms; ++index)
			{
				bool hasType = false;
				{
					for (int inputIndex = 0; inputIndex < static_cast<int>(inputTypes.size()); ++inputIndex)
					{
						if (spglibVariables.types[index] == inputTypes[inputIndex])
						{
							spglibVariables.mappingToInputIndices.push_back(inputIndex);
							hasType = true;
							break;
						}
					}
				}

				if (!hasType)
				{
					OperatingSystem::Configuration::ReportException reportException;
					throw std::logic_error(reportException(typeid(*this), "findPrimitiveSpglibVariables", "Could not assign mapping to input indices."));
				}
			}
		}

		spglibVariables.siteSymmetrySymbols.clear();
		spglibVariables.wyckoffSymbols.clear();
		spglibVariables.siteIndices.clear();
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private utility

std::string SpglibWrapper::toWyckoffSymbol(const size_type num) const
{
	switch (num)
	{
	case 0:
		return std::string{ "a" };
	case 1:
		return std::string{ "b" };
	case 2:
		return std::string{ "c" };
	case 3:
		return std::string{ "d" };
	case 4:
		return std::string{ "e" };
	case 5:
		return std::string{ "f" };
	case 6:
		return std::string{ "g" };
	case 7:
		return std::string{ "h" };
	case 8:
		return std::string{ "i" };
	case 9:
		return std::string{ "j" };
	case 10:
		return std::string{ "k" };
	case 11:
		return std::string{ "l" };
	case 12:
		return std::string{ "m" };
	case 13:
		return std::string{ "n" };
	case 14:
		return std::string{ "o" };
	case 15:
		return std::string{ "p" };
	case 16:
		return std::string{ "q" };
	case 17:
		return std::string{ "r" };
	case 18:
		return std::string{ "s" };
	case 19:
		return std::string{ "t" };
	case 20:
		return std::string{ "u" };

	default:
		OperatingSystem::Configuration::ReportException reportException;
		throw std::runtime_error(reportException(typeid(*this), "toWyckoffSymbol", "\"num\" is out of definitions."));
	}
}

std::string SpglibWrapper::toElementSymbol(const size_type atomicNumber) const
{
	return ChemToolkit::Generic::PeriodicTable::getElementSymbol(atomicNumber);
}

SpglibWrapper::SpglibVariables SpglibWrapper::getInputSpglibVariables(const UnitCell& unitCell, const std::vector<AtomBase>& atomicArrangement) const
{
	SpglibVariables spglibVariables;
	{
		spglibVariables.numAtoms = static_cast<int>(atomicArrangement.size());

		spglibVariables.lattice[0][0] = unitCell.basisVectors()(0, 0);
		spglibVariables.lattice[0][1] = unitCell.basisVectors()(0, 1);
		spglibVariables.lattice[0][2] = unitCell.basisVectors()(0, 2);
		spglibVariables.lattice[1][0] = unitCell.basisVectors()(1, 0);
		spglibVariables.lattice[1][1] = unitCell.basisVectors()(1, 1);
		spglibVariables.lattice[1][2] = unitCell.basisVectors()(1, 2);
		spglibVariables.lattice[2][0] = unitCell.basisVectors()(2, 0);
		spglibVariables.lattice[2][1] = unitCell.basisVectors()(2, 1);
		spglibVariables.lattice[2][2] = unitCell.basisVectors()(2, 2);

		spglibVariables.types = std::unique_ptr<int[]>{ new int[spglibVariables.numAtoms] };
		{
			for (int index = 0; index < spglibVariables.numAtoms; ++index)
				spglibVariables.types[index] = static_cast<int>(ChemToolkit::Generic::PeriodicTable::getAtomicNumber(atomicArrangement[index].elementSymbol()));
		}

		spglibVariables.positions = std::unique_ptr<double[][3]>{ new double[spglibVariables.numAtoms][3] };
		{
			for (int index = 0; index < spglibVariables.numAtoms; ++index)
			{
				MathToolkit::Generic::NumericalVector<double> fractionalCoordinate = unitCell.inverseBasisVectors() * atomicArrangement[index].coordinate();
				spglibVariables.positions[index][0] = fractionalCoordinate[0];
				spglibVariables.positions[index][1] = fractionalCoordinate[1];
				spglibVariables.positions[index][2] = fractionalCoordinate[2];
			}
		}

		spglibVariables.spaceGroupNumber = 1;
		spglibVariables.mappingToInputIndices.clear();
		spglibVariables.siteSymmetrySymbols.clear();
		spglibVariables.wyckoffSymbols.clear();
		spglibVariables.siteIndices.clear();
	}

	return spglibVariables;
}

// Private utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
