#include "StructuralRelaxer.h"

using namespace MathematicalCrystalChemistry::Design::Optimization;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

StructuralRelaxer::StructuralRelaxer()
	: m_structureRelaxationParameters{}
	, m_maxUnitCellDisplacement{ 0.0 }
	, m_maxAtomicDisplacement{ 0.0 }
	, m_displacementReductionRate{ 0.0 }
{
}

StructuralRelaxer::StructuralRelaxer(const StructuralRelaxationParameters& structureRelaxationParameters)
	: m_structureRelaxationParameters{ structureRelaxationParameters }
	, m_maxUnitCellDisplacement{ 0.0 }
	, m_maxAtomicDisplacement{ 0.0 }
	, m_displacementReductionRate{ 0.0 }
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

bool StructuralRelaxer::run(ModelCrystalStructure& modelCrystalStructure)
{
	preprocess(modelCrystalStructure);
	GeometricalBarrierHandler::initialize(modelCrystalStructure);
	StructureCustomizer::importStructure(modelCrystalStructure);


	for (size_type structureRelaxing = 0; structureRelaxing < m_structureRelaxationParameters.maxStructureOptimizing(); ++structureRelaxing)
		relaxStructure();

	StructureCustomizer::exportStructure(modelCrystalStructure);


	if (modelCrystalStructure.isOptimalSolution(m_structureRelaxationParameters.feasibleConstraintErrorRate()))
		return true;
	else
		return false;
}

bool StructuralRelaxer::run(ModelCrystalStructure& modelCrystalStructure, StructureDesignRecorder& structureDesignRecorder)
{
	preprocess(modelCrystalStructure);
	GeometricalBarrierHandler::initialize(modelCrystalStructure);
	StructureCustomizer::importStructure(modelCrystalStructure);

	m_mdRecordTimer = 0;
	m_chemicalBondRecordTimer = 0;
	m_mdRecordFrequency = structureDesignRecorder.dynamicsRecorder().recordFrequency();
	m_chemicalBondRecordFrequency = structureDesignRecorder.chemicalBondRecorder().recordFrequency();


	for (size_type structureRelaxing = 0; structureRelaxing < m_structureRelaxationParameters.maxStructureOptimizing(); ++structureRelaxing)
	{
		relaxStructure();
		recordStructure(modelCrystalStructure, structureDesignRecorder);
	}

	StructureCustomizer::exportStructure(modelCrystalStructure);
	structureDesignRecorder.addStructureOptimizing(m_structureRelaxationParameters.maxStructureOptimizing());

	m_mdRecordTimer = 0;
	m_chemicalBondRecordTimer = 0;


	if (modelCrystalStructure.isOptimalSolution(m_structureRelaxationParameters.feasibleConstraintErrorRate()))
		return true;
	else
		return false;
}

bool StructuralRelaxer::rerun(ModelCrystalStructure& modelCrystalStructure)
{
	preprocess(modelCrystalStructure);


	for (size_type structureRelaxing = 0; structureRelaxing < m_structureRelaxationParameters.maxStructureOptimizing(); ++structureRelaxing)
		relaxStructure();

	StructureCustomizer::exportStructure(modelCrystalStructure);


	if (modelCrystalStructure.isOptimalSolution(m_structureRelaxationParameters.feasibleConstraintErrorRate()))
		return true;
	else
		return false;
}

bool StructuralRelaxer::rerun(ModelCrystalStructure& modelCrystalStructure, StructureDesignRecorder& structureDesignRecorder)
{
	preprocess(modelCrystalStructure);
	
	m_mdRecordTimer = 0;
	m_chemicalBondRecordTimer = 0;
	m_mdRecordFrequency = structureDesignRecorder.dynamicsRecorder().recordFrequency();
	m_chemicalBondRecordFrequency = structureDesignRecorder.chemicalBondRecorder().recordFrequency();


	for (size_type structureRelaxing = 0; structureRelaxing < m_structureRelaxationParameters.maxStructureOptimizing(); ++structureRelaxing)
	{
		relaxStructure();
		recordStructure(modelCrystalStructure, structureDesignRecorder);
	}

	StructureCustomizer::exportStructure(modelCrystalStructure);
	structureDesignRecorder.addStructureOptimizing(m_structureRelaxationParameters.maxStructureOptimizing());

	m_mdRecordTimer = 0;
	m_chemicalBondRecordTimer = 0;


	if (modelCrystalStructure.isOptimalSolution(m_structureRelaxationParameters.feasibleConstraintErrorRate()))
		return true;
	else
		return false;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
