#include "StructureCreationPlan.h"

#include "FileReader.h"

using namespace MathematicalCrystalChemistry::Runtime::Creation;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

StructureCreationPlan::StructureCreationPlan()
	: m_chemicalParameters{}
	, m_geometricalConstraintParameters{}
	, m_structureCreationReportOptions{}
	, m_targetModelAtomCompositions{}
	, m_initialStructureGenerationOptions{}
	, m_structuralOptimizationParameters{}
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void StructureCreationPlan::initialize()
{
	m_chemicalParameters.initialize();
	m_geometricalConstraintParameters.initialize();

	m_structureCreationReportOptions.initialize();
	m_targetModelAtomCompositions.initialize();

	m_initialStructureGenerationOptions.initialize();
	m_structuralOptimizationParameters.initialize();
}

void StructureCreationPlan::initializeFromFiles(const std::vector<std::filesystem::path>& filePaths)
{
	m_initialStructureGenerationOptions.initializeFromFiles(filePaths);

	std::string inputTexts;
	{
		for (const auto& filePath : filePaths)
		{
			OperatingSystem::IO::FileReader fileReader{ filePath };
			inputTexts += fileReader.readFile();
		}
	}


	m_chemicalParameters.initializeFromInputFileTexts(inputTexts);
	m_geometricalConstraintParameters.initializeFromInputFileTexts(inputTexts);

	m_structureCreationReportOptions.initializeFromInputFileTexts(inputTexts);
	m_targetModelAtomCompositions.initializeFromInputFileTexts(inputTexts);

	m_structuralOptimizationParameters.initializeFromInputFileTexts(inputTexts);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
