#include "StructureCustomizer.h"

using namespace MathematicalCrystalChemistry::Design::Optimization::Internal;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

StructureCustomizer::StructureCustomizer()
	: StructureCustomizerBase{}
	, m_pressure{ StructureCustomizerBase::commonPressure() }
	, m_attractiveForceConstant{ StructureCustomizerBase::commonAttractiveForceConstant() }
	, m_repulsiveForceConstant{ StructureCustomizerBase::commonRepulsiveForceConstant() }
	, m_basisVectors(3, 3)
	, m_inverseBasisVectors(3, 3)
	, m_basisVectorsTransformation(3, 3)
	, m_atomCoordinates{}
	, m_atomAppliedForces{}
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Protected methods

void StructureCustomizer::transformBasisVectors(const double maxBasisVectorDisplacement)
{
	double normSquare = 0.0;
	{
		for (const auto& value : m_basisVectorsTransformation)
			normSquare += (value * value);
	}

	if ((maxBasisVectorDisplacement * maxBasisVectorDisplacement) < normSquare)
	{
		const double transformationScalingMultiplier = (maxBasisVectorDisplacement / std::sqrt(normSquare));
		m_basisVectorsTransformation *= transformationScalingMultiplier;
	}


	m_basisVectors += m_basisVectorsTransformation;
	m_inverseBasisVectors = m_basisVectors.getInverseMatrix();
	m_basisVectorsTransformation = NumericalMatrix(3, 3);
}

void StructureCustomizer::moveAtoms(const double maxAtomDisplacement)
{
	auto appliedForceIterEnd = m_atomAppliedForces.end();

	for (auto appliedForceIter = m_atomAppliedForces.begin(); appliedForceIter != appliedForceIterEnd;)
	{
		auto scalingIter = appliedForceIter;
		double normSquare = 0.0;
		{
			normSquare += (*appliedForceIter) * (*appliedForceIter);
			++appliedForceIter;
			normSquare += (*appliedForceIter) * (*appliedForceIter);
			++appliedForceIter;
			normSquare += (*appliedForceIter) * (*appliedForceIter);
			++appliedForceIter;
		}

		if ((maxAtomDisplacement * maxAtomDisplacement) < normSquare)
		{
			const double scalingMultiplier = (maxAtomDisplacement / std::sqrt(normSquare));

			(*scalingIter) *= scalingMultiplier;
			++scalingIter;
			(*scalingIter) *= scalingMultiplier;
			++scalingIter;
			(*scalingIter) *= scalingMultiplier;
		}
	}

	m_atomCoordinates += m_atomAppliedForces;
	m_atomAppliedForces = NumericalMatrix(3, m_atomCoordinates.rowSize());
}

// Protected methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
