#include "StructureFingerprint.h"

#include "FileWriter.h"

#include <algorithm>

using namespace MathematicalCrystalChemistry::StructureModel::Components;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

StructureFingerprint::StructureFingerprint()
	: m_atomCategoryComposition{}
	, m_spaceGroupNumber{ 0 }
	, m_atomicEnvironments{}
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

std::string StructureFingerprint::getSymbol() const
{
	std::string fingerprintString;
	{
		fingerprintString += m_atomCategoryComposition.name();
		fingerprintString += "__SpaceGroupNumber:_";
		fingerprintString += std::to_string(m_spaceGroupNumber);
		fingerprintString += "_____AtomicEnvironments";

		for (const auto& atomicEnvironment : m_atomicEnvironments)
		{
			fingerprintString += "_____";
			fingerprintString += atomicEnvironment.getFingerprint();
		}
	}

	return fingerprintString;
}

std::string StructureFingerprint::getPlainSymbol() const
{
	std::string fingerprintString;
	{
		fingerprintString += m_atomCategoryComposition.name();
		fingerprintString += "_____AtomicEnvironments";

		for (const auto& atomicEnvironment : m_atomicEnvironments)
		{
			fingerprintString += "_____";
			fingerprintString += atomicEnvironment.getPlainFingerprint();
		}
	}

	return fingerprintString;
}

void StructureFingerprint::report(const std::filesystem::path& filePath) const
{
	std::string outputText;
	{
		outputText += "Atom.Category.Composition     ";
		outputText += m_atomCategoryComposition.name();
		outputText += "\n";
		outputText += "Space.Group.Number     ";
		outputText += std::to_string(spaceGroupNumber());
		outputText += "\n\n";

		outputText += "<Atom.Sites\n\n";
		outputText += "_element_symbol\n";
		outputText += "_atom_category_symbol\n";
		outputText += "_site_index\n";
		outputText += "_site_symmetry\n";
		outputText += "_ionic_bond_coordination\n";
		outputText += "_metallic_bond_coordination\n\n";
		{
			for (const auto& atomicEnvironment : m_atomicEnvironments)
				outputText += atomicEnvironment.getReportText();
		}
		outputText += "\nAtom.Sites>\n";
	}


	OperatingSystem::IO::FileWriter fileWriter{ filePath };
	fileWriter.createFile(OperatingSystem::IO::FileCreateOptions::skip_existing, outputText);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void StructureFingerprint::setAtomicEnvironments(const std::vector<AtomicEnvironment>& aes)
{
	m_isHashValueValid = false;
	m_atomicEnvironments = aes;
	std::sort(m_atomicEnvironments.begin(), m_atomicEnvironments.end());
}

void StructureFingerprint::setAtomicEnvironments(std::vector<AtomicEnvironment>&& aes)
{
	m_isHashValueValid = false;
	m_atomicEnvironments = std::move(aes);
	std::sort(m_atomicEnvironments.begin(), m_atomicEnvironments.end());
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
