#include "TargetModelAtomCompositions.h"

#include <regex>
#include <sstream>

#include "FileReader.h"
#include "PeriodicTable.h"

using namespace MathematicalCrystalChemistry::Runtime::Creation;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

TargetModelAtomCompositions::TargetModelAtomCompositions()
	: m_targetChemicalCompositions{}
	, m_coordinationFeatureSymbolNames{}
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void TargetModelAtomCompositions::initialize()
{
	m_targetChemicalCompositions.clear();
	m_coordinationFeatureSymbolNames.clear();
}

void TargetModelAtomCompositions::initializeFromInputFileTexts(const std::string& inputTexts)
{
	initialize();

	initializeTargetChemicalCompositions(inputTexts);
	initializeCoordinationFeatureSymbols(inputTexts);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void TargetModelAtomCompositions::initializeTargetChemicalCompositions(const std::string& inputTexts)
{
	for (const auto& lineData : OperatingSystem::IO::FileReader::readDataEnumeration(inputTexts, "Chemical.Compositions.of.Model.Atoms"))
	{
		ChemicalComposition chemicalComposition;
		std::regex pat("([[:alpha:]]+)[\\_]{1}([\\d]+)");


		for (const auto& symbolAndNum : lineData)
		{
			std::smatch sm;

			if (std::regex_match(symbolAndNum, sm, pat))
			{
				if (ChemToolkit::Generic::PeriodicTable::isElementSymbol(sm.str(1)))
					chemicalComposition.add(sm.str(1), static_cast<size_type>(std::stoi(sm.str(2))));

				else
				{
					std::string errorMessage;
					{
						errorMessage += "\"";
						errorMessage += sm.str(1);
						errorMessage += "\" is not an element symbol.";
					}

					OperatingSystem::Configuration::ReportException reportException;
					throw std::logic_error(reportException(typeid(*this), "initializeTargetChemicalCompositions", errorMessage));
				}
			}
		}

		m_targetChemicalCompositions.push_back(std::move(chemicalComposition));
	}
}

void TargetModelAtomCompositions::initializeCoordinationFeatureSymbols(const std::string& inputTexts)
{
	for (const auto& lineData : OperatingSystem::IO::FileReader::readDataEnumeration(inputTexts, "Coordination.Features.of.Chemical.Elements"))
	{
		std::string elementSymbol = lineData.at(0);
		{
			if (!(ChemToolkit::Generic::PeriodicTable::isElementSymbol(elementSymbol)))
			{
				std::string errorMessage;
				{
					errorMessage += "\"";
					errorMessage += elementSymbol;
					errorMessage += "\" is not an element symbol.";
				}

				OperatingSystem::Configuration::ReportException reportException;
				throw std::logic_error(reportException(typeid(*this), "initializeCoordinationFeatureSymbols", errorMessage));
			}
		}

		std::vector<std::string> coordinationFeatureSymbols;
		{
			for (size_type index = 1; index < lineData.size(); ++index)
				coordinationFeatureSymbols.push_back(lineData[index]);
		}

		m_coordinationFeatureSymbolNames.emplace(elementSymbol, std::move(coordinationFeatureSymbols));
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
