#include "ThreadController.h"

#include <thread>
#include <mkl_service.h>

#include "ReportException.h"


using namespace OperatingSystem::Configuration;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static members

std::size_t ThreadController::m_maxThreading = std::thread::hardware_concurrency();
std::size_t ThreadController::m_defaultMaxThreading = std::thread::hardware_concurrency();
std::size_t ThreadController::m_defaultMaxMklThreading = MKL_Get_Max_Threads();


// Static members
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

std::size_t ThreadController::mklThreading()
{
	return static_cast<std::size_t>(MKL_Get_Max_Threads());
}

std::size_t ThreadController::mklDomainThreading(const std::string& domainName)
{
	return static_cast<std::size_t>(MKL_Domain_Get_Max_Threads(translateMklDomainName(domainName)));
}

std::size_t ThreadController::mklStriping()
{
	return static_cast<std::size_t>(MKL_Get_Num_Stripes());
}

bool ThreadController::isMklDynamic()
{
	if (0 == MKL_Get_Dynamic())
		return false;
	else
		return true;
}

void ThreadController::setMklThreading(std::size_t mklThreading)
{
	if (mklThreading == 0)
		MKL_Set_Num_Threads(static_cast<int>(m_maxThreading));
	else
		MKL_Set_Num_Threads(static_cast<int>(mklThreading));
}

void ThreadController::setLocalMklThreading(std::size_t localMklThreading)
{
	MKL_Set_Num_Threads_Local(static_cast<int>(localMklThreading));
}

void ThreadController::setMklDomainThreading(const std::string& domainName, std::size_t mklDomainThreading)
{
	if (MKL_Domain_Set_Num_Threads(static_cast<int>(mklDomainThreading), translateMklDomainName(domainName)) == 0)
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException("OperatingSystem::Configuration::ThreadController::resetMklDomainThreading", "The function of \"MKL_Domain_Set_Num_Threads\" indicates a failure."));
	}
}

void ThreadController::setMklStriping(std::size_t mklStriping)
{
	MKL_Set_Num_Stripes(static_cast<int>(mklStriping));
}

void ThreadController::setMklDynamic(const bool requestDynamic)
{
	if (requestDynamic)
		MKL_Set_Dynamic(1);
	else
		MKL_Set_Dynamic(0);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

int ThreadController::translateMklDomainName(const std::string& domainName)
{
	if (domainName == "ALL" || domainName == "All" || domainName == "all")
		return MKL_DOMAIN_ALL;

	else
	{
		if (domainName == "BLAS" || domainName == "CBLAS")
			return MKL_DOMAIN_BLAS;

		else
		{
			if (domainName == "FFT")
				return MKL_DOMAIN_FFT;

			else
			{
				if (domainName == "VML")
					return MKL_DOMAIN_VML;

				else
				{
					if (domainName == "LAPACK" || domainName == "LAPACKE")
						return MKL_DOMAIN_LAPACK;

					else
					{
						OperatingSystem::Configuration::ReportException reportException;
						throw std::invalid_argument(reportException("OperatingSystem::Configuration::ThreadController::translateMklDomainName", "\"domainName\" is nonsense."));
					}
				}
			}
		}
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
