#include "UnitCell.h"

#include "MathematicalConstants.h"

using namespace ChemToolkit::Crystallography;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

UnitCell::UnitCell()
	: m_basisVectors(3, 3)
	, m_inverseBasisVectors(3, 3)
	, m_appliedTransformation(3, 3)
{
	m_basisVectors(0, 0) = 1.0;
	m_basisVectors(1, 1) = 1.0;
	m_basisVectors(2, 2) = 1.0;

	m_inverseBasisVectors(0, 0) = 1.0;
	m_inverseBasisVectors(1, 1) = 1.0;
	m_inverseBasisVectors(2, 2) = 1.0;
}

UnitCell::UnitCell(const NumericalMatrix& basisVectors)
	: m_basisVectors{ basisVectors }
	, m_inverseBasisVectors{}
	, m_appliedTransformation(3, 3)
{
	if (m_basisVectors.columnSize() == 3 && m_basisVectors.rowSize() == 3)
		m_inverseBasisVectors = m_basisVectors.getInverseMatrix();

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "constructor", "The sizes of the column and/or row of \"basisVectors\" are not three."));
	}
}

UnitCell::UnitCell(NumericalMatrix&& basisVectors)
	: m_basisVectors{ std::move(basisVectors) }
	, m_inverseBasisVectors{}
	, m_appliedTransformation(3, 3)
{
	if (m_basisVectors.columnSize() == 3 && m_basisVectors.rowSize() == 3)
		m_inverseBasisVectors = m_basisVectors.getInverseMatrix();

	else
	{
		OperatingSystem::Configuration::ReportException reportException;
		throw std::invalid_argument(reportException(typeid(*this), "constructor", "The sizes of the column and/or row of \"basisVectors\" are not three."));
	}
}

UnitCell::UnitCell(const LatticeParameters& latticeParameters)
	: m_basisVectors(3, 3)
	, m_inverseBasisVectors(3, 3)
	, m_appliedTransformation(3, 3)
{
	{
		const double alphaRadian = latticeParameters.angles.alpha;
		const double betaRadian = latticeParameters.angles.beta;
		const double gammaRadian = latticeParameters.angles.gamma;
		m_basisVectors(0, 0) = latticeParameters.lengths.a;
		m_basisVectors(0, 1) = (latticeParameters.lengths.b * std::cos(gammaRadian));
		m_basisVectors(1, 1) = (latticeParameters.lengths.b * std::sin(gammaRadian));
		m_basisVectors(0, 2) = (latticeParameters.lengths.c * std::cos(betaRadian));
		m_basisVectors(1, 2) = ((latticeParameters.lengths.c * (std::cos(alphaRadian) - (std::cos(betaRadian) * std::cos(gammaRadian)))) / std::sin(gammaRadian));
		m_basisVectors(2, 2) = std::sqrt((latticeParameters.lengths.c * latticeParameters.lengths.c) - (m_basisVectors(0, 2) * m_basisVectors(0, 2)) - (m_basisVectors(1, 2) * m_basisVectors(1, 2)));
	}

	m_inverseBasisVectors = m_basisVectors.getInverseMatrix();
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

MathToolkit::Generic::NumericalMatrix<double> UnitCell::reciprocalVectors() const
{
	return ((2.0 * MathToolkit::Numerics::MathematicalConstants::getPi()) * m_inverseBasisVectors);
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
